MODULE mo_aero_diag

  ! *mo_aero_diag* holds routines for extended diagnostics
  !
  ! Authors:
  ! --------
  ! Philip Stier, MPI-Met, Hamburg,                          04/2004
  ! Silvia Kloster, MPI-Met, Hamburg,                        04/2004

  USE mo_aero_mem, ONLY: diag
  USE mo_kind,     ONLY: dp
  USE mo_aero,     ONLY: laero_rad

  IMPLICIT NONE

  REAL(dp), PUBLIC, POINTER :: velo10m_a5(:,:)
  REAL(dp), PUBLIC, POINTER :: velo10m_a8(:,:)
  REAL(dp), PUBLIC, POINTER :: velo10m_a10(:,:)

  REAL(dp), PUBLIC, POINTER :: rh_b30_70(:,:,:)
  REAL(dp), PUBLIC, POINTER :: rh_b70_85(:,:,:)
  REAL(dp), PUBLIC, POINTER :: rh_b85_95(:,:,:)
  REAL(dp), PUBLIC, POINTER :: rh_b95_100(:,:,:)

  REAL(dp), PUBLIC, POINTER :: rhd(:,:)
  REAL(dp), PUBLIC, POINTER :: presd(:,:)
  REAL(dp), PUBLIC, POINTER :: dpg(:,:,:)

  REAL(dp), PUBLIC, POINTER :: lwc(:,:,:)
  REAL(dp), PUBLIC, POINTER :: iwc(:,:,:)

  REAL(dp), PUBLIC, POINTER :: reff(:,:)
  REAL(dp), PUBLIC, POINTER :: reff_fine(:,:)
  REAL(dp), PUBLIC, POINTER :: reff_coarse(:,:)
  REAL(dp), PUBLIC, POINTER :: reff_dry(:,:)
  REAL(dp), PUBLIC, POINTER :: reff_fine_dry(:,:)
  REAL(dp), PUBLIC, POINTER :: reff_coarse_dry(:,:)


  CONTAINS 

    SUBROUTINE aero_diag(kproma, kbdim, &
                         klev,   krow,  &
                         pxtm1,  pxtte, &
                         pxlm1,  pxlte, &
                         pxim1,  pxite, &
                         papp1,  paphp1 )

      ! *aero_diag* extended diagnostics based on the AEROCOM protocol
      !
      ! Authors:
      ! --------
      ! Philip Stier, MPI-Met, Hamburg,                          04/2004
      ! Silvia Kloster, MPI-Met, Hamburg,                        04/2004
      !
      ! Method:
      ! -------


      USE mo_aero,         ONLY: nindex, lorad
      USE mo_aero_mem,     ONLY: velo10m, relhum, rwet, rdry
      USE mo_aero_rad_mem, ONLY: tau_mode, tau_2d
      USE mo_aero_m7,      ONLY: nmod, cmr2ras, cmr2ram
      USE mo_constants,    ONLY: g
      USE mo_tracer,       ONLY: ntrac
      USE mo_time_control, ONLY: delta_time, time_step_len

      IMPLICIT NONE

      INTEGER, INTENT(in) :: kproma, kbdim, klev, krow

      REAL(dp),INTENT(in) :: pxlm1(kbdim,klev),       pxlte(kbdim,klev), &
                             pxim1(kbdim,klev),       pxite(kbdim,klev)

      REAL(dp),INTENT(in) :: papp1(kbdim,klev),       paphp1(kbdim,klev+1)

      REAL(dp),INTENT(in) :: pxtm1(kbdim,klev,ntrac), pxtte(kbdim,klev,ntrac)

      !--- Local variables:

      INTEGER             :: jl, jk, jmod, imod

      REAL(dp)            :: zeps

      REAL(dp)            :: za(kbdim,nmod),     zv(kbdim,nmod), &
                             za_dry(kbdim,nmod), zv_dry(kbdim,nmod)

      REAL(dp)            :: zdpg(kbdim,klev)

      REAL(dp)            :: zxtp1(kbdim,klev,ntrac)

      REAL(dp), POINTER   :: tau_mode_p(:,:), rwet_p(:,:), rdry_p(:,:)

      LOGICAL,PARAMETER   :: lofine(nmod)=(/.TRUE.,.TRUE.,.TRUE.,.FALSE.,.TRUE.,.TRUE.,.FALSE./)


      !--- 0) Initialization:

      zeps=EPSILON(1.0_dp)

      za(:,:)=0._dp
      zv(:,:)=0._dp

      za_dry(:,:)=0._dp
      zv_dry(:,:)=0._dp

      !--- Mass of air in each layer [[kg m-2]:      

      DO jk=1, klev
         DO jl=1, kproma
            zdpg(jl,jk)=(paphp1(jl,jk+1)-paphp1(jl,jk))/g
         END DO
      END DO

      dpg(1:kproma,:,krow)=zdpg(1:kproma,:)

      !--- Current tracer concentrations:

      zxtp1(1:kproma,:,:)=pxtm1(1:kproma,:,:)+pxtte(1:kproma,:,:)*time_step_len


      !--- 1) Calculate effective radii:

      !--- Column integrated volume and surface area:

      DO jmod=1, nmod

         rwet_p=>rwet(jmod)%ptr(:,:,krow)
         rdry_p=>rdry(jmod)%ptr(:,:,krow)

         DO jk=1, klev
            imod=nindex(jmod)
            DO jl=1, kproma
               zv(jl,jmod)    =zv(jl,jmod)    +zxtp1(jl,jk,imod)*zdpg(jl,jk)*(rwet_p(jl,jk)*cmr2ram(jmod))**3
               za(jl,jmod)    =za(jl,jmod)    +zxtp1(jl,jk,imod)*zdpg(jl,jk)*(rwet_p(jl,jk)*cmr2ras(jmod))**2

               zv_dry(jl,jmod)=zv_dry(jl,jmod)+zxtp1(jl,jk,imod)*zdpg(jl,jk)*(rdry_p(jl,jk)*cmr2ram(jmod))**3
               za_dry(jl,jmod)=za_dry(jl,jmod)+zxtp1(jl,jk,imod)*zdpg(jl,jk)*(rdry_p(jl,jk)*cmr2ras(jmod))**2
            END DO
         END DO

      END DO

      !--- Effective radius - total:

      DO jmod=1, nmod
         DO jl=1, kproma
            IF(za(jl,jmod)>zeps .AND. zv(jl,jmod)>zeps) THEN

               reff(jl,krow)    =reff(jl,krow)    +(zv(jl,jmod)    /za(jl,jmod))
               reff_dry(jl,krow)=reff_dry(jl,krow)+(zv_dry(jl,jmod)/za_dry(jl,jmod))

               IF(lofine(jmod)) THEN
                  reff_fine(jl,krow)    =reff_fine(jl,krow)    +(zv(jl,jmod)/za(jl,jmod))
                  reff_fine_dry(jl,krow)=reff_fine_dry(jl,krow)+(zv_dry(jl,jmod)/za_dry(jl,jmod))
               ELSE
                  reff_coarse(jl,krow)    =reff_coarse(jl,krow)+(zv(jl,jmod)/za(jl,jmod))
                  reff_coarse_dry(jl,krow)=reff_coarse_dry(jl,krow)+(zv_dry(jl,jmod)/za_dry(jl,jmod))
               END IF

            END IF
         END DO
      END DO


      !--- 2) monthly mean time fraction of wind speed of 10m height
      
      !--- 2a) .gt. 5 m/s
      WHERE (velo10m(1:kproma,krow) > 5.0_dp )
         velo10m_a5(1:kproma,krow) = velo10m_a5(1:kproma,krow) + delta_time
      END WHERE
      !--- 2b) .gt. 8 m/s
      WHERE (velo10m(1:kproma,krow) > 8.0_dp )
         velo10m_a8(1:kproma,krow) = velo10m_a8(1:kproma,krow) + delta_time
      END WHERE
      !--- 2c) .gt. 10 m/s
      WHERE (velo10m(1:kproma,krow) > 10.0_dp )
         velo10m_a10(1:kproma,krow) = velo10m_a10(1:kproma,krow) + delta_time
      END WHERE
      
      !--- 3) monthly mean time fraction relative humidity
      
      !--- 3a) 30 - 70 %
      WHERE ( 0.30_dp < relhum(1:kproma,:,krow) .AND. relhum(1:kproma,:,krow) <= 0.70_dp ) 
         rh_b30_70(1:kproma,:,krow) = rh_b30_70(1:kproma,:,krow) + delta_time
      ENDWHERE
      !--- 3b) 70 - 85 %
      WHERE ( 0.70_dp < relhum(1:kproma,:,krow) .AND. relhum(1:kproma,:,krow) <= 0.85_dp ) 
         rh_b70_85(1:kproma,:,krow) = rh_b70_85(1:kproma,:,krow) + delta_time
      ENDWHERE
      !--- 3c) 85 - 95 %
      WHERE ( 0.85_dp < relhum(1:kproma,:,krow) .AND. relhum(1:kproma,:,krow) <= 0.95_dp ) 
         rh_b85_95(1:kproma,:,krow) = rh_b85_95(1:kproma,:,krow) + delta_time
      ENDWHERE
      !--- 3c) 95 - 100 %
      WHERE ( 0.95_dp < relhum(1:kproma,:,krow) .AND. relhum(1:kproma,:,krow) <= 1.00_dp ) 
         rh_b95_100(1:kproma,:,krow) = rh_b95_100(1:kproma,:,krow) + delta_time
      ENDWHERE

      !--- 4) Optical thickness-weighted relative humidity and pressure:

      IF (laero_rad) THEN 

         rhd(1:kproma,krow)  =0.0_dp
         presd(1:kproma,krow)=0.0_dp

         DO jmod=1, nmod

            IF (lorad(jmod)) THEN

               tau_mode_p=>tau_mode(jmod)%ptr(1:kproma,:,krow)

               DO jk=1, klev
                  rhd(1:kproma,krow)  =rhd(1:kproma,krow)  +relhum(1:kproma,jk,krow)*tau_mode_p(1:kproma,jk)
                  presd(1:kproma,krow)=presd(1:kproma,krow)+papp1(1:kproma,jk)      *tau_mode_p(1:kproma,jk)
               END DO

            END IF

         END DO

         WHERE(tau_2d(1:kproma,krow)>zeps)
            rhd(1:kproma,krow)  =rhd(1:kproma,krow)  /tau_2d(1:kproma,krow)
            presd(1:kproma,krow)=presd(1:kproma,krow)/tau_2d(1:kproma,krow)
         ENDWHERE

      END IF

      !--- 5) Cloud water content:

      lwc(1:kproma,:,krow)=(pxlm1(1:kproma,:)+pxlte(1:kproma,:)*time_step_len)*zdpg(1:kproma,:)

      iwc(1:kproma,:,krow)=(pxim1(1:kproma,:)+pxite(1:kproma,:)*time_step_len)*zdpg(1:kproma,:)


    END SUBROUTINE aero_diag

    SUBROUTINE aero_diag_mem

      ! *aero_diag_mem* allocates stream elements for diagnostics
      !
      ! Authors:
      ! --------
      ! Philip Stier, MPI-Met, Hamburg,                          04/2004
      ! Silvia Kloster, MPI-Met, Hamburg,                        04/2004
      !
      ! Method:
      ! -------
      ! To be called after the initialization of the diag stream

      USE mo_aero_mem,    ONLY: diag, aero
      USE mo_memory_base, ONLY: add_stream_element, default_stream_setting, AUTO
      USE mo_linked_list, ONLY: HYBRID, SURFACE


      IMPLICIT NONE


      !--- Add stream elements:

      CALL default_stream_setting (diag, lrerun    = .TRUE. , &
                                         laccu     = .TRUE. , &
                                         lpost     = .TRUE. , &
                                         table     = 199,     &
                                         code      = AUTO     )


      CALL default_stream_setting (diag, leveltype=SURFACE)

      CALL add_stream_element (diag, 'VELO10M_A5',             velo10m_a5,       units='%',             &
                               longname='time fraction with wind speed > 5 m/s', laccu=.TRUE.           )
     
      CALL add_stream_element (diag, 'VELO10M_A8',             velo10m_a8,       units='%',             &
                               longname='time fraction with wind speed > 8 m/s', laccu=.TRUE.           )

      CALL add_stream_element (diag, 'VELO10M_A10',            velo10m_a10,      units='%',             &
                               longname='time fraction with wind speed > 10 m/s',laccu=.TRUE.           )

      CALL default_stream_setting (diag, leveltype=HYBRID)

      CALL add_stream_element (diag, 'RH_B30_70',              rh_b30_70,        units='%',             &
                               longname='time fraction with RH betw. 30-70%',    laccu=.TRUE.           )

      CALL add_stream_element (diag, 'RH_B70_85',              rh_b70_85,        units='%',             &
                               longname='time fraction with RH betw. 70-85%',    laccu=.TRUE.           )

      CALL add_stream_element (diag, 'RH_B85_95',              rh_b85_95,        units='%',             &
                               longname='time fraction with RH betw. 85-95%',    laccu=.TRUE.           )

      CALL add_stream_element (diag, 'RH_B95_100',             rh_b95_100,       units='%',             &
                               longname='time fraction with RH betw. 95-100%',   laccu=.TRUE.           )

      !--- Non accumulated fields:

      CALL add_stream_element (aero, 'RHD',                    rhd,              units='%',             &
                               longname='optical thickness weighted RH',         laccu=.FALSE.          )      

      CALL add_stream_element (aero, 'PRESD',                  presd,            units='Pa',            &
                               longname='optical thickness weighted pressure',   laccu=.FALSE.          )

      CALL add_stream_element (aero, 'DPG',                    dpg,              units='kg m-2',        &
                               longname='mass of air in each layer',             laccu=.FALSE.          )

      CALL add_stream_element (aero, 'LWC',                    lwc,              units='kg m-2',        &
                               longname='cloud liquid water content',            laccu=.FALSE.          )

      CALL add_stream_element (aero, 'IWC',                    iwc,              units='kg m-2',        &
                               longname='cloud ice water content',               laccu=.FALSE.          )

      CALL default_stream_setting (diag, leveltype=SURFACE)

      CALL add_stream_element (aero, 'REFF',                   reff,             units='m',             &
                               longname='effective radius',                      laccu=.FALSE.          )

      CALL add_stream_element (aero, 'REFF_FINE',              reff_fine,        units='m',             &
                               longname='effective radius particles below 50 um',laccu=.FALSE.          )

      CALL add_stream_element (aero, 'REFF_COARSE',            reff_coarse,      units='m',             &
                               longname='effective radius particles above 50 um',laccu=.FALSE.          )

      CALL add_stream_element (aero, 'REFF_DRY',               reff_dry,         units='m',             &
                               longname='dry effective radius',                  laccu=.FALSE.          )

      CALL add_stream_element (aero, 'REFF_FINE_DRY',          reff_fine_dry,    units='m',             &
                               longname='dry effective radius particles below 50 um',laccu=.FALSE.      )

      CALL add_stream_element (aero, 'REFF_COARSE_DRY',        reff_coarse_dry,  units='m',             &
                               longname='dry effective radius particles above 50 um',laccu=.FALSE.      )

    END SUBROUTINE aero_diag_mem

END MODULE mo_aero_diag
