MODULE mo_aero_rad

  ! *mo_aero_rad* holds the parameters and routines for
  !               the calculation of the optical parameters
  !               for the aerosol distribution simulated
  !               by the ECHAM/HAM aerosol module. 
  !
  ! Authors:
  ! --------
  ! Olivier Boucher, Univ. Lille,                                  2003
  !    (original source of most radiation routines)
  ! Philip Stier, MPI-Met, Hamburg,                          08/05/2003
  !    (adaption to ECHAM5/HAM, refractive indices,...)
  !
  ! References:
  ! -----------

  USE mo_aero,        ONLY: lambda_opt => cwv, &
                            Nwv_opt    => nwv, &
                            lorad,             &
                            ntype,             &
                            lodiagrad
  USE mo_aero_m7,     ONLY: nmod
  USE mo_kind,        ONLY: dp


  IMPLICIT NONE

  PRIVATE

  PUBLIC aero_rad,                  & !
         aero_rad_num,              & ! 
         aero_rad_diag,             & !
         aero_rad_cache,            & !
         aero_rad_cache_cleanup,    & ! 
         aero_rad_mem,              & !
         aero_rad_mem_cleanup,      & !
         aero_rad_initialize,       & !
         nmod


  !--- Optical properties at GCM wavebands:

  REAL(dp), PUBLIC, ALLOCATABLE :: sigma(:,:,:,:)
  REAL(dp), PUBLIC, ALLOCATABLE :: omega(:,:,:,:)
  REAL(dp), PUBLIC, ALLOCATABLE :: asym(:,:,:,:)

  !--- Optical properties at steamer wavebands:

  REAL(dp), PUBLIC, POINTER :: sigma_sr(:,:,:,:)
  REAL(dp), PUBLIC, POINTER :: omega_sr(:,:,:,:)
  REAL(dp), PUBLIC, POINTER :: asym_sr(:,:,:,:)

  !--- Parameters:

  INTEGER, PUBLIC    :: nhamaer=0            !--Aerosol modes potentially 
                                             !  considered in radiation
  INTEGER, PARAMETER :: Ndismax=100
  INTEGER, PARAMETER :: Nnrmax=60
  INTEGER, PARAMETER :: Nnimax=200

  INTEGER, PARAMETER :: Nwvmax     = 24, &   !--SW STREAMER wavebands
                        Nwv_sw     =  4, &   !--SW GCM wavebands
                        Nwv_lw     = 16      !--LW GCM wavebands

  INTEGER            :: Nwv_tot,         &   !--SW+LW GCM + additional optional wavebands
                        Nwv_lwsw             !--SW+LW GCM wavebands (standard)

  INTEGER, PARAMETER :: i550=8,          &   !-- 550 nm streamer array index
                        i825=13              !-- 825 nm streamer array index

  INTEGER, PARAMETER :: it_so4 = 1, &
                        it_bc  = 2, &
                        it_oc  = 3, &
                        it_ss  = 4, &
                        it_du  = 5, &
                        it_wa  = 6

  LOGICAL :: lowv_opt                        !-- Consider optional wavelengths

  !--- BOA solar fluxes in Nwvmax spectral bands:

  REAL(dp) :: weight(1:Nwvmax)

  !--- Real and imaginary part of the refracive index for each 
  !    wavelength and aerosol type (including water):

  REAL(dp) :: cnr(Nwvmax,Ntype), cni(Nwvmax,Ntype)

  !--- Look-up tables:

  REAL(dp), DIMENSION(:,:,:), ALLOCATABLE :: lut1_sigma, lut1_g, lut1_omega, lut1_pp180
  REAL(dp), DIMENSION(:,:,:), ALLOCATABLE :: lut2_sigma, lut2_g, lut2_omega, lut2_pp180

  REAL(dp), DIMENSION(1:Nwv_sw)           :: weight_gcm
  REAL(dp), DIMENSION(1:Nwvmax)           :: lambda_int

  !@@@ Currently lut_pp180 are always allocated and read - need for switch?

CONTAINS

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_refractive(krow,             &
                                 pxtm1, pnr,  pni, &
                                 kmod,  pcnr, pcni )

    ! *aero_rad_refractive* calculates volume averaged refracive
    !                       indices for the ECHAM/HAM aerosol
    !                       distributions
    ! 
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg,  09/04/2003
    !
    ! Method:
    ! -------
    ! The real and imaginary parts of the refractive index 
    ! are obtained by taking a volume-weighted average over
    ! all compounds in the respecive mode.
    !
    ! Interface:
    ! ----------
    ! *aero_rad_refractive* is called by *aero_rad*

    USE mo_kind,         ONLY: dp
    USE mo_tracer,       ONLY: ntrac, flag, trlist, AEROSOLMASS
    USE mo_exception,    ONLY: finish
    USE mo_aero,         ONLY: csulfate2sulfur
    USE mo_aero_rad_mem, ONLY: nr_mode, ni_mode

    IMPLICIT NONE

    !--- Arguments:

    INTEGER,INTENT(in)  :: kmod, krow

    REAL(dp),   INTENT(in)  :: pxtm1(:,:,:)
    REAL(dp),   INTENT(in)  :: pcnr(:,:),  pcni(:,:)
    
    REAL(dp),   INTENT(out) :: pnr(:,:,:), pni(:,:,:)

    !--- Local variables:

    INTEGER :: jl, jk, jwv, jt, itype
    INTEGER :: Nx, Nz, Nwv
    REAL(dp)    :: zdensity, zeps, zv, zmassfac

    REAL(dp), ALLOCATABLE :: zvsum(:,:),             &
                         znrsum(:,:), znisum(:,:)

    REAL(dp), POINTER     :: nr_p(:,:,:), ni_p(:,:,:)


    !--- 0) Initialization:

    zeps=EPSILON(1.0_dp)

    Nx=SIZE(pnr,1)
    Nz=SIZE(pnr,2)
    Nwv=SIZE(pnr,3)

    IF(SIZE(pcnr,1) /= Nwv) &
       CALL finish('aero_rad_refractive','missmatch for refractive indices')

    ALLOCATE(zvsum(Nx,Nz))
    ALLOCATE(znrsum(Nx,Nz))
    ALLOCATE(znisum(Nx,Nz))


    !--- 1) Summation over all compounds:

    wv: DO jwv=1, Nwv

       zvsum(:,:) =0._dp 
       znrsum(:,:)=0._dp
       znisum(:,:)=0._dp

       tracer: DO jt=1, ntrac

          IF (trlist%ti(jt)%mode==kmod .AND. trlist%ti(jt)%nphase==AEROSOLMASS) THEN

             zdensity=flag('density',jt)

             zmassfac=1.0_dp

             SELECT CASE (trlist%ti(jt)%basename)
             CASE("SO4")
                itype=1
                zmassfac=csulfate2sulfur
             CASE("BC")
                itype=2
             CASE("OC")
                itype=3
             CASE("SS")
                itype=4
             CASE("DU")
                itype=5
             CASE("WAT")
                itype=6
             END SELECT

             DO jk=1, Nz
                DO jl=1, Nx
                   IF(pxtm1(jl,jk,jt)>zeps) THEN

                      zv=pxtm1(jl,jk,jt)*zmassfac/zdensity

                      znrsum(jl,jk)=znrsum(jl,jk)+pcnr(jwv,itype)*zv
                      znisum(jl,jk)=znisum(jl,jk)+pcni(jwv,itype)*zv

                      zvsum(jl,jk) =zvsum(jl,jk)+zv

                   END IF
                END DO
             END DO

          END IF

       END DO tracer

       !--- 4) Weighted averaging:

       DO jk=1, Nz
          DO jl=1, Nx
             IF(zvsum(jl,jk)>zeps) THEN

                pnr(jl,jk,jwv)=znrsum(jl,jk)/zvsum(jl,jk)
                pni(jl,jk,jwv)=znisum(jl,jk)/zvsum(jl,jk)

             ELSE

                pnr(jl,jk,jwv)=0._dp
                pni(jl,jk,jwv)=0._dp

             END IF
          END DO
       END DO

    END DO wv

    !--- Diagnostics:

    IF (lodiagrad) THEN

       nr_p => nr_mode(kmod)%ptr
       ni_p => ni_mode(kmod)%ptr

       nr_p(1:Nx,:,krow)=pnr(1:Nx,:,i550)
       ni_p(1:Nx,:,krow)=pni(1:Nx,:,i550)

    END IF

    !--- Cleanup:

    DEALLOCATE(zvsum, znrsum, znisum)

  END SUBROUTINE aero_rad_refractive

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad(kproma, kbdim, klev, krow, pxtm1)

    ! *aero_rad* calculates optical properties for
    !            aerosol distributions from look-up
    !            tables.
    !
    ! Author:
    ! -------
    ! Oliver Boucher, Univ. Lille,        2003
    !
    ! Modifications:
    ! --------------
    ! Philip Stier, MPI-MET, Hamburg,     2003
    !
    ! Method:
    ! -------
    ! To be done!
    !
    ! Interface:
    ! ----------
    ! *aero_rad* is called by *radiation*

    USE mo_aero,         ONLY: lorad
    USE mo_aero_m7,      ONLY: modesigma=>sigma
    USE mo_constants,    ONLY: api
    USE mo_kind,         ONLY: dp
    USE mo_exception,    ONLY: finish
    USE mo_tracer,       ONLY: ntrac
    USE mo_aero_mem,     ONLY: rwet

    IMPLICIT NONE

    !--- Arguments:

    INTEGER,INTENT(IN) :: kproma , kbdim, klev, krow

    REAL(dp),INTENT(IN)    :: pxtm1(kbdim,klev,ntrac)

    !--- Local Variables:

    INTEGER :: jmod, jl, jk, jwv, imode

    REAL(dp)    :: zeps

    REAL(dp), ALLOCATABLE :: zxx(:,:,:),    zsigma(:,:,:), &
                         zomega(:,:,:), zasym(:,:,:),  &
                         znr(:,:,:),    zni(:,:,:),    &
                         zpp180(:,:,:) 

    !--- Stream:
    REAL(dp), POINTER     :: rwet_p(:,:,:)

    !--- 0) Initialization:
    
    zeps=EPSILON(1.0_dp)

    ALLOCATE(zxx(kbdim,klev,Nwvmax))
    ALLOCATE(zsigma(kbdim,klev,Nwvmax))
    ALLOCATE(zomega(kbdim,klev,Nwvmax))
    ALLOCATE(zasym(kbdim,klev,Nwvmax))
    ALLOCATE(znr(kbdim,klev,Nwvmax))
    ALLOCATE(zni(kbdim,klev,Nwvmax))

    sigma(:,:,:,:)=0._dp
    omega(:,:,:,:)=0._dp
    asym(:,:,:,:) =0._dp

    !--- 1) Calculate optical properties for GCM bands:

    DO jmod=1, nmod
       IF (lorad(jmod)) THEN

          rwet_p => rwet(jmod)%ptr

          !--- 1.1) Calculate volume averaged refractive index znr and zni:

          CALL aero_rad_refractive(krow,                                   &
                                   pxtm1, znr,             zni,            &
                                   jmod,  cnr(1:Nwvmax,:), cni(1:Nwvmax,:) )


          !--- 1.1) Calculate size parameter:
          
          DO jwv=1, Nwvmax
             zxx(1:kproma,:,jwv)=2._dp*api*rwet_p(1:kproma,:,krow)/lambda_int(jwv)
          END DO

          !--- 1.2) Table-lookup for optical properties:

          IF (ABS(modesigma(jmod)-1.59_dp)<zeps) THEN

             imode=1

             CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                   imode,  lut1_sigma, zsigma, kproma )
             CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                   imode,  lut1_omega, zomega, kproma )
             CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                   imode,  lut1_g,     zasym,  kproma )

          ELSE IF (ABS(modesigma(jmod)-2.0_dp)<zeps) THEN

             imode=2

             CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                   imode,  lut2_sigma, zsigma, kproma )
             CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                   imode,  lut2_omega, zomega, kproma )
             CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                   imode,  lut2_g,     zasym,  kproma )

          ELSE

             CALL finish('aero_rad1','incompatible standard deviation')

          END IF

          DO jwv=1, Nwvmax
             zsigma(1:kproma,:,jwv)=zsigma(1:kproma,:,jwv)*lambda_int(jwv)*lambda_int(jwv)
          END DO


          !--- 1.3) Weighted mapping to GCM spectral bands:

          !--Nwv between  1 and 10 ==> 0.28-0.69 um

          DO jwv=1, 10
             DO jk=1, klev
                DO jl=1, kproma
                   sigma(jl,jk,1,jmod)=sigma(jl,jk,1,jmod) + zsigma(jl,jk,jwv)*weight(jwv)
                   omega(jl,jk,1,jmod)=omega(jl,jk,1,jmod) + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*weight(jwv)
                   asym(jl,jk,1,jmod) =asym(jl,jk,1,jmod)  + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*zasym(jl,jk,jwv)*weight(jwv)
                END DO
             END DO
          END DO

          !--Nwv between 11 and 16 ==> 0.69-1.19 um

          DO jwv=11, 16
             DO jk=1, klev
                DO jl=1, kproma
                   sigma(jl,jk,2,jmod)=sigma(jl,jk,2,jmod) + zsigma(jl,jk,jwv)*weight(jwv)
                   omega(jl,jk,2,jmod)=omega(jl,jk,2,jmod) + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*weight(jwv)
                   asym(jl,jk,2,jmod) =asym(jl,jk,2,jmod)  + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*zasym(jl,jk,jwv)*weight(jwv)
                END DO
             END DO
          END DO

          !--Nwv between 17 and 21 ==> 1.19-2.38 um

          DO jwv=17, 21
             DO jk=1, klev
                DO jl=1, kproma
                   sigma(jl,jk,3,jmod)=sigma(jl,jk,3,jmod) + zsigma(jl,jk,jwv)*weight(jwv)
                   omega(jl,jk,3,jmod)=omega(jl,jk,3,jmod) + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*weight(jwv)
                   asym(jl,jk,3,jmod) =asym(jl,jk,3,jmod)  + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*zasym(jl,jk,jwv)*weight(jwv)
                END DO
             END DO
          END DO

          !--Nwv between 22 and 24 ==> 2.38-4.00 um

          DO jwv=22, 24
             DO jk=1, klev
                DO jl=1, kproma
                   sigma(jl,jk,4,jmod)=sigma(jl,jk,4,jmod) + zsigma(jl,jk,jwv)*weight(jwv)
                   omega(jl,jk,4,jmod)=omega(jl,jk,4,jmod) + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*weight(jwv)
                   asym(jl,jk,4,jmod) =asym(jl,jk,4,jmod)  + zsigma(jl,jk,jwv)*zomega(jl,jk,jwv)*zasym(jl,jk,jwv)*weight(jwv)
                END DO
             END DO
          END DO

          DO jwv=1, Nwv_sw
             DO jk=1, klev
                DO jl=1, kproma
                   IF(omega(jl,jk,jwv,jmod) > zeps)THEN
                      asym(jl,jk,jwv,jmod)=asym(jl,jk,jwv,jmod)/omega(jl,jk,jwv,jmod) 
                      omega(jl,jk,jwv,jmod)=omega(jl,jk,jwv,jmod)/sigma(jl,jk,jwv,jmod) 
                      sigma(jl,jk,jwv,jmod)=sigma(jl,jk,jwv,jmod)/weight_gcm(jwv)
                   END IF
                END DO
             END DO
          END DO !jwv

          sigma_sr(1:kproma,:,:,jmod)=zsigma(1:kproma,:,:)
          omega_sr(1:kproma,:,:,jmod)=zomega(1:kproma,:,:)
          asym_sr(1:kproma,:,:,jmod) =zasym(1:kproma,:,:)

       END IF !lorad
    END DO !nmod

    DEALLOCATE(zxx, zsigma, zomega, zasym, znr, zni)


    !--- 2) Calculate optical properties for optional wavelengths:

    IF (lowv_opt) THEN

       ALLOCATE(zxx(kbdim,klev,Nwv_opt))
       ALLOCATE(zsigma(kbdim,klev,Nwv_opt))
       ALLOCATE(zomega(kbdim,klev,Nwv_opt))
       ALLOCATE(zasym(kbdim,klev,Nwv_opt))
       ALLOCATE(znr(kbdim,klev,Nwv_opt))
       ALLOCATE(zni(kbdim,klev,Nwv_opt))

       DO jmod=1, nmod
          IF (lorad(jmod)) THEN

             rwet_p => rwet(jmod)%ptr

             !--- 2.1) Calculate volume averaged refractive index znr and zni:
             
             CALL aero_rad_refractive(krow,                                                                 &
                                      pxtm1, znr,                            zni,                           &
                                      jmod,  cnr(Nwvmax+1:Nwvmax+Nwv_opt,:), cni(Nwvmax+1:Nwvmax+Nwv_opt,:) )


             DO jwv=1, Nwv_opt

                zxx(1:kproma,:,jwv)=2._dp*api*rwet_p(1:kproma,:,krow)/lambda_opt(jwv)

                !--- 2.2) Table-lookup for optical properties:

                IF (ABS(modesigma(jmod)-1.59_dp)<zeps) THEN

                   imode=1

                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut1_sigma, zsigma, kproma )
                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut1_omega, zomega, kproma )
                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut1_g,     zasym,  kproma )
                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut1_pp180, zpp180, kproma )

                ELSE IF (ABS(modesigma(jmod)-2.0_dp)<zeps) THEN

                   imode=2

                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut2_sigma, zsigma, kproma )
                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut2_omega, zomega, kproma )
                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut2_g,     zasym,  kproma )
                   CALL aero_rad_fitplus(zxx,    znr,        zni,   &
                                         imode,  lut2_pp180, zpp180, kproma )

                ELSE

                   CALL finish('aero_rad2','incompatible standard deviation')

                END IF

                !--- Extinction cross-section:

                sigma(1:kproma,:,Nwvmax+jwv,jmod)=zsigma(1:kproma,:,jwv)*lambda_opt(jwv)*lambda_opt(jwv)

                !@@@ To be done:
                !@@@
                !@@@         !--- Lidar ratio:
                !@@@          
                !@@@         IF (lolidar_ratio(jwv)) THEN
                !@@@            lidar_ratio(1:kproma,:,jwv)=4.*pi/zpp180(1:kproma,:,jwv)/omega(1:kproma,:,jwv)
                !@@@         ENDIF

             END DO !jwv

          END IF !lorad
       END DO !nmod

       DEALLOCATE(zxx, zsigma, zomega, zasym, znr, zni)

    END IF !Nwv_opt>0

  END SUBROUTINE aero_rad

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_fitplus(pxx,  pnr,  pni, &
                              kmode, plut, pfit, kproma )

    ! *aero_rad_fitplus* returns linear interpolated fit of the 
    !                    look-up tables for the aerosol optical 
    !                    properties.
    !
    ! Authors:
    ! --------
    ! Olivier Boucher, Univ. Lille,                                  2003
    !    (original source)
    ! Philip Stier, MPI-Met, Hamburg,                          08/05/2003
    !    (adaption to ECHAM5/HAM, reduced memory expense)
    ! Luis Kornblueh, MPI-Met, Hamburg                         2006-07-07
    !    (bugfix for uninitialized variables by adding kproma)
    ! 
    ! Interface:
    ! ----------
    !

    USE mo_kind,      ONLY: dp
    USE mo_doctor,    ONLY: nout

    IMPLICIT NONE

    !--- Arguments:

    REAL(dp), INTENT(out)  :: pfit(:,:,:)

    REAL(dp), INTENT(in)   :: pnr(:,:,:),  pni(:,:,:), pxx(:,:,:)

    REAL(dp), INTENT(in)   :: plut(0:Nnrmax, 0:Nnimax, 0:Ndismax)

    INTEGER,  INTENT(in)   :: kproma   

    !--- Local variables:

    LOGICAL, PARAMETER :: lodiag=.FALSE. !-- Diagnostic for values out of range
    LOGICAL, PARAMETER :: loint =.FALSE. !-- Linear interpolation in look-up table

    INTEGER            :: jl,jk,jwv,kmode

    REAL(dp)               :: zeps

    REAL(dp), PARAMETER    :: nr_min=1.33_dp,  nr_max=1.63_dp
    REAL(dp), PARAMETER    :: ni_min=1.E-9_dp, ni_max=0.50_dp

    REAL(dp), PARAMETER    :: x0_min(2)=(/ 0.001_dp, 0.4_dp/), & !--min Mie parameter
                          x0_max(2)=(/25.0_dp,  40.0_dp/)    !--max Mie parameter


    INTEGER :: Ndis, Nnr, Nni, Nwv, Nx, Nz
    INTEGER :: countpixel1, countpixel2, countpixel3
    REAL(dp)    :: xx1, xx2, nr1, nr2, ni1, ni2
    REAL(dp)    :: fitndisnr, fitndisp1nr, fitndisnrp1, fitndisp1nrp1, fitndis, fitndisp1

    REAL(dp) :: inc_nr, inc_ni

    !--- 0) 

    zeps=EPSILON(1.0_dp)

    !--- Real part:

    inc_nr=(nr_max-nr_min)/REAL(Nnrmax,dp)

    !--- Imaginary part now in log-space:

    inc_ni=(LOG(ni_max)-LOG(ni_min))/REAL(Nnimax,dp)

    !--- Security check:

    IF (lodiag) THEN

       countpixel1=COUNT(pxx(:kproma,:,:).LT.x0_min(kmode))+COUNT(pxx(:kproma,:,:).GT.x0_max(kmode))
       countpixel2=COUNT(pnr(:,:,:).LT.nr_min)       +COUNT(pnr(:,:,:).GT.nr_max)
       countpixel3=COUNT(pni(:,:,:).LT.ni_min)       +COUNT(pni(:,:,:).GT.ni_max)

       IF (countpixel1.GT.0.OR.countpixel2.GT.0.OR.countpixel3.GT.0) THEN 
          WRITE(nout,*) ' xx: ', countpixel1, ' nr: ', countpixel2, ' ni: ', countpixel3
          WRITE(nout,*)'aero_rad_fitplus: WARNING: Mie parameter or refractive index out of bound'
       ENDIF

    END IF

    !--- Set size of the arrays:

    Nx=kproma
    Nz=SIZE(pxx,2)
    Nwv=SIZE(pxx,3)

    !--- 1) Quick and dirty linear interpolation: 

    IF(loint) THEN

       DO jwv=1, Nwv
          DO jk=1, Nz
             DO jl=1, Nx
                IF ((pxx(jl,jk,jwv)>=x0_min(kmode)) .AND. (pxx(jl,jk,jwv)<=x0_max(kmode)) .AND. &
                    (pnr(jl,jk,jwv)>=nr_min)        .AND. (pnr(jl,jk,jwv)<=nr_max)        .AND. &
                    (pni(jl,jk,jwv)>=ni_min)        .AND. (pni(jl,jk,jwv)<=ni_max)              ) THEN

                   Ndis=INT( (LOG(pxx(jl,jk,jwv))-LOG(x0_min(kmode))) / &
                             (LOG(x0_max(kmode)) -LOG(x0_min(kmode)))*REAL(Ndismax,dp) )
                   Ndis=MIN(Ndismax-1,MAX(0,Ndis))

                   xx1=EXP(LOG(x0_min(kmode))+(LOG(x0_max(kmode))-LOG(x0_min(kmode)))*REAL(Ndis,dp)/REAL(Ndismax,dp))
                   xx2=EXP(LOG(x0_min(kmode))+(LOG(x0_max(kmode))-LOG(x0_min(kmode)))*REAL(Ndis+1,dp)/REAL(Ndismax,dp))

                   Nnr=INT((pnr(jl,jk,jwv)-nr_min)/inc_nr)
                   Nnr=MIN(Nnrmax-1,MAX(0,Nnr))

                   nr1=nr_min+REAL(Nnr,dp)*inc_nr
                   nr2=nr_min+REAL(Nnr+1,dp)*inc_nr

                   Nni=INT((LOG(pni(jl,jk,jwv))-LOG(ni_min))/inc_ni)
                   Nni=MIN(Nnimax-1,MAX(0,Nni))

                   ni1=EXP(LOG(ni_min)+REAL(Nni,dp)*inc_ni)
                   ni2=EXP(LOG(ni_min)+REAL(Nni+1,dp)*inc_ni)

                   fitndisnr      =plut(Nnr,Nni,Ndis)+(pni(jl,jk,jwv)-ni1)/(ni2-ni1)*  & 
                                   (plut(Nnr,Nni+1,Ndis)-plut(Nnr,Nni,Ndis))
                   fitndisnrp1    =plut(Nnr+1,Nni,Ndis)+(pni(jl,jk,jwv)-ni1)/(ni2-ni1)*  &
                                   (plut(Nnr+1,Nni+1,Ndis)-plut(Nnr+1,Nni,Ndis))
                   fitndisp1nr    =plut(Nnr,Nni,Ndis+1)+(pni(jl,jk,jwv)-ni1)/(ni2-ni1)*  &
                                   (plut(Nnr,Nni+1,Ndis+1)-plut(Nnr,Nni,Ndis+1))
                   fitndisp1nrp1  =plut(Nnr+1,Nni,Ndis+1)+(pni(jl,jk,jwv)-ni1)/(ni2-ni1)* & 
                                   (plut(Nnr+1,Nni+1,Ndis+1)-plut(Nnr+1,Nni,Ndis+1))
                   fitndis        =fitndisnr+(pnr(jl,jk,jwv)-nr1)/(nr2-nr1)*(fitndisnrp1-fitndisnr)
                   fitndisp1      =fitndisp1nr+(pnr(jl,jk,jwv)-nr1)/(nr2-nr1)*(fitndisp1nrp1-fitndisp1nr)

                   pfit(jl,jk,jwv)=fitndis+(pxx(jl,jk,jwv)-xx1)/(xx2-xx1)*(fitndisp1-fitndis)

                ELSE
                
                   pfit(jl,jk,jwv)=0._dp

                END IF
             END DO
          END DO
       END DO
       
    ELSE ! .NOT. loint

       !--- 2) Quicker and dirtier table look-up:

       DO jwv=1, Nwv
          DO jk=1, Nz
             DO jl=1, Nx
                IF ((pxx(jl,jk,jwv)>=x0_min(kmode)) .AND. (pxx(jl,jk,jwv)<=x0_max(kmode)) .AND. &
                    (pnr(jl,jk,jwv)>=nr_min)        .AND. (pnr(jl,jk,jwv)<=nr_max)        .AND. &
                    (pni(jl,jk,jwv)>=ni_min)        .AND. (pni(jl,jk,jwv)<=ni_max)              ) THEN

                   Ndis=NINT( (LOG(pxx(jl,jk,jwv))-LOG(x0_min(kmode))) / &
                              (LOG(x0_max(kmode)) -LOG(x0_min(kmode)))*REAL(Ndismax,dp) )
                   Ndis=MIN(Ndismax-1,MAX(0,Ndis))

                   Nnr=NINT((pnr(jl,jk,jwv)-nr_min)/inc_nr)
                   Nnr=MIN(Nnrmax-1,MAX(0,Nnr))

                   Nni=NINT((LOG(pni(jl,jk,jwv))-LOG(ni_min))/inc_ni)
                   Nni=MIN(Nnimax-1,MAX(0,Nni))

                   pfit(jl,jk,jwv)=plut(Nnr,Nni,Ndis)

                ELSE
                
                   pfit(jl,jk,jwv)=0._dp

                END IF
             END DO
          END DO
       END DO

    END IF

  END SUBROUTINE aero_rad_fitplus

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_initialize

    ! *aero_rad_initialize* initializes the HAM aerosol radiation module
    !                       mo_aero_rad. It sets module variables and 
    !                       reads in the look-up tables for the optical
    !                       properties.
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg          05/2003
    !
    ! Interface:
    ! ----------
    ! *aero_rad_initialize* is called from *call_init_submodels_2
    !
    ! References:
    ! -----------
    ! M. Hess, P. Koepke, and I. Schult (1998): Optical Properties
    !    of Aerosols and clouds: The software package OPAC, 
    !    Bull. Am. Met. Soc., 79, 831-844.

    USE mo_mpi,           ONLY: p_io, p_parallel_io, p_bcast
    USE mo_doctor,        ONLY: nout
    USE mo_exception,     ONLY: finish
    USE mo_sw,            ONLY: nsw
    USE mo_parrrtm,       ONLY: jpband
    USE mo_read_netcdf77, ONLY: read_var_nf77_3d

    IMPLICIT NONE

    INTEGER :: jwv, ierr

    LOGICAL :: lex

    CHARACTER(len=256) :: cfile

    !--- 1) Initialize Data:

    !--- 1.1) Wavelength bounds to wavebands:

    REAL(dp)    :: zlambda(0:Nwvmax)

    DATA zlambda(0:Nwvmax) &
             /0.28E-6_dp,    0.30E-6_dp,    0.33E-6_dp,    0.36E-6_dp,    0.40E-6_dp,    0.44E-6_dp,  &
              0.48E-6_dp,    0.52E-6_dp,    0.57E-6_dp,    0.64E-6_dp,    0.69E-6_dp,    0.75E-6_dp,  &
              0.78E-6_dp,    0.87E-6_dp,    1.00E-6_dp,    1.10E-6_dp,    1.19E-6_dp,    1.28E-6_dp,  &
              1.53E-6_dp,    1.64E-6_dp,    2.13E-6_dp,    2.38E-6_dp,    2.91E-6_dp,    3.42E-6_dp,  &
              4.00E-6_dp                                                               /

    !--- 1.2) BOA solar fluxes in Nwvmax spectral bands

    REAL(dp) :: zweight(1:Nwvmax) 

    DATA zweight(1:Nwvmax) &
             /0.839920E1_dp, 0.231208E2_dp, 0.322393E2_dp, 0.465058E2_dp, 0.678199E2_dp, 0.798964E2_dp, &
              0.771359E2_dp, 0.888472E2_dp, 0.115281E3_dp, 0.727565E2_dp, 0.816992E2_dp, 0.336172E2_dp, &
              0.914603E2_dp, 0.112706E3_dp, 0.658840E2_dp, 0.524470E2_dp, 0.391067E2_dp, 0.883864E2_dp, &
              0.276672E2_dp, 0.681812E2_dp, 0.190966E2_dp, 0.250766E2_dp, 0.128704E2_dp, 0.698720E1_dp  /


    REAL(dp)    :: zcnr(Nwvmax,Ntype), &
               zcni(Nwvmax,Ntype)

    !--- 1.3) Refractive indices:

    !--- Sulfate (OPAC: M. Hess, P. Koepke, and I. Schult, 1998):

    DATA zcnr(1:Nwvmax,it_so4) &
             /1.47_dp,       1.46_dp,       1.45_dp,       1.44_dp,       1.44_dp,       1.43_dp,       &
              1.43_dp,       1.43_dp,       1.43_dp,       1.43_dp,       1.43_dp,       1.43_dp,       &
              1.43_dp,       1.42_dp,       1.42_dp,       1.42_dp,       1.41_dp,       1.40_dp,       &
              1.40_dp,       1.39_dp,       1.36_dp,       1.33_dp,       1.31_dp,       1.39_dp        /

    DATA zcni(1:Nwvmax,it_so4) &
             /1.00E-08_dp,   1.00E-08_dp,   1.00E-08_dp,   1.00E-08_dp,   1.00E-08_dp,   1.00E-08_dp,   &
              1.00E-08_dp,   1.00E-08_dp,   1.40E-08_dp,   1.70E-08_dp,   4.10E-08_dp,   7.50E-08_dp,   &
              1.28E-07_dp,   7.01E-07_dp,   2.26E-06_dp,   4.76E-06_dp,   6.61E-06_dp,   7.70E-05_dp,   &
              1.45E-04_dp,   5.30E-04_dp,   1.90E-03_dp,   3.04E-02_dp,   1.28E-01_dp,   1.35E-01_dp    /

    !--- Black Carbon (OPAC: M. Hess, P. Koepke, and I. Schult, 1998):

    DATA zcnr(1:Nwvmax,it_bc) &
             /1.74_dp,       1.75_dp,       1.75_dp,       1.75_dp,       1.75_dp,       1.75_dp,       &
              1.75_dp,       1.75_dp,       1.75_dp,       1.75_dp,       1.75_dp,       1.75_dp,       &
              1.75_dp,       1.75_dp,       1.76_dp,       1.76_dp,       1.76_dp,       1.77_dp,       &
              1.77_dp,       1.80_dp,       1.81_dp,       1.82_dp,       1.86_dp,       1.90_dp        /

    DATA zcni(1:Nwvmax,it_bc) &
             /0.47_dp,       0.47_dp,       0.47_dp,       0.46_dp,       0.46_dp,       0.46_dp,       &
              0.45_dp,       0.44_dp,       0.44_dp,       0.44_dp,       0.43_dp,       0.43_dp,       &
              0.43_dp,       0.44_dp,       0.44_dp,       0.45_dp,       0.45_dp,       0.46_dp,       &
              0.48_dp,       0.49_dp,       0.51_dp,       0.51_dp,       0.54_dp,       0.57_dp        /

    !--- Organic Carbon (provided by Stefan Kinne, MPI-Met, best guess):

    DATA zcnr(1:Nwvmax,it_oc) &
             /1.53_dp,       1.53_dp,       1.53_dp,       1.53_dp,       1.53_dp,       1.53_dp,       &
              1.53_dp,       1.53_dp,       1.53_dp,       1.53_dp,       1.53_dp,       1.53_dp,       &
              1.53_dp,       1.53_dp,       1.53_dp,       1.52_dp,       1.52_dp,       1.51_dp,       &
              1.50_dp,       1.47_dp,       1.45_dp,       1.48_dp,       1.48_dp,       1.48_dp        /

    DATA zcni(1:Nwvmax,it_oc) &
             /0.50E-02_dp,   0.50E-02_dp,   0.50E-02_dp,   0.50E-02_dp,   0.50E-02_dp,   0.50E-02_dp,   &
              0.55E-02_dp,   0.55E-02_dp,   0.60E-02_dp,   0.70E-02_dp,   0.75E-02_dp,   0.80E-02_dp,   &
              0.90E-02_dp,   0.10E-01_dp,   0.15E-01_dp,   0.16E-01_dp,   0.17E-01_dp,   0.18E-01_dp,   &
              0.20E-01_dp,   0.10E-01_dp,   0.10E-01_dp,   0.80E-02_dp,   0.70E-02_dp,   0.70E-02_dp    /

    !--- Sea Salt (Shettle & Fenn, 1979):

    DATA zcnr(1:Nwvmax,it_ss) &
             /1.51_dp,       1.51_dp,       1.50_dp,       1.50_dp,       1.50_dp,       1.50_dp,       &
              1.50_dp,       1.49_dp,       1.49_dp,       1.49_dp,       1.48_dp,       1.48_dp,       &
              1.48_dp,       1.48_dp,       1.47_dp,       1.47_dp,       1.47_dp,       1.46_dp,       &
              1.46_dp,       1.45_dp,       1.44_dp,       1.41_dp,       1.52_dp,       1.48_dp        /

    DATA zcni(1:Nwvmax,it_ss) &
             /2.51E-06_dp,   1.00E-06_dp,   3.16E-07_dp,   6.31E-08_dp,   2.00E-08_dp,   2.00E-08_dp,   &
              1.00E-08_dp,   1.00E-08_dp,   1.00E-08_dp,   3.98E-08_dp,   1.58E-07_dp,   3.98E-07_dp,   &
              1.26E-06_dp,   2.00E-05_dp,   1.00E-04_dp,   2.51E-04_dp,   3.16E-04_dp,   5.01E-04_dp,   &
              6.31E-04_dp,   1.00E-03_dp,   2.00E-03_dp,   7.94E-03_dp,   3.16E-03_dp,   1.26E-03_dp    /

    !--- Dust (provided by Martin Werner, MPI-BGS, Jena, 
    !          mainly based on Sokolik & Toon, JGR, 1999)
    !          Stefan Kinne, MPI-MET:
    !          imaginary parts in the visible modified according to AERONET statistics
    !          much less absorption in the VIS (mainly based on Kinne et.al, JGR, 2003):

    DATA zcnr(1:Nwvmax,it_du) &
             /1.519_dp,      1.528_dp,      1.527_dp,      1.516_dp,      1.512_dp,      1.515_dp,      &
              1.516_dp,      1.517_dp,      1.517_dp,      1.517_dp,      1.517_dp,      1.518_dp,      &
              1.518_dp,      1.519_dp,      1.519_dp,      1.519_dp,      1.519_dp,      1.518_dp,      &
              1.518_dp,      1.518_dp,      1.518_dp,      1.518_dp,      1.518_dp,      1.518_dp       /

    DATA zcni(1:Nwvmax,it_du) &
             /0.0207_dp,     0.0185_dp,     0.0165_dp,     0.0025_dp,     0.0015_dp,     0.0013_dp,     &
              0.0012_dp,     0.0011_dp,     0.0010_dp,     0.0010_dp,     0.0009_dp,     0.0009_dp,     &
              0.0008_dp,     0.0008_dp,     0.0007_dp,     0.0006_dp,     0.0006_dp,     0.0008_dp,     &
              0.0008_dp,     0.0011_dp,     0.0020_dp,     0.0054_dp,     0.0151_dp,     0.0389_dp      /

    !--- Water (provided by Stefan Kinne, MPI-Met, Downing & Williams, 1975):

    DATA zcnr(1:Nwvmax,it_wa) &
             /1.34_dp,       1.34_dp,       1.34_dp,       1.33_dp,       1.33_dp,       1.33_dp,       &
              1.33_dp,       1.33_dp,       1.33_dp,       1.33_dp,       1.33_dp,       1.33_dp,       &
              1.33_dp,       1.33_dp,       1.33_dp,       1.32_dp,       1.32_dp,       1.31_dp,       &
              1.31_dp,       1.30_dp,       1.29_dp,       1.35_dp,       1.46_dp,       1.46_dp        /

    DATA zcni(1:Nwvmax,it_wa) &
             /0.65E-08_dp,   0.40E-08_dp,   0.35E-08_dp,   0.30E-08_dp,   0.20E-08_dp,   0.10E-08_dp,   &
              0.20E-08_dp,   0.20E-08_dp,   0.16E-07_dp,   0.16E-07_dp,   0.50E-07_dp,   0.10E-06_dp,   &
              0.50E-06_dp,   0.10E-05_dp,   0.29E-05_dp,   0.60E-05_dp,   0.90E-05_dp,   0.20E-04_dp,   &
              0.80E-04_dp,   0.10E-03_dp,   0.20E-03_dp,   0.34E-03_dp,   0.20E-00_dp,   0.20E-00_dp    /

    !--- Store in module variabes:

    weight(:) = zweight(:)
    cnr(:,:)  = zcnr(:,:)
    cni(:,:)  = zcni(:,:)

    IF (p_parallel_io) THEN
       WRITE(nout,*) ''
       WRITE(nout,*) '----------------------------------------------------------'
       WRITE(nout,*) '----------------------------------------------------------'
       WRITE(nout,*) '--- Initialization of the aerosol radiation scheme     ---'
       WRITE(nout,*) '---'
    END IF

    !--- Set up aerosol modes potentially considered in radiation:

    nhamaer=nmod

    !--- 1) Set total number of wavelengths (GCM + optional):
    !@@@ Nwv_tot  = Nwv_sw+Nwv_lw+Nwv_opt
    !@@@ Nwv_lw should be replaced by the number of internal LW wavebands
    Nwv_tot  = Nwvmax+Nwv_lw+Nwv_opt
    Nwv_lwsw = Nwv_sw+Nwv_lw

    !--- Switch for otional wavelengths:

    lowv_opt=Nwv_opt>0

    !--- 2) Consistency checks:

    IF ( Nwv_sw /= nsw .OR. Nwv_lw /= jpband) THEN

       CALL finish('aero_rad_initialize','inconsistent number of wavelengths')

    END IF

    IF ( SIZE(cnr,1)<Nwvmax .OR. SIZE(cni,1)<Nwvmax) THEN

       CALL finish('aero_rad_initialize', &
                   'insufficient number of refractive indices defined')

    ELSE IF ( SIZE(cnr,1)/=Nwvmax .OR. SIZE(cni,1)/=Nwvmax ) THEN

       IF (p_parallel_io) THEN
       WRITE(nout,*) '---    WARNING: refractive indices for ', Nwv_opt, ' requested'
       WRITE(nout,*) '---             optional wavelength(s) not defined!'
       WRITE(nout,*) '---             Continue with standard wavebands!'
       WRITE(nout,*) '---'
       END IF

       lowv_opt=.FALSE.

    END IF

    !--- Allocate memory for look-up tables:

    CALL aero_rad_mem

    !--- 2) Read in look-up tables:

    IF (p_parallel_io) THEN

       cfile='lut_optical_properties.nc'

       !--- First:

       WRITE(nout,*) '---    Reading look-up table: ', cfile

       INQUIRE (file=cfile,exist=lex)

       IF (lex) THEN

          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "sigma_1", lut1_sigma, ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "sigma_2", lut2_sigma, ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "omega_1", lut1_omega, ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "omega_2", lut2_omega, ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "asym_1",  lut1_g,     ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "asym_2",  lut2_g,     ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "pp180_1", lut1_pp180, ierr                 )
          CALL read_var_nf77_3d (cfile,     "nr",       "ni",     "dis",     &
                                 "pp180_2", lut2_pp180, ierr                 )

          WRITE(nout,*) '---             Done!'
          WRITE(nout,*) '---'

       ELSE

          CALL finish('aero_rad_initialize','look-up table not available')

       END IF

    END IF

    CALL p_bcast(lut1_sigma,p_io)
    CALL p_bcast(lut1_g    ,p_io)
    CALL p_bcast(lut1_omega,p_io)
    CALL p_bcast(lut1_pp180,p_io)
    CALL p_bcast(lut2_sigma,p_io)
    CALL p_bcast(lut2_g    ,p_io)
    CALL p_bcast(lut2_omega,p_io)
    CALL p_bcast(lut2_pp180,p_io)

    !--- 3) Calculate weighting factors:

    weight_gcm(1)=SUM(weight(1:10))    !   1 to 10 ==> 0.28-0.69 um
    weight_gcm(2)=SUM(weight(11:16))   !  11 to 16 ==> 0.69-1.19 um
    weight_gcm(3)=SUM(weight(17:21))   !  17 to 21 ==> 1.19-2.38 um
    weight_gcm(4)=SUM(weight(22:24))   !  22 to 24 ==> 2.38-4.00 um

    !--- 4) Interpolate wavelengths:

    DO jwv=1, Nwvmax
       lambda_int(jwv)=( zlambda(jwv-1)+zlambda(jwv) ) /2._dp
    ENDDO

    IF (p_parallel_io) THEN
    WRITE(nout,*) '---'
    WRITE(nout,*) '--- initialization finished                            ---'
    WRITE(nout,*) '----------------------------------------------------------'
    WRITE(nout,*) '----------------------------------------------------------'
    END IF

  END SUBROUTINE aero_rad_initialize

!----------------------------------------------------------------------------------------------------------------

  FUNCTION aero_rad_num(kbdim,  kproma, klev, &
                        klevp1, ktrac,        &
                        pxtm1,  paphm1        )

    ! *aero_rad_num* returns the aerosol numbers for each mode
    !                and each model layer of thicknes dp in 
    !                the units [# cm-2]
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg         2003
    !
    ! Interface:
    ! ----------
    ! *aero_rad_num* is called from *radiation*

    USE mo_aero,       ONLY: lorad, nindex
    USE mo_constants,  ONLY: g

    IMPLICIT NONE

    INTEGER :: kbdim, kproma, klev, klevp1, ktrac

    REAL(dp)    :: paphm1(kbdim,klevp1)

    REAL(dp)    :: pxtm1(kbdim,klev,ktrac)

    REAL(dp)    :: aero_rad_num(kproma,klev,nmod)

    INTEGER :: itrac, jl, jk, jmod

    REAL(dp)    :: zdpg(kproma,klev)

    !--- Auxiliary variable [kg(air) m-2]:

    DO jk=1,klev
       DO  jl=1,kproma
          zdpg(jl,jk)=(paphm1(jl,jk+1)-paphm1(jl,jk))/g
       END DO
    END DO

    !--- Calculate number of particles in layer of thickness dp in [1 m-2]:
    !    Store in array from 1:nmod.

    DO jmod=1, nmod
       IF (lorad(jmod)) THEN
          itrac=nindex(jmod)
          DO jk=1, klev
             DO jl=1, kproma
                aero_rad_num(jl,jk,jmod)=pxtm1(jl,jk,itrac)*zdpg(jl,jk)
             END DO
          END DO
       ELSE
          DO jk=1, klev
             DO jl=1, kproma
                aero_rad_num(jl,jk,jmod)=0._dp
             END DO
          END DO
       END IF
    END DO

  END FUNCTION aero_rad_num

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_diag(kproma, kbdim, klev, klevp1, krow, &
                           pxtm1,  paphm1                     )

    ! *aero_rad_diag* calculated type specific diagnostics
    !                 of aerosol optical properties
    !
    ! Authors:
    ! --------
    ! Philip Stier, MPI-Met, Hamburg,       05/2003
    
    USE mo_kind,         ONLY: dp
    USE mo_tracer,       ONLY: ntrac, flag, trlist
    USE mo_aero,         ONLY: ctype, csulfate2sulfur
    USE mo_aero_rad_mem, ONLY: tau_mode, abs_mode, omega_mode, sigma_mode, asym_mode,    &
                               tau_comp, abs_comp, tau_2d,     abs_2d,                   &
                               tau_comp_825,       tau_2d_825, tau_mode_825,             &
                               ang

    IMPLICIT NONE

    INTEGER   :: kproma, kbdim, klev, klevp1, krow

    REAL(dp)      :: paphm1(kbdim,klevp1)

    REAL(dp)      :: pxtm1(kbdim,klev,ntrac)

    INTEGER   :: jl, jk, jmod, jtype, jt

    INTEGER   :: idt

    REAL(dp)      :: zv, zdensity, zeps, zmassfac

    REAL(dp)      :: ztaucomp(kbdim), ztaucomp_825(kbdim), zabscomp(kbdim)

    REAL(dp)      :: zvsum(kbdim,klev,nmod), znivsum(kbdim,klev,nmod), &
                 znum(kbdim,klev,nmod)

    REAL(dp)      :: zvcomp(kbdim,klev,Ntype,nmod)

    REAL(dp), POINTER :: tau_comp_p(:,:), tau_comp_825_p(:,:), abs_comp_p(:,:)

    REAL(dp), POINTER :: omega_p(:,:,:), sigma_p(:,:,:), &
                     asym_p(:,:,:),  abs_p(:,:,:),   &
                     tau_p(:,:,:),   tau_825_p(:,:,:)


    !--- 0)
    
    zeps=EPSILON(1.0_dp)


    !--- 1) Store in 3D streams for each mode:

    IF (lodiagrad) THEN

       DO jmod=1, nmod
          IF(lorad(jmod))THEN

             omega_p => omega_mode(jmod)%ptr
             sigma_p => sigma_mode(jmod)%ptr
             asym_p  => asym_mode(jmod)%ptr

             omega_p(1:kproma,:,krow)=omega_sr(1:kproma,:,i550,jmod)
             sigma_p(1:kproma,:,krow)=sigma_sr(1:kproma,:,i550,jmod)
             asym_p(1:kproma,:,krow) =asym_sr(1:kproma,:,i550,jmod)

          END IF
       END DO

    END IF

    !--- Optical thickness for streamer wavebands:

    !--- Get aerosol number density for each layer in [1 cm-2]:

    znum(1:kproma,:,:)=aero_rad_num(kbdim,  kproma, klev, &
                                    klevp1, ntrac,        &
                                    pxtm1,  paphm1        )

    !--- Optical thickness:

    DO jmod=1, nmod
       IF(lorad(jmod))THEN

          tau_p     => tau_mode(jmod)%ptr
          tau_825_p => tau_mode_825(jmod)%ptr

          tau_p(1:kproma,:,krow)    =znum(1:kproma,:,jmod)*sigma_sr(1:kproma,:,i550,jmod)
          tau_825_p(1:kproma,:,krow)=znum(1:kproma,:,jmod)*sigma_sr(1:kproma,:,i825,jmod)

       END IF
    END DO

    !--- 2) Vertical integral summed over all modes:

    tau_2d(:,krow)     = 0._dp
    tau_2d_825(:,krow) = 0._dp

    DO jmod=1, nmod
       IF(lorad(jmod))THEN

          !--- 550 nm:

          tau_p => tau_mode(jmod)%ptr

          DO jk=1, klev
             DO jl=1, kproma
                tau_2d(jl,krow)=tau_2d(jl,krow)+tau_p(jl,jk,krow)
             END DO
          END DO

          !--- 825 nm:

          tau_825_p => tau_mode_825(jmod)%ptr

          DO jk=1, klev
             DO jl=1, kproma
                tau_2d_825(jl,krow)=tau_2d_825(jl,krow)+tau_825_p(jl,jk,krow)
             END DO
          END DO

       END IF
    END DO

    !--- 3) Split up according to compounds 
    !       (based on volume average for optcial thickness,
    !        additionally weighted with ni for absorption ):

    zvcomp(:,:,:,:)  = 0._dp
    zvsum(:,:,:)     = 0._dp
    znivsum(:,:,:)   = 0._dp

    DO jtype=1, ntype
       
       IF(TRIM(ctype(jtype))=='SO4') THEN
          zmassfac=csulfate2sulfur
       ELSE
          zmassfac=1.0_dp
       END IF

       DO jmod=1, nmod
          IF (lorad(jmod)) THEN

             !--- Get tracer index of compound jtype:

             idt=0
             DO jt=1, ntrac
                IF (trlist%ti(jt)%mode==jmod .AND. &
                    TRIM(trlist%ti(jt)%basename)==TRIM(ctype(jtype))) THEN
                   idt=jt
                   EXIT
                END IF
             END DO

             !--- Sum volume of compound weighted by volume:

             IF(idt>0) THEN

                zdensity=flag('density',idt)

                DO jk=1, klev
                   DO jl=1, kproma
                      IF(pxtm1(jl,jk,idt)>zeps) THEN

                         zv=pxtm1(jl,jk,idt)*zmassfac/zdensity

                         zvcomp(jl,jk,jtype,jmod)=zvcomp(jl,jk,jtype,jmod) + zv
                         zvsum(jl,jk,jmod)       =zvsum(jl,jk,jmod)        + zv
                         znivsum(jl,jk,jmod)     =znivsum(jl,jk,jmod)      + zv * cni(i550,jtype)

                      END IF
                   END DO
                END DO
             END IF

          END IF !lorad(jmod)

       END DO !jmod

    END DO !jtype

    DO jtype=1, ntype

       ztaucomp(:)    =0._dp
       ztaucomp_825(:)=0._dp
       zabscomp(:)    =0._dp

       !--- Weighted averaging and vertical integration:

       DO jmod=1, nmod
          IF (lorad(jmod)) THEN

             tau_p     => tau_mode(jmod)%ptr
             tau_825_p => tau_mode_825(jmod)%ptr
             abs_p     => abs_mode(jmod)%ptr

             DO jk=1, klev
                DO jl=1, kproma
                   IF (zvsum(jl,jk,jmod)>zeps) THEN
                      ztaucomp(jl)    =ztaucomp(jl) + &
                                       tau_p(jl,jk,krow)*zvcomp(jl,jk,jtype,jmod)/zvsum(jl,jk,jmod)
                      ztaucomp_825(jl)=ztaucomp_825(jl) + &
                                       tau_825_p(jl,jk,krow)*zvcomp(jl,jk,jtype,jmod)/zvsum(jl,jk,jmod)
                      zabscomp(jl)    =zabscomp(jl) + &
                                       abs_p(jl,jk,krow)*zvcomp(jl,jk,jtype,jmod)*cni(i550,jtype) / &
                                                         znivsum(jl,jk,jmod)
                   END IF
                END DO
             END DO

          END IF
       END DO

       !--- Store in output streams:

       tau_comp_p     => tau_comp(jtype)%ptr
       tau_comp_p(1:kproma,krow)=ztaucomp(1:kproma)

       tau_comp_825_p => tau_comp_825(jtype)%ptr
       tau_comp_825_p(1:kproma,krow)=ztaucomp_825(1:kproma)

       abs_comp_p     => abs_comp(jtype)%ptr
       abs_comp_p(1:kproma,krow)=zabscomp(1:kproma)

    END DO !jtype


    !--- 4) Calculate Angstroem parameter between 550 and 825 nm:

    WHERE(tau_2d(1:kproma,krow)>zeps) 
       ang(1:kproma,krow)=LOG(tau_2d_825(1:kproma,krow)/tau_2d(1:kproma,krow)) / LOG(550._dp/825._dp)
    ENDWHERE


    !--- 5) Calculate absorption at 550 nm:

    abs_2d(1:kproma,krow)=.0_dp

    DO jmod=1, nmod
       IF(lorad(jmod))THEN

          abs_p   => abs_mode(jmod)%ptr
          tau_p   => tau_mode(jmod)%ptr

          !--- For each mode:

          abs_p(1:kproma,:,krow)  =(1.0_dp-omega_sr(1:kproma,:,i550,jmod))*tau_p(1:kproma,:,krow)

          !--- Total vertical integral:

          DO jk=1, klev
             abs_2d(1:kproma,krow)=abs_2d(1:kproma,krow)+abs_p(1:kproma,jk,krow)
          END DO

       END IF
    END DO

  END SUBROUTINE aero_rad_diag

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_cache(kbdim,klev)

    ! *aero_rad_mem* allocates local (blocked) memory for the 
    !                HAM aerosol optical properties
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg          05/2003
    !
    ! Interface:
    ! ----------
    ! *aero_rad_cache* is called from *radiation*

    IMPLICIT NONE

    INTEGER, INTENT(in) :: kbdim,klev

    ALLOCATE(sigma(kbdim,klev,Nwv_tot,nmod))
    ALLOCATE(omega(kbdim,klev,Nwv_tot,nmod))
    ALLOCATE(asym (kbdim,klev,Nwv_tot,nmod))

    ALLOCATE(sigma_sr(kbdim,klev,Nwvmax,nmod))
    ALLOCATE(omega_sr(kbdim,klev,Nwvmax,nmod))
    ALLOCATE(asym_sr(kbdim,klev,Nwvmax,nmod))

  END SUBROUTINE aero_rad_cache

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_cache_cleanup

    ! *aero_rad_mem_cleanup* de-allocates local (blocked) memory for the 
    !                        HAM aerosol optical properties
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg          05/2003
    !
    ! Interface:
    ! ----------
    ! *aero_rad_cache_cleanup* is called from *radiation*
    
    IMPLICIT NONE

    DEALLOCATE(sigma,omega,asym)

    DEALLOCATE(sigma_sr,omega_sr,asym_sr)

  END SUBROUTINE aero_rad_cache_cleanup

!----------------------------------------------------------------------------------------------------------------
  SUBROUTINE aero_rad_mem

    ! *aero_rad_mem* allocates memory for the 
    !                HAM aerosol optical properties
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg          05/2003
    !
    ! Interface:
    ! ----------
    ! *aero_rad_mem* is called from *call_init_submodel_memory*
    ! in *call_submodels.f90*

    IMPLICIT NONE

    ALLOCATE(lut1_sigma(0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut1_g    (0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut1_omega(0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut1_pp180(0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut2_sigma(0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut2_g    (0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut2_omega(0:Nnrmax, 0:Nnimax, 0:Ndismax), &
             lut2_pp180(0:Nnrmax, 0:Nnimax, 0:Ndismax)  )

  END SUBROUTINE aero_rad_mem

!----------------------------------------------------------------------------------------------------------------

  SUBROUTINE aero_rad_mem_cleanup

    ! *aero_rad_cleanup* de-allocates memory of the 
    !                    module mo_aero_rad
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg          05/2003
    !
    ! Interface:
    ! ----------
    ! *aero_rad_cleanup* is called from *call_free_submodels_memory*
    ! in *call_submodels.f90*

    IMPLICIT NONE

    DEALLOCATE(lut1_sigma, lut1_g, lut1_omega, lut1_pp180, &
               lut2_sigma, lut2_g, lut2_omega, lut2_pp180  )

  END SUBROUTINE aero_rad_mem_cleanup

!----------------------------------------------------------------------------------------------------------------

END MODULE mo_aero_rad
