MODULE mo_aero_rad_mem

  ! *mo_aero_rad_mem* hold the rad stream for the 
  !                   HAM aerosol optical properties
  !
  ! Author:
  ! -------
  ! Philip Stier, MPI-Met, Hamburg          05/2003

  USE mo_linked_list, ONLY: t_stream
  USE mo_kind,        ONLY: dp
  USE mo_aero,        ONLY: nmod, ntype, lodiagrad
  USE mo_aero_mem,    ONLY: vmem2d, vmem3d


  IMPLICIT NONE


  PRIVATE

  PUBLIC construct_stream_aero_rad 

  TYPE (t_stream), PUBLIC, POINTER :: rad

! TYPE(io_time_event)       :: rad_out=io_time_event(2,'hours','first',0)

  TYPE (vmem3d), PUBLIC :: tau_mode(nmod)
  TYPE (vmem3d), PUBLIC :: tau_mode_825(nmod)
  TYPE (vmem3d), PUBLIC :: abs_mode(nmod)
  TYPE (vmem3d), PUBLIC :: sigma_mode(nmod)
  TYPE (vmem3d), PUBLIC :: omega_mode(nmod)
  TYPE (vmem3d), PUBLIC :: asym_mode(nmod)
  TYPE (vmem3d), PUBLIC :: nr_mode(nmod)
  TYPE (vmem3d), PUBLIC :: ni_mode(nmod)

  TYPE (vmem2d), PUBLIC :: tau_comp(ntype)
  TYPE (vmem2d), PUBLIC :: tau_comp_825(ntype)
  TYPE (vmem2d), PUBLIC :: abs_comp(ntype)

  REAL(dp), PUBLIC, POINTER :: tau_2d(:,:)
  REAL(dp), PUBLIC, POINTER :: tau_2d_825(:,:)
  REAL(dp), PUBLIC, POINTER :: abs_2d(:,:)
  REAL(dp), PUBLIC, POINTER :: ang(:,:)

CONTAINS

  SUBROUTINE construct_stream_aero_rad

    ! *construct_stream_aero_rad* constructs diagnostics stream rad
    !                             and defines stream elemenst for
    !                             HAM aerosol optical properties
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met, Hamburg          05/2003
    !
    ! Interface:
    ! ----------
    ! *construct_stream_aero_rad* is called from *call_init_submodel_memory*
    !                             in *call_submodels.f90*

    USE mo_memory_base,   ONLY: new_stream, add_stream_element, &
                                default_stream_setting, add_stream_reference, AUTO
    USE mo_linked_list,   ONLY: SURFACE
    USE mo_aero,          ONLY: NFILETYPE, cmode, ctype, ntype, lorad
    USE mo_aero_m7,       ONLY: nmod


    IMPLICIT NONE

    INTEGER :: jmod, jtype


    !--- Create new stream:

    CALL new_stream (rad ,'rad',filetype=NFILETYPE)

    !--- 1) Add standard fields for post-processing:

    CALL add_stream_reference (rad, 'geosp'   ,'g3b'   ,lpost=.TRUE.)
    CALL add_stream_reference (rad, 'lsp'     ,'sp'    ,lpost=.TRUE.)
    CALL add_stream_reference (rad, 'aps'     ,'g3b'   ,lpost=.TRUE.)    
    CALL add_stream_reference (rad, 'gboxarea','geoloc',lpost=.TRUE.)

    !--- 2) Add stream elements:

    CALL default_stream_setting (rad, lrerun   = .TRUE.,   &
                                      laccu    = .FALSE.,  &
                                      lpost    = .TRUE.,   &
                                      table    = 200,      &
                                      code     = AUTO      )

    !--- 2.1) Diagnostics for each mode: 

    DO jmod=1, nmod

       IF(lorad(jmod)) THEN

          CALL add_stream_element(rad,      'TAU_MODE_'//cmode(jmod),       tau_mode(jmod)%ptr,     units='1',          &
                                  longname='Optical thickness (550 nm)'                                                 )

          CALL add_stream_element(rad,      'TAU_MODE_825_'//cmode(jmod),   tau_mode_825(jmod)%ptr, units='1',          &
                                  longname='Optical thickness (825 nm)'                                                 )

          CALL add_stream_element(rad,      'ABS_MODE_'//cmode(jmod),       abs_mode(jmod)%ptr,     units='1',          &
                                  longname='Absorption optical thickness (550 nm)'                                      )

          IF (lodiagrad) THEN

          CALL add_stream_element(rad,      'SIGMA_MODE_'//cmode(jmod),     sigma_mode(jmod)%ptr,   units='cm+2 part-1',&
                                  longname='Extinction cross section per particle (550 nm)'                             )

          CALL add_stream_element(rad,      'OMEGA_MODE_'//cmode(jmod),     omega_mode(jmod)%ptr,   units='1',          &
                                  longname='Single scattering albedo (550 nm)'                                          )

          CALL add_stream_element(rad,      'ASYM_MODE_'//cmode(jmod),      asym_mode(jmod)%ptr,    units='1',          &
                                  longname='Asymetry Factor (550 nm)'                                                   )

          CALL add_stream_element(rad,      'NR_MODE_'//cmode(jmod),        nr_mode(jmod)%ptr,      units='1',          &
                                  longname='Refractive Index - real part (550 nm)'                                      )

          CALL add_stream_element(rad,      'NI_MODE_'//cmode(jmod),        ni_mode(jmod)%ptr,      units='1',          &
                                  longname='Refractive Index - imaginary part (550 nm)'                                 )

!@@@      CALL add_stream_element (rad,      'LIDAR_MODE_'//cmode(jmod),    lidar_mode(jmod)%ptr, units='1',          &
!@@@                               longname='Lidar ratio at specified wavelength'                                     )

          END IF ! lodiagrad

       END IF
    END DO

    !--- 2.2) Diagnostics for each compound:

    DO jtype=1, ntype

       CALL add_stream_element(rad,    'TAU_COMP_'//TRIM(ctype(jtype)),     tau_comp(jtype)%ptr,     units='1', &
                               longname='Optical thickness '//TRIM(ctype(jtype))//' (550 nm)'                   )

       CALL add_stream_element(rad,    'TAU_COMP_825_'//TRIM(ctype(jtype)), tau_comp_825(jtype)%ptr, units='1', &
                               longname='Optical thickness '//TRIM(ctype(jtype))//' (825 nm)'                   )

       CALL add_stream_element(rad,    'ABS_COMP_'//TRIM(ctype(jtype)),     abs_comp(jtype)%ptr,     units='1', &
                               longname='Absorption optical thickness '//TRIM(ctype(jtype))//' (550 nm)'        )

    END DO

    !--- 2.3) 2D total fields:

    CALL default_stream_setting (rad, leveltype = SURFACE )

    CALL add_stream_element(rad,      'TAU_2D',     tau_2d,     units='1',                            &
                            longname='Optical thickness - total (550 nm)'                             )

    CALL add_stream_element(rad,      'TAU_2D_825', tau_2d_825, units='1',                            &
                            longname='Optical thickness - total (825 nm)'                             )

    CALL add_stream_element(rad,      'ABS_2D',     abs_2d,     units='1',                            &
                            longname='Absorption optical thickness - total (0.55 um)'                 )

    CALL add_stream_element(rad,      'ANG',        ang,        units='1',                            &
                            longname='Angstroem parameter between 550 and 825 nm'                     )

  END SUBROUTINE construct_stream_aero_rad

END MODULE mo_aero_rad_mem
