MODULE mo_aero_tools

  ! *mo_aero_tools* hold auxiliary routines for the 
  !                 HAM aerosol model

  USE mo_kind,               ONLY: dp
  USE mo_exception,          ONLY: finish

  IMPLICIT NONE

CONTAINS

  SUBROUTINE geoindex (plon, plat, klon, klat)

    !   *geoindex* calculates the corresponding lat-lon (klon,klat) index
    !              for given lat-lon [-90,90][0,360] coordinates (plon, plat)
    !
    !   Authors: 
    !   --------- 
    !   Philip Stier, MPI-MET            2002 
    !
    !   Externals 
    !   ----------- 
    !   none 
    

    USE mo_exception, ONLY: finish
    USE mo_control,   ONLY: ngl, nlon
    USE mo_gaussgrid, ONLY: philat, philon
    IMPLICIT NONE

    REAL(dp),INTENT(IN)  :: plon, plat   ! Coordinates in degrees
    INTEGER, INTENT(OUT) :: klon, klat   ! Corresponding indices

    REAL(dp)            :: zxdif, zydif
    INTEGER             :: jlon, jlat
      

    IF((NINT(plon)<0).OR.(NINT(plon)>360).OR.(NINT(plat)<-90).OR.(NINT(plat)>90)) THEN
       CALL finish('geoindex:', 'Coordinates out of range')
    END IF

    klon=-999
    klat=-999

    zxdif=360._dp
    zydif=180._dp

    !--- 1) Search for closest longitude in [0,360}:

    DO jlon = 1, nlon
       IF(ABS(plon-philon(jlon)) < zxdif) THEN
          klon=jlon
          zxdif=ABS(plon-philon(jlon))
       END IF
    END DO

    !--- 2) Search for closest latitude [90,-90]:

    DO jlat = 1, ngl
       IF( ABS(plat-philat(jlat)) < zydif ) THEN
          klat=jlat
          zydif=ABS(plat-philat(jlat)) 
       END IF
    END DO


    IF(klon==-999 .OR. klat==-999) CALL finish('geoindex:', 'No index found')

  END SUBROUTINE geoindex

!---------------------------------------------------------------------------------------------

  SUBROUTINE calc_daylength

    ! *dayfac* calculates relative daylength
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-Met          14/11/2002
    !
    ! Method:
    ! -------
    ! Adapted from the routines "solang" and "prerad"

    USE mo_kind,           ONLY: dp
    USE mo_control,        ONLY: nlon, ngl
    USE mo_constants,      ONLY: api
    USE mo_gaussgrid,      ONLY: coslon, sinlon, gl_twomu, gl_sqcst
    USE mo_physc1,         ONLY: cdisse
    USE mo_time_control,   ONLY: get_clock,    julian_date,  l_orbvsop87, &
                                 get_year_len, current_date, start_date,  &
                                 get_date_components
    USE mo_time_conversion,ONLY: TC_get
    USE mo_time_base,      ONLY: Set_JulianDay
    USE mo_orbit,          ONLY: orbit
    USE mo_decomposition,  ONLY: gdc => global_decomposition
    USE mo_transpose,      ONLY: scatter_gp
    USE mo_test_trans,     ONLY: test_gridpoint
    USE mo_aero_mem,       ONLY: daylength

    IMPLICIT NONE

    INTEGER :: jl, jlat

    LOGICAL :: lo

    TYPE(julian_date)  :: jul_date, jul_date_pal
    REAL(dp):: zclock, zdoy, zdoyve
    REAL(dp):: zra, zdec, zdis, zgha
    REAL(dp):: zvemar0, zyearve
    INTEGER :: yr, mo, dy, hr, mn, se, jday, jsec
    REAL(dp):: czen1,  czen2,  czen3, ztim1, ztim2, ztim3, zsum

    REAL(dp):: zmu0(nlon), zrdayl(nlon)

    REAL(dp), POINTER :: zdaylength(:,:)


    !--- 0) Initialization:

    ALLOCATE (zdaylength(nlon,ngl))


    !--- 1) Compute orbital parameters for present time step:

    zclock = get_clock(current_date)

    CALL get_date_components(current_date, yr, mo, dy, hr, mn, se)
    CALL TC_get             (current_date, jday, jsec)
    CALL Set_JulianDay(yr, mo, dy, jsec, jul_date)

    ! PCMDI-Orbit
    IF (.NOT. l_orbvsop87) THEN

       CALL get_date_components(start_date,  yr ,mo ,dy ,hr ,mn ,se)
       CALL Set_JulianDay(1900, 1, 1, 0, jul_date_pal)

       zdoy =  (jul_date    %day + jul_date    %fraction) &
             - (jul_date_pal%day + jul_date_pal%fraction)

       ! Determine day of vernal equinox in March 1900

       zvemar0 = 78.41_dp - 0.0078_dp*(1900-1987) + 0.25_dp*MOD(1900,4)
       zyearve = zdoy + get_year_len() - zvemar0
       zdoyve = MOD(zyearve/get_year_len(),1.0_dp)*2._dp*api
       CALL orbit (zdoyve, cdisse, zdec, zra)
       !
       ! vsop87-orbit
    ELSE
       zdoy = jul_date%day + jul_date%fraction
       CALL orbit (zdoy, zra, zdec, zdis, zgha)
       cdisse = 1.0_dp/zdis**2
    END IF

    czen1 = SIN(zdec)
    czen2 = COS(zdec)*COS(zclock)
    czen3 = COS(zdec)*SIN(zclock)


    !--- 2) Calculate relative daylength (from routine solang):

    DO jlat = 1, ngl

       ztim1 =  czen1*0.5_dp*gl_twomu(jlat)
       ztim2 = -czen2*gl_sqcst(jlat)
       ztim3 =  czen3*gl_sqcst(jlat)

       DO jl = 1, nlon
          zmu0(jl) = ztim1 + ztim2*coslon(jl) + ztim3*sinlon(jl)
          lo = zmu0(jl) >= 0._dp
          zrdayl(jl) = MERGE(1._dp,0._dp,lo)
       END DO

       zsum = SUM(zrdayl(1:nlon))

       IF (ABS(zsum) > 0._dp) THEN
          zdaylength(:,jlat) = zsum/REAL(nlon,dp)
       ELSE
          zdaylength(:,jlat) = 0._dp
       END IF

    END DO


    !--- 3) Scatter in stream element:

    CALL scatter_gp(zdaylength, daylength, gdc)

    CALL test_gridpoint(daylength, 'daylength')

    DEALLOCATE (zdaylength)

  END SUBROUTINE calc_daylength

!---------------------------------------------------------------------------------------------

  SUBROUTINE aero_logtail(kproma, kbdim,  klev,  krow, &
                          pr,     pfracn, pfracm       )

    ! *aero_logtail* calculates mass- and number-fraction larger than
    !                the radius pr for each mode of a superposition 
    !                of nmod log-normal aerosol distributions
    !
    ! Author:
    ! -------
    ! Philip Stier, MPI-MET                       2004
    !
    ! Method:
    ! -------

    !
    ! The calculation of the activated number fraction and mass fraction
    ! from the radius of activation is done by a transformation of the 
    ! log-normal distribution to the error function which is then computed
    ! using the routine m7_cumnor:
    !
    !                        / x                              _
    !                 N      |       1           1   ln(R)-ln(R)  2
    !    N(0,x) = ---------  |   --------  exp(- - ( ----------- )   ) d ln(R) 
    !             ln(sigma)  |   sqrt(2PI)       2    ln(sigma)
    !                        / 0 
    !                         
    !                         /tx                   2
    !                        |        1            t
    !           =     N      |     --------  exp(- - ) d t 
    !                        |     sqrt(2PI)       2 
    !                        /-inf
    ! 
    !    where:                   
    !
    !                        _
    !               ln(R)-ln(R)
    !    t      =   -----------
    !                ln(sigma)
    !
    !    and:
    !                        _
    !               ln(x)-ln(R)
    !    tx     =   -----------
    !                ln(sigma)


    USE mo_aero_m7,  ONLY: nmod, cmedr2mmedr, sigmaln
    USE mo_aero_mem, ONLY: rdry


    IMPLICIT NONE


    !--- Subroutine parameters:

    INTEGER :: kproma, kbdim, klev, krow

    REAL(dp)    :: pr(kbdim,klev,nmod)

    REAL(dp)    :: pfracn(kbdim,klev,nmod), pfracm(kbdim,klev,nmod)

    !--- Local variables:

    INTEGER :: jl, jk, jmod

    REAL(dp)    :: ztn, ztm, zdummy, zeps

    REAL(dp), POINTER :: rdry_p(:,:,:)


    !--- 0) 

    zeps=EPSILON(1.0_dp)

    !--- 1) 

    DO jmod=1, nmod

       rdry_p => rdry(jmod)%ptr

       DO jk=1, klev
          DO jl=1, kproma

             IF (pr(jl,jk,jmod)>zeps .AND. rdry_p(jl,jk,krow)>zeps) THEN

                !--- Transform number distribution to error function:

                ztn=(LOG(pr(jl,jk,jmod))-LOG(rdry_p(jl,jk,krow)))/sigmaln(jmod)

                !--- Transform mass distribution to error function:

                ztm=(LOG(pr(jl,jk,jmod))-LOG(rdry_p(jl,jk,krow)*cmedr2mmedr(jmod)))/sigmaln(jmod)

                !--- Calculate the cumulative of the log-normal number distribution:

                CALL m7_cumnor(ztn,zdummy,pfracn(jl,jk,jmod))

                !--- Calculate the cumulative of the log-normal mass distribution:

                CALL m7_cumnor(ztm,zdummy,pfracm(jl,jk,jmod))

             ELSE IF (pr(jl,jk,jmod)<zeps .AND. rdry_p(jl,jk,krow)>zeps) THEN

                pfracn(jl,jk,jmod)=1.0_dp
                pfracm(jl,jk,jmod)=1.0_dp

             ELSE

                pfracn(jl,jk,jmod)=0.0_dp
                pfracm(jl,jk,jmod)=0.0_dp

             END IF

          END DO !kproma
       END DO !klev
    END DO !nmod

  END SUBROUTINE aero_logtail

END MODULE mo_aero_tools
