MODULE mo_aero_trac

  ! *mo_aero_trac* contains routines to requests tracers for ECHAM/HAM and 
  !                prescribes their physical and chemical properties.
  !                Controls the aerosol physics by providing the necessary
  !                switches.
  !
  ! Author:
  ! -------
  ! Philip Stier, MPI                          2001
  !
  ! Parameters:
  ! -----------
  ! User defined flags: density   density                    [kg m-3]
  !                     osm       osmotic coefficient        [???]
  !                     nion      number of ions the tracer 
  !                               dissolves into             [1]

  USE mo_kind,      ONLY: dp
  USE mo_tracer,    ONLY: new_tracer,           & ! request resources for a tracer
                          GAS,                  & ! phase indicators
                          AEROSOLMASS,          & 
                          AEROSOLNUMBER,        &
                          SOLUBLE,              & ! soluble indicator
                          OFF, ON
  USE mo_tracdef,   ONLY: t_flag                  ! flag type definition
  USE mo_constants, ONLY: rhoh2o

  IMPLICIT NONE

  !--- Public entities:

  PRIVATE

  PUBLIC :: idt_dms,   idt_so2,   idt_so4,                 &
            idt_oh,    idt_h2o2,  idt_o3,    idt_no2,      &
            idt_ms4ns, idt_ms4ks, idt_ms4as, idt_ms4cs,    &
            idt_mbcki, idt_mbcks, idt_mbcas, idt_mbccs,    &
            idt_mocki, idt_mocks, idt_mocas, idt_moccs,    &
            idt_mssas, idt_msscs,                          &
            idt_mduai, idt_mduas, idt_mduci, idt_mducs,    &
            idt_nns,   idt_nki,   idt_nks,   idt_nai,      &
            idt_nas,   idt_nci,   idt_ncs,                 &
            idt_cdnc,  idt_icnc,                           &
            idt_mwans, idt_mwaks, idt_mwaas, idt_mwacs

  PUBLIC:: ham_request_tracer

  !--- Module variables:
  !
  !    Tracer indices:
  !
  !    Legend: iABBCD
  !
  !            A:  m  = particle mass mixing ratio, n number mixing ratio
  !            BB: s4 = sulfate, bc/oc = black/organic carbon, du = dust, ss = seasalt
  !            C:  n  = nucleation , k = Aitken, a = accumulation, c = coarse mode
  !            D:  i  = insoluble,  s = soluble

  INTEGER :: idt_dms    ! mass mixing ratio dms
  INTEGER :: idt_so2    ! mass mixing ratio so2
  INTEGER :: idt_so4    ! mass mixing ratio so4

  INTEGER :: idt_oh     ! mass mixing ratio oh    - currently prescribed offline
  INTEGER :: idt_h2o2   ! mass mixing ratio h2o2  - currently prescribed offline
  INTEGER :: idt_o3     ! mass mixing ratio o3    - currently prescribed offline
  INTEGER :: idt_no2    ! mass mixing ratio no2   - currently prescribed offline

  INTEGER :: idt_ms4ns  ! mass mixing ratio sulfate        nuclea. soluble
  INTEGER :: idt_ms4ks  ! mass mixing ratio sulfate        aitken  soluble
  INTEGER :: idt_ms4as  ! mass mixing ratio sulfate        accum.  soluble
  INTEGER :: idt_ms4cs  ! mass mixing ratio sulfate        coarse  soluble
  INTEGER :: idt_mbcki  ! mass mixing ratio black carbon   aitken  insoluble
  INTEGER :: idt_mbcks  ! mass mixing ratio black carbon   aitken  soluble
  INTEGER :: idt_mbcas  ! mass mixing ratio black carbon   accum.  soluble
  INTEGER :: idt_mbccs  ! mass mixing ratio black carbon   coarse  soluble
  INTEGER :: idt_mocki  ! mass mixing ratio organic carbon aitken  insoluble
  INTEGER :: idt_mocks  ! mass mixing ratio organic carbon aitken  soluble
  INTEGER :: idt_mocas  ! mass mixing ratio organic carbon accum.  soluble
  INTEGER :: idt_moccs  ! mass mixing ratio organic carbon coarse  soluble
  INTEGER :: idt_mssas  ! mass mixing ratio seasalt        accum.  soluble
  INTEGER :: idt_msscs  ! mass mixing ratio seasalt        coarse  soluble
  INTEGER :: idt_mduai  ! mass mixing ratio dust           accum.  insoluble
  INTEGER :: idt_mduas  ! mass mixing ratio dust           accum.  soluble
  INTEGER :: idt_mduci  ! mass mixing ratio dust           coarse  insoluble
  INTEGER :: idt_mducs  ! mass mixing ratio dust           coarse  soluble
  INTEGER :: idt_mwans  ! mass mixing ratio aerosol water  nuclea. soluble
  INTEGER :: idt_mwaks  ! mass mixing ratio aerosol water  aitken  soluble
  INTEGER :: idt_mwaas  ! mass mixing ratio aerosol water  accum.  soluble
  INTEGER :: idt_mwacs  ! mass mixing ratio aerosol water  coarse  soluble


  INTEGER :: idt_nns    ! number mixing ratio              nuclea. soluble
  INTEGER :: idt_nki    ! number mixing ratio              aitken  insoluble
  INTEGER :: idt_nks    ! number mixing ratio              aitken  soluble
  INTEGER :: idt_nai    ! number mixing ratio              accum.  insoluble
  INTEGER :: idt_nas    ! number mixing ratio              accum.  soluble
  INTEGER :: idt_nci    ! number mixing ratio              coarse  insoluble
  INTEGER :: idt_ncs    ! number mixing ratio              coarse  soluble

  INTEGER :: idt_cdnc   ! cloud droplet number concentration
  INTEGER :: idt_icnc   ! ice   cristal number concentration

CONTAINS
!------------------------------------------------------------------------------
  SUBROUTINE HAM_request_tracer
  !
  ! *HAM_request_tracer* request tracers from ECHAM
  !
  ! 
  ! local grib table 131 (tracer), reserved grib code numbers:

  ! grib codes 129,   152 reserved for surface pressure, geopotential

  ! grib codes   1 .. 100 MOZART chemistry

  ! grib codes 101 .. 128 CO tracer

  ! grib codes 161 .. 180 Radon etc.

  ! grib codes 201 .. 255 aerosol model

  ! grib codes 181 .. 200 sulphur chemistry

  ! grib codes 131 .. 150 anything else

  CALL new_tracer('DMS'        ,'HAM'                     ,idx=idt_dms          ,units='kg(S) kg-1'       &
                  ,table=131   ,code=181                                                                  &
                  ,ndrydep=OFF ,nwetdep=OFF               ,nphase=GAS                                     &
                  ,moleweight=62.019_dp                                                                      &
                  ,longname='DMS mass mixing ratio - gas phase',henry=0.56_dp                                )

  CALL new_tracer('SO2'        ,'HAM'                     ,idx=idt_so2          ,units='kg(S) kg-1'       &
                  ,table=131   ,code=182                                                                  &
                  ,ndrydep=2   ,nwetdep=ON                ,nphase=GAS                                     &
                  ,moleweight=63.962_dp       ,henry=1.23_dp   ,dryreac=0._dp                                     &
                  ,longname='Sulfur Dioxide mass mixing ratio - gas phase'                                )

  CALL new_tracer('SO4'        ,'HAM'       ,subname='gas',idx=idt_so4          ,units='kg(S) kg-1'       &
                  ,table=131   ,code=183                                                                  &
                  ,ndrydep=2   ,nwetdep=ON                ,nphase=GAS                                     &
                  ,moleweight=95.952_dp                      ,dryreac=0._dp                                     &
                  ,longname='Sulfate mass mixing ratio - gas phase'                                       )

  !--- Define offline fields as tracers that are not transported but can be accessed as a tracer:

  CALL new_tracer('OH'         ,'HAM'                     ,idx=idt_oh           ,units='VMR'              &
                  ,table=131   ,code=184                                                                  &
                                                          ,nphase=GAS                                     &
                  ,nconv=OFF   ,nvdiff=OFF  ,ntran=OFF    ,nint=OFF             ,nwrite=OFF               &
                  ,moleweight=17.003_dp        ,henry=25._dp                                                    &
                  ,longname='OH volume mixing ratio'                                                      )

  CALL new_tracer('H2O2'       ,'HAM'                     ,idx=idt_h2o2         ,units='VMR'              &
                  ,table=131   ,code=185                                                                  &
                                                          ,nphase=GAS                                     &
                  ,nconv=OFF   ,nvdiff=OFF  ,ntran=OFF    ,nint=OFF             ,nwrite=OFF               &
                  ,moleweight=34.005_dp        ,henry=7.45E4_dp ,dryreac=1.0_dp                                    &
                  ,longname='H2O2 volume mixing ratio'                                                    )

  CALL new_tracer('O3'         ,'HAM'                     ,idx=idt_o3           ,units='VMR'              &
                  ,table=131   ,code=186                                                                  &
                                                          ,nphase=GAS                                     &
                  ,nconv=OFF   ,nvdiff=OFF  ,ntran=OFF    ,nint=OFF             ,nwrite=OFF               &
                                            ,henry=1.13E-2_dp,dryreac=1.0_dp                                    &
                  ,longname='Ozone volume mixing ratio'                                                   )

  CALL new_tracer('NO2'        ,'HAM'                     ,idx=idt_no2          ,units='VMR'              &
                  ,table=131   ,code=187                                                                  &
                                                          ,nphase=GAS                                     &
                  ,nconv=OFF   ,nvdiff=OFF  ,ntran=OFF    ,nint=OFF             ,nwrite=OFF               &
                  ,moleweight=46.006_dp        ,henry=1.0E-2_dp ,dryreac=1.0_dp                                    &
                  ,longname='NO2 volume mixing ratio'                                                     )
  
  ! grib codes 201 .. 255 aerosol model

  !--- 2) Allocate aerosol masses according to the modes to obtain succeding tracer identifiers:

  !--- Mode 1 - Nucleation Soluble:

  CALL new_tracer('SO4'        ,'HAM'       ,subname='NS' ,idx=idt_ms4ns        ,units='kg(S) kg-1'       &
                  ,table=131   ,code=201                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=OFF    ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=1 &
                  ,myflag=(/t_flag('density',1841._dp),t_flag('osm',1._dp),t_flag('nion',2._dp)/)                  &
                  ,moleweight=95.952_dp                                                                      &
                  ,longname='Sulfate mass mixing ratio - aerosol mode nucleation soluble'                 )

  !--- Mode 2 - Aitken Soluble:

  CALL new_tracer('SO4'        ,'HAM'       ,subname='KS' ,idx=idt_ms4ks        ,units='kg(S) kg-1'       &
                  ,table=131   ,code=202                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=OFF    ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=2 &
                  ,myflag=(/t_flag('density',1841._dp),t_flag('osm',1._dp),t_flag('nion',2._dp)/)                  &
                  ,moleweight=95.952_dp                                                                      &
                  ,longname='Sulfate mass mixing ratio - aerosol mode aitken soluble'                     )

  CALL new_tracer('BC'         ,'HAM'       ,subname='KS' ,idx=idt_mbcks        ,units='kg kg-1'          &
                  ,table=131   ,code=206                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=OFF    ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=2 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Black Carbon mass mixing ratio - aerosol mode aitken soluble'                )

  CALL new_tracer('OC'         ,'HAM'       ,subname='KS' ,idx=idt_mocks        ,units='kg kg-1'          &
                  ,table=131   ,code=210                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=OFF    ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=2 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Organic Carbon mass mixing ratio - aerosol mode aitken soluble'              )

  !--- Mode 3 - Accumulation Soluble:

  CALL new_tracer('SO4'        ,'HAM'       ,subname='AS' ,idx=idt_ms4as        ,units='kg(S) kg-1'       &
                  ,table=131   ,code=203                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=ON     ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=3 &
                  ,myflag=(/t_flag('density',1841._dp),t_flag('osm',1._dp),t_flag('nion',2._dp)/)                  &
                  ,moleweight=95.952_dp                                                                      &
                  ,longname='Sulfate mass mixing ratio - aerosol mode accumulation soluble'               )

  CALL new_tracer('BC'         ,'HAM'       ,subname='AS' ,idx=idt_mbcas        ,units='kg kg-1'          &
                  ,table=131   ,code=207                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=ON     ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=3 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Black Carbon mass mixing ratio - aerosol mode accumulation soluble'          )

  CALL new_tracer('OC'         ,'HAM'       ,subname='AS' ,idx=idt_mocas        ,units='kg kg-1'          &
                  ,table=131   ,code=211                                                                  & 
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=ON     ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=3 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Organic Carbon mass mixing ratio - aerosol mode accumulation soluble'        )

  CALL new_tracer('SS'         ,'HAM'       ,subname='AS' ,idx=idt_mssas        ,units='kg kg-1'          &
                  ,table=131   ,code=213                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=ON     ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=3 &
                  ,myflag=(/t_flag('density',2165._dp),t_flag('osm',1._dp),t_flag('nion',2._dp)/)                  &
                  ,moleweight=58.44_dp                                                                       &
                  ,longname='Sea Salt mass mixing ratio - aerosol mode accumulation soluble'              )

  CALL new_tracer('DU'         ,'HAM'       ,subname='AS' ,idx=idt_mduas        ,units='kg kg-1'          &
                  ,table=131   ,code=216                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=ON     ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=3 &
                  ,myflag=(/t_flag('density',2650._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Mineral Dust mass mixing ratio - aerosol mode accumulation soluble'          )

  !--- Mode 4 - Coarse Soluble:

  CALL new_tracer('SO4'        ,'HAM'       ,subname='CS' ,idx=idt_ms4cs        ,units='kg(S) kg-1'       &
                  ,table=131   ,code=204                                                                  &
                  ,ndrydep=2   ,nwetdep=ON  ,nsedi=ON     ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=4 &
                  ,myflag=(/t_flag('density',1841._dp),t_flag('osm',1._dp),t_flag('nion',2._dp)/)                  &
                  ,moleweight=95.952_dp                                                                      &
                  ,longname='Sulfate mass mixing ratio - aerosol mode coarse soluble'                     )

  CALL new_tracer('BC'         ,'HAM'      ,subname='CS'  ,idx=idt_mbccs        ,units='kg kg-1'          &
                  ,table=131   ,code=208                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=4 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Black Carbon mass mixing ratio - aerosol mode coarse soluble'                )

  CALL new_tracer('OC'         ,'HAM'      ,subname='CS'  ,idx=idt_moccs        ,units='kg kg-1'          &
                  ,table=131   ,code=212                                                                  & 
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=4 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Organic Carbon mass mixing ratio - aerosol mode coarse soluble'              )

  CALL new_tracer('SS'         ,'HAM'      ,subname='CS'  ,idx=idt_msscs        ,units='kg kg-1'          &
                  ,table=131   ,code=214                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=4 &
                  ,myflag=(/t_flag('density',2165._dp),t_flag('osm',1._dp),t_flag('nion',2._dp)/)                  &
                  ,moleweight=58.44_dp                                                                       &
                  ,longname='Sea Salt mass mixing ratio - aerosol mode coarse soluble'                    )

  CALL new_tracer('DU'         ,'HAM'      ,subname='CS'  ,idx=idt_mducs        ,units='kg kg-1'          &
                  ,table=131   ,code=218                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLMASS   ,nsoluble=SOLUBLE ,mode=4 &
                  ,myflag=(/t_flag('density',2650._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Mineral Dust mass mixing ratio - aerosol mode coarse soluble'                )

  !--- Mode 5 - Aitken Insoluble:

  CALL new_tracer('BC'         ,'HAM'      ,subname='KI'  ,idx=idt_mbcki        ,units='kg kg-1'          &
                  ,table=131   ,code=205                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=OFF     ,nphase=AEROSOLMASS                     ,mode=5 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Black Carbon mass mixing ratio - aerosol mode aitken insoluble'              )

  CALL new_tracer('OC'         ,'HAM'      ,subname='KI'  ,idx=idt_mocki        ,units='kg kg-1'          &
                  ,table=131   ,code=209                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=OFF     ,nphase=AEROSOLMASS                     ,mode=5 &
                  ,myflag=(/t_flag('density',2000._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Organic Carbon mass mixing ratio - aerosol mode aitken insoluble'            )

  !--- Mode 6 - Accumulation Insoluble:

  CALL new_tracer('DU'         ,'HAM'      ,subname='AI'  ,idx=idt_mduai        ,units='kg kg-1'          &
                  ,table=131   ,code=215                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLMASS                     ,mode=6 &
                  ,myflag=(/t_flag('density',2650._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Mineral Dust mass mixing ratio - aerosol mode accumulation insoluble'        )

  !--- Mode 7 - Coarse Insoluble:

  CALL new_tracer('DU'         ,'HAM'      ,subname='CI'  ,idx=idt_mduci        ,units='kg kg-1'          &
                  ,table=131   ,code=217                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLMASS                     ,mode=7 &
                  ,myflag=(/t_flag('density',2650._dp),t_flag('osm',0._dp),t_flag('nion',0._dp)/)                  &
                  ,longname='Mineral Dust mass mixing ratio - aerosol mode coarse insoluble'              )

  !--- 3) Aerosol Numbers:

  CALL new_tracer('NUM'        ,'HAM'      ,subname='NS'  ,idx=idt_nns          ,units='1 kg-1'           &
                  ,table=131   ,code=219                                                                  &
                  ,ndrydep=2  ,nwetdep=ON ,nsedi=OFF     ,nphase=AEROSOLNUMBER ,nsoluble=SOLUBLE ,mode=1 &
                  ,longname='number mixing ratio - aerosol mode nucleation soluble'                       )

  CALL new_tracer('NUM'        ,'HAM'      ,subname='KS'  ,idx=idt_nks          ,units='1 kg-1'           &
                  ,table=131   ,code=221                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=OFF     ,nphase=AEROSOLNUMBER ,nsoluble=SOLUBLE ,mode=2 &
                  ,longname='number mixing ratio - aerosol mode aitken soluble'                           )

  CALL new_tracer('NUM'        ,'HAM'      ,subname='AS'  ,idx=idt_nas          ,units='1 kg-1'           &
                  ,table=131   ,code=223                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLNUMBER ,nsoluble=SOLUBLE ,mode=3 &
                  ,longname='number mixing ratio - aerosol mode accumulation soluble'                     )

  CALL new_tracer('NUM'        ,'HAM'      ,subname='CS'  ,idx=idt_ncs          ,units='1 kg-1'           &
                  ,table=131   ,code=225                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLNUMBER ,nsoluble=SOLUBLE ,mode=4 &
                  ,longname='number mixing ratio - aerosol mode coarse soluble'                           )

  CALL new_tracer('NUM'        ,'HAM'      ,subname='KI'  ,idx=idt_nki          ,units='1 kg-1'           &
                  ,table=131   ,code=220                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=OFF     ,nphase=AEROSOLNUMBER                   ,mode=5 &
                  ,longname='number mixing ratio - aerosol mode aitken insoluble'                         )

  CALL new_tracer('NUM'        ,'HAM'      ,subname='AI'  ,idx=idt_nai          ,units='1 kg-1'           &
                  ,table=131   ,code=222                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLNUMBER                   ,mode=6 &
                  ,longname='number mixing ratio - aerosol mode accumulation insoluble'                   )

  CALL new_tracer('NUM'        ,'HAM'      ,subname='CI'  ,idx=idt_nci          ,units='1 kg-1'           &
                  ,table=131   ,code=224                                                                  &
                  ,ndrydep=2   ,nwetdep=ON ,nsedi=ON      ,nphase=AEROSOLNUMBER                   ,mode=7 &
                  ,longname='number mixing ratio -aerosol mode coarse insoluble'                          )

  !--- 4) Define Aerosol Water as Pseudo-Tracers:

  CALL new_tracer('WAT'        ,'HAM'      ,subname='NS'  ,idx=idt_mwans        ,units='kg kg-1'          &
                  ,table=131   ,code=225                                                                  &
                  ,ndrydep=OFF ,nwetdep=OFF,nsedi=OFF     ,nphase=AEROSOLMASS                     ,mode=1 &
                  ,nconv=OFF   ,nvdiff=OFF ,ntran=OFF     ,nint=ON              ,nwrite=ON                &
                  ,myflag=(/t_flag('density',rhoh2o)/)                                                    &
                  ,longname='Aerosol water mass mixing ratio - aerosol mode nucleation soluble'           )

  CALL new_tracer('WAT'        ,'HAM'      ,subname='KS'  ,idx=idt_mwaks        ,units='kg kg-1'          &
                  ,table=131   ,code=226                                                                  &
                  ,ndrydep=OFF ,nwetdep=OFF,nsedi=OFF     ,nphase=AEROSOLMASS                     ,mode=2 &
                  ,nconv=OFF   ,nvdiff=OFF ,ntran=OFF     ,nint=ON              ,nwrite=ON                &
                  ,myflag=(/t_flag('density',rhoh2o)/)                                                    &
                  ,longname='Aerosol water mass mixing ratio - aerosol mode aitken soluble'               )

  CALL new_tracer('WAT'        ,'HAM'      ,subname='AS'  ,idx=idt_mwaas        ,units='kg kg-1'          &
                  ,table=131   ,code=227                                                                  &
                  ,ndrydep=OFF ,nwetdep=OFF,nsedi=OFF     ,nphase=AEROSOLMASS                     ,mode=3 &
                  ,nconv=OFF   ,nvdiff=OFF ,ntran=OFF     ,nint=ON              ,nwrite=ON                &
                  ,myflag=(/t_flag('density',rhoh2o)/)                                                    &
                  ,longname='Aerosol water mass mixing ratio - aerosol mode aitken soluble'               )

  CALL new_tracer('WAT'        ,'HAM'      ,subname='CS'  ,idx=idt_mwacs        ,units='kg kg-1'          &
                  ,table=131   ,code=228                                                                  &
                  ,ndrydep=OFF ,nwetdep=OFF,nsedi=OFF     ,nphase=AEROSOLMASS                     ,mode=4 &
                  ,nconv=OFF   ,nvdiff=OFF ,ntran=OFF     ,nint=ON              ,nwrite=ON                &
                  ,myflag=(/t_flag('density',rhoh2o)/)                                                    &
                  ,longname='Aerosol water mass mixing ratio - aerosol mode aitken soluble'               )

END SUBROUTINE HAM_request_tracer

!------------------------------------------------------------------------------
END MODULE mo_aero_trac
