 MODULE mo_bgc_dust_mem

  ! Description:
  ! ------------
  ! *mo_bgc_dust_mem* handles necessary parameters and variables for MPI BGC dust emission scheme.
  ! The various routines of the scheme are found in *mo_bgc_dust_emis*.
  !
  ! Author:
  ! -------
  ! Martin Werner, MPI Biogeochemistry (MPI BGC), Jena, October 2002
  ! Philip Stier,  MPI Meteorology, Hamburg, 2003:
  !                fitted parameters for emissions size distribution
  !                introduced scale factor for wind stress threshold
  ! Martin Werner, MPI Biogeochemistry (MPI BGC), Jena, October 2004:
  !                changed scheme from 0.5x0.5 degree grid to default
  !                Echam5-HAM grid (to save computational costs)
  !
  ! Interface:
  ! ----------
  ! The routines for reading in necessary input files and for calculating the emissions are
  ! called from the routine *read_chem_files* in module *mo_chem_bcond*.
  !
  ! Method:
  ! -------
  ! *mo_bgc_dust_emis* holds the various routines for the MPI BGC dust emission scheme, while 
  ! public parameters and variables are stored in *mo_bgc_dust_mem*.
  !
  ! The stream *emi_du_src* of the HAM model is used to store the calculated dust emissions after
  ! interpolation to the defined ECHAM resolution is performed. The routine *dust_emissions_bgc*
  ! (called by routine *call_chem_bcond* in call_submodels) links the MPI BGC emissions to
  ! the HAM aerosol model developed by Philip Stier et al., MPI Meteorology, Hamburg.
  
   USE mo_kind,          ONLY: dp
   USE mo_control,       ONLY: nlon, ngl

   IMPLICIT NONE

! *** dust parameter
   REAL(dp), PARAMETER     :: a_rnolds=1331.647_dp          ! Reynolds constant
   REAL(dp), PARAMETER     :: aeff=0.35_dp                  ! efficient fraction

   REAL(dp), PARAMETER     :: b_rnolds=0.38194_dp           ! Reynolds constant
  
   REAL(dp), PARAMETER     :: d_thrsld=0.00000231_dp        ! thresold value
   REAL(dp), PARAMETER     :: Dmin=0.00002_dp               ! minimum particules diameter (cm)
   REAL(dp), PARAMETER     :: Dmax=0.130_dp                 ! maximum particules diameter (cm)
   REAL(dp), PARAMETER     :: Dstep=0.0460517018598807_dp   ! diameter increment (cm)
       
   INTEGER, PARAMETER      :: ntrace=8                      ! number of tracers
   INTEGER, PARAMETER      :: nbin=24                       ! number of bins per tracer
   INTEGER, PARAMETER      :: nclass=ntrace*nbin            ! number of particle classes
   INTEGER, PARAMETER      :: nats =12                      ! number of soil types
   INTEGER, PARAMETER      :: nmode=4
   INTEGER, PARAMETER      :: nspe = nmode*3+2
    
   REAL(dp), PARAMETER     :: roa=0.001227_dp               ! air density (g/cm-3)
   REAL(dp), PARAMETER     :: rop=2.65_dp                   ! particle density (g/cm-3)
  
   REAL(dp), PARAMETER     :: umin=21._dp                   ! minimum threshold friction windspeed (cm/s)
 
   REAL(dp), PARAMETER     :: vk=0.4_dp                     ! Von Karman constant: 0.4 (0.35 <--> 0.42 see Stull) 
  
   REAL(dp), PARAMETER     :: w0=0.99_dp                    ! threshold of relative soil humidity 
  
   REAL(dp), PARAMETER     :: x_rnolds=1.561228_dp          ! Reynolds constant
   REAL(dp), PARAMETER     :: xeff=10._dp                   ! efficient fraction
  
   REAL(dp), PARAMETER     :: zz=1000._dp                   ! wind measurment height (cm)
   REAL(dp), PARAMETER     :: z0s=0.001_dp                  ! roughness length of surface without obstacles (cm) 
                                                            ! (see: Thesis of B. Marticorena, p.85)
   REAL(dp), PARAMETER     :: z01=0.001_dp                  ! roughness length parameter (see: Thesis of B. Marticorena)
   REAL(dp), PARAMETER     :: z02=Z01                       ! roughness length parameter (see: Thesis of B. Marticorena)

   REAL(dp), PARAMETER     :: cuscale=0.86_dp               ! Scale factor for wind stress threshold
                                                            ! to adjust dust emission occurence (Tegen et al., 2004)
  
! *** Parameters to adjust the emission size-distribution:

   ! Bin boundaries:

   INTEGER, PARAMETER :: min_ai = 1
   INTEGER, PARAMETER :: max_ai = 1

   INTEGER, PARAMETER :: min_ci = 2
   INTEGER, PARAMETER :: max_ci = 4 

   ! Source mass median radii [m]:

   REAL(dp), PARAMETER :: mmr_ai = 0.35E-6_dp
   REAL(dp), PARAMETER :: mmr_ci = 1.75E-6_dp

   ! Source standard deviations:
    
   REAL(dp), PARAMETER :: sig_ai = 1.59_dp
   REAL(dp), PARAMETER :: sig_ci = 2.00_dp

! *** dust emssion scheme related variables
   REAL(dp), ALLOCATABLE    :: biome(:,:)            ! biome distribution 

   REAL(dp)                 :: c_eff                 ! fraction efficiency

   REAL(dp)                 :: dustsum(4)            ! total dust flux (for different paricle sizes)
   REAL(dp), ALLOCATABLE    :: duststat(:,:,:)       ! total dust flux sum for statistics
   REAL(dp), ALLOCATABLE    :: duststat2(:,:,:)      ! total dust flux sum for statistics

   REAL(dp), ALLOCATABLE    :: flux_6h(:,:,:)        ! 6h flux (for tm3 use)
   REAL(dp), ALLOCATABLE    :: flux_ann(:,:)         ! mean annual flux 
   REAL(dp), ALLOCATABLE    :: flux_a1(:,:)          ! for statistics of small particles
   REAL(dp), ALLOCATABLE    :: flux_a2(:,:)          ! for statistics of small particles
   REAL(dp), ALLOCATABLE    :: flux_a10(:,:)         ! for statistics of small particles

   INTEGER, ALLOCATABLE     :: idust(:,:)            ! dust potential source (1: source, 0: no source)
   
   REAL(dp), ALLOCATABLE    :: k_fpar_eff(:,:)       ! KERNEL effective FPAR

   REAL(dp), ALLOCATABLE    :: mat_s1(:,:)           ! soil type #1 area (in relative percentage)
   REAL(dp), ALLOCATABLE    :: mat_s2(:,:)           ! soil type #2 area (in relative percentage)
   REAL(dp), ALLOCATABLE    :: mat_s3(:,:)           ! soil type #3 area (in relative percentage)
   REAL(dp), ALLOCATABLE    :: mat_s4(:,:)           ! soil type #4 area (in relative percentage)
   REAL(dp), ALLOCATABLE    :: mat_s6(:,:)           ! soil type #6 area (in relative percentage)
   REAL(dp), ALLOCATABLE    :: mat_psrc(:,:)         ! preferential source area (in relative percentage)

   REAL(dp)                 :: srel(nats,Nclass)     ! relative surface 
   REAL(dp)                 :: srelV(nats,Nclass)
   REAL(dp)                 :: su_srelV(nats,nclass)
   
   REAL(dp)                 :: Uth(Nclass)           ! threshold friction velocity

END MODULE mo_bgc_dust_mem
