MODULE mo_chem_bcond_io

  ! *mo_chem_bcond_io* holds NetCDF i/o routines for chemical (and other) boundary conditions
  !                    such as emissions, radiative properties for the HAM aerosol model
  !
  ! Authors:
  ! --------
  ! Sebastian Rast, MPI-Met, (i/o routines),     2006
  ! Philip Stier, Caltech,   (module),        06/2006
  !

  USE mo_kind,        ONLY: dp
  USE mo_aero_mem,    ONLY: input
    USE mo_decomposition,     ONLY: gc=>global_decomposition &
                              ,dc=>local_decomposition


  IMPLICIT NONE

  PRIVATE

  PUBLIC :: xt_read_emiss_fields_so2 ! read in emissions
  PUBLIC :: xt_read_emiss_fields_fossil ! read in emissions
  PUBLIC :: xt_read_emiss_fields_bio ! read in emissions
  PUBLIC :: xt_read_emiss_fields_soa ! read in emissions
  PUBLIC :: xt_read_emiss_fields_terr ! read in emissions
  PUBLIC :: xt_read_emiss_fields_sea ! read in emissions
  PUBLIC :: xt_read_emiss_fields_wildfire ! read in emissions
  PUBLIC :: xt_read_emiss_fields_annual ! read in emissions
  PUBLIC :: xt_read_emiss_fields_m1x1 ! read in emissions
  PUBLIC :: xt_read_emiss_fields_m42 ! read in emissions
  PUBLIC :: xt_read_emiss_fields_m63 ! read in emissions
  PUBLIC :: xt_read_emiss_fields_biofuel ! read in emissions
  PUBLIC :: xt_read_emiss_fields_ffires ! read in emissions
  PUBLIC :: xt_read_emiss_fields_so2ann ! read in emissions
  PUBLIC :: xt_read_emiss_fields_bcfoss ! read in emissions


  CONTAINS

  SUBROUTINE xt_read_emiss_fields_so2

    !**** *xt_read_emiss_fields_so2* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_so2* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_nf77_2d

    USE mo_aero_mem,          ONLY: emi_so2_ant_low, emi_so2_ant_high

    INTEGER, PARAMETER              :: nfield=2
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)

    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'EMI_SO2_ANT_LOW'
    yvar_name(2)   = 'EMI_SO2_ANT_HIGH'

    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_nf77_2d ('emiss_aerocom_annual_SO2_ant.nc', 'lon', 'lat',          &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of anthrop. emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_aerocom_annual_SO2_ant.nc: '
    WRITE (nout,*) '      reading EMI_SO2_ANT_LOW and EMI_SO2_ANT_HIGH '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_so2_ant_low(jl,jr)=lfield(jl,1,jr)
      emi_so2_ant_high(jl,jr)=lfield(jl,2,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_so2

  SUBROUTINE xt_read_emiss_fields_fossil

    !**** *xt_read_emiss_fields_fossil* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_fossil* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_ant, emi_oc_ant

    INTEGER, PARAMETER              :: nfield=2
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'OC1ff'
    yvar_name(2)   = 'BC1ff'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_nf77_2d ('emiss_aerocom_annual_fossil.nc', 'lon', 'lat',          &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of BC/OC anthrop. emissions (fossil fuel) -----------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_aerocom_annual_fossil.nc : '
    WRITE (nout,*) '      reading EMI_OC_ANT and EMI_BC_ANT '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_oc_ant(jl,jr)=lfield(jl,1,jr)
      emi_bc_ant(jl,jr)=lfield(jl,2,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_fossil

  SUBROUTINE xt_read_emiss_fields_bio

    !**** *xt_read_emiss_fields_fossil* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_fossil* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_ag, emi_oc_ag

    INTEGER, PARAMETER              :: nfield=2
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'BC1bfuel'
    yvar_name(2)   = 'OC1bfuel'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_nf77_2d ('emiss_aerocom_annual_bio.nc', 'lon', 'lat',          &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of BC/OC agric. waste and fuelwood emiss. -----------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_aerocom_annual_bio.nc: '
    WRITE (nout,*) '      reading EMI_BC_AG and EMI_OC_AG '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_ag(jl,jr)=lfield(jl,1,jr)
      emi_oc_ag(jl,jr)=lfield(jl,2,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_bio
!--------------------------------------------------------------------------------------------------------------------
  SUBROUTINE xt_read_emiss_fields_soa(kmonth)

    !**** *xt_read_emiss_fields_soa* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_soa* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition
    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_oc_bge

    INTEGER, INTENT(IN)      :: kmonth
    CHARACTER(len=256)    :: yvar_name

    INTEGER                           :: ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,dc% nlat))
    ENDIF


    !--- 1) Read monthly emission fields:

    yvar_name   = 'SOA'
    IF (p_parallel_io) THEN
         CALL read_var_hs_nf77_2d ('emiss_aerocom_monthly_SOA.nc', 'lon', 'lat', 'time', kmonth, &
                   TRIM(yvar_name), field(:,:), ierr)
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of OC biogenic emissions ----------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_aerocom_monthly_SOA.nc: '
    WRITE (nout,*) '      reading EMI_OC_BGE                              '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks
     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF
     DO jl=1, nproma
      emi_oc_bge(jl,jr)=lfield(jl,jr)
    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_soa

  SUBROUTINE xt_read_emiss_fields_terr(kmonth)

    !**** *xt_read_emiss_fields_terr* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_terr* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition
    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_dms_ter

    INTEGER, INTENT(IN)      :: kmonth
    CHARACTER(len=256)    :: yvar_name

    INTEGER                           :: ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,dc% nlat))
    ENDIF


    !--- 1) Read monthly emission fields:

    yvar_name   = 'DMS_terr'
    IF (p_parallel_io) THEN
         CALL read_var_hs_nf77_2d ('emiss_aerocom_monthly_dms_terr.nc', 'lon', 'lat', 'time', kmonth, &
                   TRIM(yvar_name), field(:,:), ierr)
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of DMS terrestrial emissions ------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_aerocom_monthly_dms_terr.nc: '
    WRITE (nout,*) '      reading EMI_DMS_TER                  '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks
     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF
     DO jl=1, nproma
      emi_dms_ter(jl,jr)=lfield(jl,jr)
    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_terr

  SUBROUTINE xt_read_emiss_fields_sea(kmonth)

    !**** *xt_read_emiss_fields_terr* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_terr* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition
    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_dms_sea

    INTEGER, INTENT(IN)      :: kmonth
    CHARACTER(len=256)    :: yvar_name

    INTEGER                           :: ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,dc% nlat))
    ENDIF


    !--- 1) Read monthly emission fields:

    yvar_name   = 'DMS_sea'
    IF (p_parallel_io) THEN
         CALL read_var_hs_nf77_2d ('emiss_fields_monthly_dms_sea.nc', 'lon', 'lat', 'time', kmonth, &
                   TRIM(yvar_name), field(:,:), ierr)
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of DMS seawater emissions ------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_fields_monthly_dms_sea.nc: '
    WRITE (nout,*) '      reading EMI_DMS_SEA                  '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks
     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF
     DO jl=1, nproma
      emi_dms_sea(jl,jr)=lfield(jl,jr)
    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_sea

  SUBROUTINE xt_read_emiss_fields_wildfire(ypname,kmonth)

    !**** *xt_read_emiss_fields_wildfire* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_wildfire* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition
    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_3d
    USE mo_exception,         ONLY: finish          ! subroutine to stop execution

    USE mo_aero_mem,          ONLY: emi_bc_wf_3d,  &
                                    emi_oc_wf_3d,  &
                                    emi_so2_wf_3d

    INTEGER, INTENT(IN)             :: kmonth
    INTEGER, PARAMETER              :: nfield=3
    CHARACTER(len=256)              :: yvar_name(nfield)
    CHARACTER(len=*), INTENT(IN)    :: ypname

    INTEGER                           :: i, ierr, jr, jl, jk
    INTEGER                           :: nproma, nlev

    REAL(dp) ,POINTER     :: field(:,:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:,:)

    nlev = dc%nlev

    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,dc% nlev,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:
    IF (nlev == 31) THEN
       yvar_name(1)   = 'gfed_bc_L31'
       yvar_name(2)   = 'gfed_pom_L31'
       yvar_name(3)   = 'gfed_so2_L31'
    ELSE IF (nlev == 19) THEN
       yvar_name(1)   = 'gfed_bc_L19'
       yvar_name(2)   = 'gfed_pom_L19'
       yvar_name(3)   = 'gfed_so2_L19'
    ELSE IF (nlev == 39) THEN
       yvar_name(1)   = 'gfed_bc_L39'
       yvar_name(2)   = 'gfed_pom_L39'
       yvar_name(3)   = 'gfed_so2_L39'
! uh+
    ELSE IF (nlev == 47) THEN
       yvar_name(1)   = 'gfed_bc_L47'
       yvar_name(2)   = 'gfed_pom_L47'
       yvar_name(3)   = 'gfed_so2_L47'

! uh-
    ELSE
       CALL finish ('xt_read_emiss_fields_wildfire', 'Vertical resol.  not supported.')
    ENDIF

    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_3d (TRIM(ypname), 'lon', 'lev', 'lat', 'time', kmonth,         &
                   TRIM(yvar_name(i)), field(:,:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of wildfire emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_aerocom_monthly_wildfire.nc: '
    WRITE (nout,*) '      reading EMI_BC_WF_3D, EMI_OC_WF_3D and EMI_SO2_WF_3D '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF

  ALLOCATE (lfield(dc%nproma, dc% nlev, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF
   DO jk=1, dc% nlev
     DO jl=1, nproma

      emi_bc_wf_3d(jl,jk,jr) =lfield(jl,jk,1,jr)
      emi_oc_wf_3d(jl,jk,jr) =lfield(jl,jk,2,jr)
      emi_so2_wf_3d(jl,jk,jr)=lfield(jl,jk,3,jr)

    ENDDO
   ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_wildfire

!--------------------------------------------------------------------------------------------------------------------
  SUBROUTINE xt_read_emiss_fields_annual

    !**** *xt_read_emiss_fields_annual* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_annual* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_ant, emi_oc_ant

    INTEGER, PARAMETER              :: nfield=2
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'BC_ant'
    yvar_name(2)   = 'OC_ant'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d ('emiss_fields_annual.nc', 'lon', 'lat', 'time', 1,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of BC/OC anthrop. emissions (1x1) -------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_fields_annual.nc : '
    WRITE (nout,*) '      reading EMI_OC_ANT and EMI_BC_ANT '
    WRITE (nout,*) '------------------------------------------------------------------------------'

    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_ant(jl,jr)=lfield(jl,1,jr)
      emi_oc_ant(jl,jr)=lfield(jl,2,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_annual

!--------------------------------------------------------------------------------------------------------------------
  SUBROUTINE xt_read_emiss_fields_biofuel(ypname,kmonth)

    !**** *xt_read_emiss_fields_biofuel* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_biofuel* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_ag

    INTEGER, PARAMETER              :: nfield=1
    CHARACTER(len=256)    :: yvar_name(nfield)
    CHARACTER(len=*), INTENT(IN)    :: ypname

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma
    INTEGER, INTENT(IN)               :: kmonth

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'BF_BC'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d (TRIM(ypname), 'lon', 'lat', 'time', kmonth,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of biofuel emissions (1x1) -------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_nies_monthly_BC_biofuel.nc : '
    WRITE (nout,*) '      reading EMI_BC_AG '
    WRITE (nout,*) '------------------------------------------------------------------------------'

    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_ag(jl,jr)=lfield(jl,1,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_biofuel

  SUBROUTINE xt_read_emiss_fields_ffires(ypname,kmonth)

    !**** *xt_read_emiss_fields_ffires* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_ffires* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_wf

    INTEGER, PARAMETER              :: nfield=1
    CHARACTER(len=256)    :: yvar_name(nfield)
    CHARACTER(len=*), INTENT(IN)    :: ypname

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma
    INTEGER, INTENT(IN)               :: kmonth

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'FF_BC'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d (TRIM(ypname), 'lon', 'lat', 'time', kmonth,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of forestfires emissions (1x1) -------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_nies_monthly_BC_forestfires.nc : '
    WRITE (nout,*) '      reading EMI_BC_WF '
    WRITE (nout,*) '------------------------------------------------------------------------------'

    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_wf(jl,jr)=lfield(jl,1,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_ffires

  SUBROUTINE xt_read_emiss_fields_so2ann(ypname)

    !**** *xt_read_emiss_fields_so2ann* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_so2ann* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_so2_ant_low

    INTEGER, PARAMETER              :: nfield=1
    CHARACTER(len=256)    :: yvar_name(nfield)
    CHARACTER(len=*), INTENT(IN)    :: ypname

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'FFC_SO'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d (TRIM(ypname), 'lon', 'lat', 'time', 1,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of anthrop. emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_nies_annual_SO2_fossil_fuel.nc: '
    WRITE (nout,*) '      reading EMI_SO2_ANT_LOW '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_so2_ant_low(jl,jr)=lfield(jl,1,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_so2ann

  SUBROUTINE xt_read_emiss_fields_bcfoss(ypname)

    !**** *xt_read_emiss_fields_bcfoss* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_bcfoss* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_ant

    INTEGER, PARAMETER              :: nfield=1
    CHARACTER(len=256)    :: yvar_name(nfield)
    CHARACTER(len=*), INTENT(IN)    :: ypname

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'FFC_BC'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d (TRIM(ypname), 'lon', 'lat', 'time', 1,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of anthrop. emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_nies_annual_BC_fossil_fuel.nc: '
    WRITE (nout,*) '      reading EMI_BC_ANT '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_ant(jl,jr)=lfield(jl,1,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_bcfoss

  SUBROUTINE xt_read_emiss_fields_m1x1(kmonth)

    !**** *xt_read_emiss_fields_m1x1* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_m1x1* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_wf,        &
                                    emi_oc_wf,        &
                                    emi_oc_bge,       &
                                    emi_so2_ant_low,  &
                                    emi_so2_ant_high, &
                                    emi_dms_sea

    INTEGER, INTENT(IN)             :: kmonth
    INTEGER, PARAMETER              :: nfield=6
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'BC_wf'
    yvar_name(2)   = 'OC_wf'
    yvar_name(3)   = 'OC_bge'
    yvar_name(4)   = 'SO2_ant_119'
    yvar_name(5)   = 'SO2_ant_118'
    yvar_name(6)   = 'DMS_sea'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d ('emiss_fields_monthly_1x1.nc', 'lon', 'lat', 'time', kmonth,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_fields_monthly_1x1.nc: '
    WRITE (nout,*) '      reading EMI_BC_WF, EMI_OC_WF, EMI_OC_BGE, EMI_SO2_ANT_LOW,       '
    WRITE (nout,*) '              EMI_SO2_ANT_HIGH and EMI_DMS_SEA       '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_wf(jl,jr)       =lfield(jl,1,jr)
      emi_oc_wf(jl,jr)       =lfield(jl,2,jr)
      emi_oc_bge(jl,jr)      =lfield(jl,3,jr)
      emi_so2_ant_low(jl,jr) =lfield(jl,4,jr)
      emi_so2_ant_high(jl,jr)=lfield(jl,5,jr)
      emi_dms_sea(jl,jr)     =lfield(jl,6,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_m1x1

  SUBROUTINE xt_read_emiss_fields_m42(kmonth)

    !**** *xt_read_emiss_fields_m42* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_m42* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_dms_ter

    INTEGER, INTENT(IN)             :: kmonth
    INTEGER, PARAMETER              :: nfield=1
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'DMS_terr'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d ('emiss_fields_monthly_T42.nc', 'lon', 'lat', 'time', kmonth,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_fields_monthly_T42.nc: '
    WRITE (nout,*) '      reading EMI_DMS_TER       '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_dms_ter(jl,jr)     =lfield(jl,1,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_m42

  SUBROUTINE xt_read_emiss_fields_m63(kmonth)

    !**** *xt_read_emiss_fields_m63* reads prescribed emission fields
    !
    !    Authors:
    !    -----------
    !    Sebastian Rast, MPI-Met, 2006
    !
    !**  Interface:
    !    -----------
    !    *xt_read_emiss_fields_m63* is called from *call_submodels*
    !

    USE mo_doctor,            ONLY: nout
    USE mo_mpi,               ONLY: p_parallel_io
    USE mo_decomposition,     ONLY: gc => global_decomposition,   &
                                    dc => local_decomposition

    USE mo_transpose,         ONLY: scatter_gp
    USE mo_read_netcdf77,     ONLY: read_var_hs_nf77_2d

    USE mo_aero_mem,          ONLY: emi_bc_ag, emi_oc_ag

    INTEGER, INTENT(IN)             :: kmonth
    INTEGER, PARAMETER              :: nfield=2
    CHARACTER(len=256)    :: yvar_name(nfield)

    INTEGER                           :: i, ierr, jr, jl
    INTEGER                           :: nproma

    REAL(dp) ,POINTER     :: field(:,:,:)
    REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)


    IF (p_parallel_io) THEN
       ALLOCATE (field (dc% nlon,nfield,dc% nlat))
    ENDIF


    !--- 1) Read annual emission fields:

    yvar_name(1)   = 'BC'
    yvar_name(2)   = 'OC'
    IF (p_parallel_io) THEN
       DO i=1, nfield
         CALL read_var_hs_nf77_2d ('emiss_fields_monthly_T63.nc', 'lon', 'lat', 'time', kmonth,     &
                   TRIM(yvar_name(i)), field(:,i,:), ierr)
       END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of emissions -------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read emiss_fields_monthly_T63.nc: '
    WRITE (nout,*) '      reading EMI_BC_AG and EMI_OC_AG       '
    WRITE (nout,*) '------------------------------------------------------------------------------'
    END IF
  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      emi_bc_ag(jl,jr)     =lfield(jl,1,jr)
      emi_oc_ag(jl,jr)     =lfield(jl,2,jr)

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_parallel_io) DEALLOCATE (field)
  RETURN

  END SUBROUTINE xt_read_emiss_fields_m63

END MODULE mo_chem_bcond_io
