MODULE mo_chem_bcond_nies
  !
  !*mo_chem_bcond_nies* handles all emissions tracers specific for nies settings
  !
  ! Authors:
  ! -------- Silvia Kloster , MPI, November 2003
  !
  ! Method: adopted from mo_chem_bcond_aerocom
  !        

  USE mo_kind,             ONLY: dp
  
  IMPLICIT NONE 

  !-- Emission and converion factors
  !-- 
  REAL(dp)    ::      fac_bc2oc_ffc    = 1._dp,    & ! fossil fuel combustion converion BC to OC (J. Penner personal com.)
                      fac_bc2oc_bfc    = 4._dp,    & ! bio fuel combustion covnsercion Bc to OC (AEROCOM)
                      fac_bc2om_ff     = 11.1_dp , & ! forest fires conversion BC to OM  /// scaling of GFED product from 1997-2002
                      fac_bc2so2_ff    = 1.28_dp , & ! forest fires conversion BC to SO2 /// scaling of GFED product from 1997-2002
                      fac_vol          = 1.0_dp  , & ! NIES emission  == AEROCOM emission
                      fac_so2_high     = 0.88_dp     ! according to AEROCOM (high :  industry + powerplant + shipping  
                                                  ! low :   Domestic + road + off_road )

CONTAINS

  SUBROUTINE convert_volc_emission

    USE mo_aero_mem, ONLY : emi_so2_evol, emi_so2_cvol

    USE mo_decomposition, ONLY: ldc=>local_decomposition
    
    IMPLICIT NONE
    
    INTEGER :: ngpblks, nproma
    
    INTEGER :: jl, jrow

    ngpblks = ldc% ngpblks ! number of rows
    
    DO jrow = 1, ngpblks          ! local number of rows
       
       IF ( jrow == ldc% ngpblks ) THEN
          nproma = ldc% npromz
       ELSE
          nproma = ldc% nproma
       END IF
       
       DO jl = 1, nproma 
          emi_so2_evol(jl,jrow) = emi_so2_evol(jl,jrow) * fac_vol
          emi_so2_cvol(jl,jrow) = emi_so2_cvol(jl,jrow) * fac_vol
       ENDDO
    ENDDO
    
  END SUBROUTINE convert_volc_emission

  SUBROUTINE convert_nies_emissions_monthly
    
    USE mo_aero_mem, ONLY : emi_bc_ag , emi_oc_ag , &
                            emi_bc_wf , emi_oc_wf, emi_so2_wf
    
    USE mo_decomposition, ONLY: ldc=>local_decomposition

    USE mo_aero         , ONLY :  so2ts, zom2oc 
    
    IMPLICIT NONE
    
    INTEGER :: ngpblks, nproma
    
    INTEGER :: jl, jrow

    REAL(dp) :: ztemp

    ngpblks = ldc% ngpblks ! number of rows
    
    DO jrow = 1, ngpblks          ! local number of rows
       
       IF ( jrow == ldc% ngpblks ) THEN
          nproma = ldc% npromz
       ELSE
          nproma = ldc% nproma
       END IF
       
       DO jl = 1, nproma 

          emi_oc_ag(jl,jrow) = emi_bc_ag(jl,jrow)* fac_bc2oc_bfc
          emi_oc_wf(jl,jrow) = emi_bc_wf(jl,jrow)* fac_bc2om_ff
          emi_so2_wf(jl,jrow) = emi_bc_wf(jl,jrow)* fac_bc2so2_ff
          
       ENDDO
       
    ENDDO
    
  END SUBROUTINE convert_nies_emissions_monthly



  SUBROUTINE convert_nies_emissions_annual
    
    USE mo_aero_mem, ONLY : emi_bc_ant, emi_oc_ant, &
                            emi_so2_ant_low, emi_so2_ant_high

    USE mo_decomposition, ONLY: ldc=>local_decomposition

    USE mo_aero         , ONLY :  so2ts, zom2oc 
    
    IMPLICIT NONE
    
    INTEGER :: ngpblks, nproma
    
    INTEGER :: jl, jrow

    REAL(dp) :: ztemp

    ngpblks = ldc% ngpblks ! number of rows
    
    DO jrow = 1, ngpblks          ! local number of rows
       
       IF ( jrow == ldc% ngpblks ) THEN
          nproma = ldc% npromz
       ELSE
          nproma = ldc% nproma
       END IF
       
       DO jl = 1, nproma 

          emi_oc_ant(jl,jrow) = emi_bc_ant(jl,jrow)* fac_bc2oc_ffc 

          ztemp=emi_so2_ant_low(jl,jrow)
          
          emi_so2_ant_low(jl,jrow)  = ztemp * (1._dp-fac_so2_high) * 1._dp/so2ts
          emi_so2_ant_high(jl,jrow) = ztemp * fac_so2_high      * 1._dp/so2ts
          
       ENDDO
       
    ENDDO
    
  END SUBROUTINE convert_nies_emissions_annual
  
  
  SUBROUTINE so2_emissions_nies(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)

    USE mo_tracer,        ONLY: ntrac
    USE mo_aero_trac,     ONLY: idt_so2,                         &
                                idt_ms4ks,    idt_ms4as ,        &
                                idt_ms4cs,                       &
                                idt_nas   ,    idt_nks ,         &
                                idt_ncs  
    USE mo_aero_mem,      ONLY: d_emi_ms,                        &
                                d_emi_so2_wf, d_emi_so2_ant,     &
                                emi_so2_wf,                      &
                                d_emi_so4_wf,  d_emi_so4_ant,    &
                                emi_so2_ant_low, emi_so2_ant_high
    USE mo_control,       ONLY: nlev
    USE mo_time_control,  ONLY: delta_time
    USE mo_constants,     ONLY: g
    USE mo_exception,     ONLY: finish
    USE mo_aero,          ONLY: so2ts,facso2,                    &             
                                zm2n_s4ks_sk, zm2n_s4as_sa ,     &
                                zm2n_s4cs_sc

    IMPLICIT NONE

    !--- Parameters:

    INTEGER, INTENT(IN)      :: kproma, kbdim, klev, klevp1, krow

    REAL(dp),    INTENT(in)      :: paphp1(kbdim,klevp1)
    REAL(dp),    INTENT(inout)   :: pxtems(kbdim,ntrac)
    REAL(dp),    INTENT(inout)   :: pxtte(kbdim,klev,ntrac)

    !--- Local Variables:

    REAL(dp)                     :: zgdp

    INTEGER                  :: jl, jk, ilev


    !--- 0) Initializations: ----------------------------------------------

    IF (nlev/=19 .AND. nlev/=31) CALL finish('so2_emissions','missmatch with number of vertical levels')

  
    DO jl=1,kproma
       !--- 1) Surface emission      
       !       anthropogenic emissions lowest model layer : Domestic + road + off_road

       pxtems(jl,idt_so2)= pxtems(jl,idt_so2)+emi_so2_ant_low(jl,krow)*so2ts*facso2

       
       !--- primary SO4 emissions 

       !--- 50% Aitken

       pxtems(jl,idt_ms4ks)=pxtems(jl,idt_ms4ks) + &
                            0.5_dp*emi_so2_ant_low(jl,krow)*so2ts*(1-facso2)

       
       pxtems(jl,idt_nks)  =pxtems(jl,idt_nks)+ &
                            0.5_dp*emi_so2_ant_low(jl,krow)*so2ts*(1-facso2)* zm2n_s4ks_sk

       !--- 50% Accumultaion

       pxtems(jl,idt_ms4as)=pxtems(jl,idt_ms4as) + &
                            0.5_dp*emi_so2_ant_low(jl,krow)*so2ts*(1-facso2)

       pxtems(jl,idt_nas)  =pxtems(jl,idt_nas)+ &
                            0.5_dp*emi_so2_ant_low(jl,krow)*so2ts*(1-facso2)* zm2n_s4as_sa

       !--- Store for diagnostics:

       d_emi_so2_ant(jl,krow)= d_emi_so2_ant(jl,krow) + emi_so2_ant_low(jl,krow)*so2ts*facso2    *delta_time
       d_emi_so4_ant(jl,krow)= d_emi_so4_ant(jl,krow) + emi_so2_ant_low(jl,krow)*so2ts*(1-facso2)*delta_time
       d_emi_ms(jl,krow)     = d_emi_ms(jl,krow)      + emi_so2_ant_low(jl,krow)*so2ts           *delta_time  
    
    END DO

    !--- 2) Emission of anthropogenic SO2 in the second lowest model level as this 
    !       corresponds to the next higher level above 100m 
    !       Note: While surface emissions are specified as fluxes, these
    !       emissions are applied as tracer tendencies in the 
    !       respective model layer industrial
    !    

    ilev=nlev-1

    DO jl=1, kproma

       zgdp=g/(paphp1(jl,ilev+1)-paphp1(jl,ilev))

       pxtte(jl,ilev,idt_so2)=pxtte(jl,ilev,idt_so2) +  &
                              emi_so2_ant_high(jl,krow)*so2ts*facso2*zgdp

       !--- Primary SO4 emissions

       !--- 50% Accumulation
       
       pxtte(jl,ilev,idt_ms4as)=pxtte(jl,ilev,idt_ms4as) +  &
                             0.5_dp*emi_so2_ant_high(jl,krow)*so2ts*(1-facso2)*zgdp
       
       pxtte(jl,ilev,idt_nas)=pxtte(jl,ilev,idt_nas) + &
                               0.5_dp*emi_so2_ant_high(jl,krow)*so2ts*(1-facso2)*zm2n_s4as_sa*zgdp 
       
       !--- 50% Coarse  

       pxtte(jl,ilev,idt_ms4cs)=pxtte(jl,ilev,idt_ms4cs) +  &
                             0.5_dp*emi_so2_ant_high(jl,krow)*so2ts*(1-facso2)*zgdp
       
       pxtte(jl,ilev,idt_ncs)=pxtte(jl,ilev,idt_ncs) + &
                               0.5_dp*emi_so2_ant_high(jl,krow)*so2ts*(1-facso2)*zm2n_s4cs_sc*zgdp 

       !--- Add emissions to total sulfur emissions:

       d_emi_ms(jl,krow)     = d_emi_ms(jl,krow)      + emi_so2_ant_high(jl,krow)*so2ts           *delta_time 
       d_emi_so2_ant(jl,krow)= d_emi_so2_ant(jl,krow) + emi_so2_ant_high(jl,krow)*so2ts*facso2    *delta_time
       d_emi_so4_ant(jl,krow)= d_emi_so4_ant(jl,krow) + emi_so2_ant_high(jl,krow)*so2ts*(1-facso2)*delta_time

       
       !--- 3) Biomass burning : All emission assumed at the surface NIES scenario
       
       pxtems(jl,idt_so2)=pxtems(jl,idt_so2) +  &
                               emi_so2_wf(jl,krow)*so2ts*facso2

       !--- Primary SO4 emissions
       
       !--- 50% Aitken
       
       pxtems(jl,idt_ms4ks)=pxtems(jl,idt_ms4ks) +  &
                             0.5_dp*emi_so2_wf(jl,krow)*so2ts*(1-facso2)
       
       pxtems(jl,idt_nks)=pxtems(jl,idt_nks) + &
                               0.5_dp*emi_so2_wf(jl,krow)*so2ts*(1-facso2)*zm2n_s4ks_sk

       !--- 50% Accumulation
       
       pxtems(jl,idt_ms4as)=pxtems(jl,idt_ms4as) +  &
                             0.5_dp*emi_so2_wf(jl,krow)*so2ts*(1-facso2)
       
       pxtems(jl,idt_nas)=pxtems(jl,idt_nas) + &
                             0.5_dp*emi_so2_wf(jl,krow)*so2ts*(1-facso2)*zm2n_s4as_sa
          
       !--- Store for diagnostics:
       
       d_emi_so2_wf(jl,krow) = d_emi_so2_wf(jl,krow) +&
                                emi_so2_wf(jl,krow)*so2ts*facso2*delta_time

       d_emi_so4_wf(jl,krow) = d_emi_so4_wf(jl,krow) +&
                                emi_so2_wf(jl,krow)*so2ts*(1-facso2)*delta_time
          
       !--- Add emissions to total sulfur emissions
       
       d_emi_ms(jl,krow)     = d_emi_ms(jl,krow)      + emi_so2_wf(jl,krow)*so2ts           *delta_time 
       d_emi_so2_ant(jl,krow)= d_emi_so2_ant(jl,krow) + emi_so2_wf(jl,krow)*so2ts*facso2    *delta_time
       d_emi_so4_ant(jl,krow)= d_emi_so4_ant(jl,krow) + emi_so2_wf(jl,krow)*so2ts*(1-facso2)*delta_time

       
    END DO
    
  END SUBROUTINE so2_emissions_nies
  
  SUBROUTINE carbon_emissions_nies(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)
    
    USE mo_tracer,        ONLY: ntrac, flag
    USE mo_constants,     ONLY: api
    USE mo_time_control,  ONLY: delta_time
    USE mo_constants,     ONLY: g     
    USE mo_geoloc,        ONLY: philat_2d
    USE mo_aero,          ONLY: zm2n_bcki_ff, zm2n_bcki_bb, &
                                zm2n_bcks_bb, zm2n_ocki_ff, &
                                zm2n_ocki_bb, zm2n_ocki_bg, &
                                zm2n_ocks_bb, zm2n_ocks_bg, &
                                zom2oc,                     &
                                zbb_wsoc_perc,zbge_wsoc_perc
    USE mo_aero_trac,     ONLY: idt_mbcki,    idt_mbcks,           &
                                idt_mocki,    idt_mocks,           &
                                idt_mocas,                         &
                                idt_nki,      idt_nks     
    USE mo_aero_mem,      ONLY: d_emi_mbc,    d_emi_moc,           &
                                d_emi_bc_ant, d_emi_bc_wf,      &
                                d_emi_bc_ag,  d_emi_oc_ant,        &
                                d_emi_oc_bge, d_emi_oc_wf,      &
                                d_emi_oc_ag,                       &
                                emi_bc_ant,   emi_bc_wf,        &
                                emi_bc_ag,    emi_oc_ant,          &
                                emi_oc_bge,   emi_oc_wf,        &
                                emi_oc_ag

    IMPLICIT NONE

    !--- Parameters:
    
    INTEGER, INTENT(in)      :: kproma,klev,klevp1,kbdim,krow
    REAL(dp),    INTENT(in)      :: paphp1(kbdim,klevp1)
    REAL(dp),    INTENT(inout)   :: pxtems(kbdim,ntrac)
    REAL(dp),    INTENT(inout)   :: pxtte(kbdim,klev,ntrac)

    !--- Local Variables:

    INTEGER                 :: jl


    !--- 1) Apply the emissions:

    DO jl=1, kproma

       !--- 1.1) AEROCOM no seasonality !

       !--- 1.2) Carbon mass flux:

       !--- Black Carbon:

       !    Anthropogenic:
 
       pxtems(jl,idt_mbcki)=pxtems(jl,idt_mbcki)+ &
                            emi_bc_ant(jl,krow)

       pxtems(jl,idt_nki)  =pxtems(jl,idt_nki)+ &
                            emi_bc_ant(jl,krow) * zm2n_bcki_ff

       !    Add agricultural waste and fuelwood:

       pxtems(jl,idt_mbcki)=pxtems(jl,idt_mbcki)+ &
                            emi_bc_ag(jl,krow)

       pxtems(jl,idt_nki)  =pxtems(jl,idt_nki)+ &
                            (emi_bc_ag(jl,krow)) * zm2n_bcki_bb

       !    Store emitted mass for diagnostics:

       d_emi_bc_ant(jl,krow)=d_emi_bc_ant(jl,krow)+ &
                             emi_bc_ant(jl,krow)*delta_time     
       d_emi_bc_ag(jl,krow) =d_emi_bc_ag(jl,krow) + &
                             emi_bc_ag(jl,krow)*delta_time
       d_emi_mbc(jl,krow)   =d_emi_mbc(jl,krow)   + &
                             (emi_bc_ant(jl,krow)+emi_bc_ag(jl,krow))*delta_time        
       
       !--- Organic Carbon:

       !    Anthropogenic:
 
       pxtems(jl,idt_mocki)=emi_oc_ant(jl,krow)*zom2oc

       pxtems(jl,idt_nki)  =pxtems(jl,idt_nki)+ &
                            emi_oc_ant(jl,krow)*zom2oc * zm2n_ocki_ff

       !    Add  agricultural waste and fuelwood - insoluble fraction:

       pxtems(jl,idt_mocki)=pxtems(jl,idt_mocki)+ &
                            (emi_oc_ag(jl,krow))*zom2oc*(1._dp-zbb_wsoc_perc)

       pxtems(jl,idt_nki)  =pxtems(jl,idt_nki)+ &
                            (emi_oc_ag(jl,krow))*zom2oc*(1._dp-zbb_wsoc_perc)*zm2n_ocki_bb

       !    Add  agricultural waste and fuelwood - soluble fraction:

       pxtems(jl,idt_mocks)=pxtems(jl,idt_mocks)+ &
                            (emi_oc_ag(jl,krow))*zom2oc*zbb_wsoc_perc

       pxtems(jl,idt_nks)  =pxtems(jl,idt_nks)+ &
                            (emi_oc_ag(jl,krow))*zom2oc*zbb_wsoc_perc * zm2n_ocks_bb

       !    Add Biogenic - insoluble fraction:

       pxtems(jl,idt_mocki)=pxtems(jl,idt_mocki)+ &
                            emi_oc_bge(jl,krow)*(1._dp-zbge_wsoc_perc)

       pxtems(jl,idt_nki)  =pxtems(jl,idt_nki)+ &
                            emi_oc_bge(jl,krow)*(1._dp-zbge_wsoc_perc) * zm2n_ocki_bg

       !    Add Biogenic - soluble fraction:

       !--- 50% Aitken mode

       pxtems(jl,idt_mocks)=pxtems(jl,idt_mocks)+ &
            0.5_dp*emi_oc_bge(jl,krow)*zbge_wsoc_perc
       
       !--- 50% Accumulation mode
       
       pxtems(jl,idt_mocas)=pxtems(jl,idt_mocas)+ &
                            0.5_dp*emi_oc_bge(jl,krow)*zbge_wsoc_perc

       !    Store emitted mass for diagnostics:

       d_emi_oc_ant(jl,krow)=d_emi_oc_ant(jl,krow)+ &
                             emi_oc_ant(jl,krow)*zom2oc             * delta_time    
       d_emi_oc_ag(jl,krow) =d_emi_oc_ag(jl,krow) + &
                             emi_oc_ag(jl,krow)*zom2oc              * delta_time
       d_emi_oc_bge(jl,krow)=d_emi_oc_bge(jl,krow)+ &
                             emi_oc_bge(jl,krow)                    * delta_time
       d_emi_moc(jl,krow)   =d_emi_moc(jl,krow)   + &
                             ( emi_oc_ant(jl,krow)*zom2oc + emi_oc_ag(jl,krow)*zom2oc + emi_oc_bge(jl,krow)  ) * delta_time

       ! BIOMASS BURNING EMISSIONS
       
       ! BLACK CARBON
       
       pxtems(jl,idt_mbcki)=pxtems(jl,idt_mbcki) +  &
                               emi_bc_wf(jl,krow)
          
       pxtems(jl,idt_nki)=pxtems(jl,idt_nki) + &
                             emi_bc_wf(jl,krow) * zm2n_bcki_bb 
          
       !--- store for diagnostic
       d_emi_bc_wf(jl,krow)=d_emi_bc_wf(jl,krow) + &
                            emi_bc_wf(jl,krow)*delta_time
          
       !--- update total mass
       d_emi_mbc(jl,krow)     =d_emi_mbc(jl,krow)   + &
                               emi_bc_wf(jl,krow)*delta_time          
          
       ! ORGANIC CARBON
       
       !--- organic carbon insoluble fraction !!!

       pxtems(jl,idt_mocki)=pxtems(jl,idt_mocki) + &
                               emi_oc_wf(jl,krow)*(1._dp-zbge_wsoc_perc) 

          
       pxtems(jl,idt_nki) =pxtems(jl,idt_nki)   + &
                              emi_oc_wf(jl,krow)*(1._dp-zbb_wsoc_perc)*zm2n_ocki_bb 
          
          !--- organic carbon soluble fraction !!!

       pxtems(jl,idt_mocks)=pxtems(jl,idt_mocks) + &
                              emi_oc_wf(jl,krow)*zbb_wsoc_perc 

       pxtems(jl,idt_nks) =pxtems(jl,idt_nks)   + &
                              emi_oc_wf(jl,krow)*zbb_wsoc_perc*zm2n_ocks_bb 
          
       !--- diagnostic !!
       
       d_emi_oc_wf(jl,krow) =d_emi_oc_wf(jl,krow) + &
                                emi_oc_wf(jl,krow)    * delta_time
          
          !--- update total mass!!!

       d_emi_moc(jl,krow)   =d_emi_moc(jl,krow)   + &
                                emi_oc_wf(jl,krow)    * delta_time
    END DO
  END SUBROUTINE carbon_emissions_nies
END MODULE mo_chem_bcond_nies
