MODULE mo_xtsurf_drydep
  !----------------------------------------------------------------------
  ! This module handles all input terms required for the calculations
  ! of the surface trace gas and aerosol dry deposition. Most of 
  ! parameters are vegetation and soil data derived from satellite data
  ! a high-resolution geograhical databases. For more details see the
  ! routine where the actual reading occurs. The data are monthly mean
  ! values, whenever there is an annual cycle in the data. In this 
  ! module, there is also the initialisation of the surface resistances
  ! used for the dry deposition calculations.
  ! 
  ! Original by: Martin Schultz and Hans-Stefan Bauer, MPIfM,    07/2000
  ! Modified by: Laurens Ganzeveld and Andreas Rhodin, October,     2001
  !              Philip Stier,                                 2002-2006
  !---------------------------------------------------------------------
  USE mo_kind,          ONLY: dp
  USE mo_decomposition, ONLY: dc => local_decomposition
  USE mo_tracer,        ONLY: ntrac
  USE mo_parameters,    ONLY: jptrac 
  USE mo_aero_mem,      ONLY: vdrydep, rwet, densaer

  IMPLICIT NONE
  !----------------
  ! Public entities
  !----------------
  PRIVATE
  PUBLIC :: xtsurf_init_drydep ! read xtsurf inputdata
  PUBLIC :: xtsurf_calcra      ! calculation aerodynamic resistance
  PUBLIC :: xtsurf_vdbl        ! 'big leaf' gas-phase dry deposition scheme
  PUBLIC :: xtsurf_vdaer       ! 'big leaf' aerosol dry deposition scheme

  !-----------------
  ! Private entities
  !-----------------

  ! ------------------------------------------------------------------------
  ! mz_LG_20020115 Section in which the surface cover data used for the trace gas and
  !     aerosol exchange processes are being declared, e.g. Leaf Area Index,
  !     canopy height, etc.

  ! mz_LG_20020115 declaration of some parameters

  ! mz_LG-20020115 declaration of the resistances, being used in the dry
  !     deposition routines, for details see Vd_bl. The declaration
  !     uses the parameter JPTRAC, which is the maximum number of
  !     tracers, determined in the module mo_parameter. Is still must be
  !     checked if this has to be replaced by diff(ntrac), but then a
  !     compilation error occurs (2002-05)

  REAL(dp), PUBLIC:: diff(jptrac),   diffrb(jptrac), &
                 rmes(jptrac),   rcut(jptrac),   &
                 rsoil(jptrac),  rws(jptrac),    &
                 rwater(jptrac), rsnow(jptrac)

  ! mz_LG_20020115 declaration and setting of some switches used in the routines
  !     for surface trace gas and aerosol exchange processes. This switches
  !     should be included in the tracer namelist so that they can be defined
  !     in the jobscript, or similar to the definition of the dry deposition
  !     switch ndrydep (see mo_def.f90)

  ! mz_LG_20020131, some aerosol dry deposition switches (see subroutine
  !     mz_xtsurf_vdaeror a more extensive explanation of these switches)

  LOGICAL, PARAMETER :: lwhitecap      = .TRUE.  ! switch for considering role of whitecaps
  LOGICAL, PARAMETER :: ldr_rh         = .FALSE. ! switch for considering relative hum. effect
  LOGICAL, PARAMETER :: lvd_integrated = .FALSE. ! switch for the integrated / non-integrated
                                                 ! dry deposition scheme. 
                                                 ! Note: Only the non-integrated scheme has been
                                                 !       tested carefully and optimized!
                                                 !       Usage of the integrated scheme currently 
                                                 !       NOT recomended!

CONTAINS

  !==============================================================================
  ! Public Routines
  !==============================================================================
  
  SUBROUTINE xtsurf_init_drydep
    !-------------------------------------------------------------
    ! This subroutine reads the input files containing data 
    ! required for the calculations of the surface trace gas and 
    ! aerosol exchange processes  
    !-------------------------------------------------------------
    USE mo_time_control,      ONLY: lresume, lstart

    ! mz_LG_20020115 calling of routines to read files containing  
    !     data for a whole year (or annual mean or 12 monthly mean
    !     fields).This is only done at the start of the simulations

    IF (lresume .OR. lstart) THEN

      ! mz_LG_20020115 call to subroutine in which the surface resistances,
      !     used for the dry deposition calculations, are initialized. 

      CALL calc_rs

    ENDIF

  END SUBROUTINE xtsurf_init_drydep
  !=============================================================================
SUBROUTINE calc_rs
  !-----------------------------------------------------------------------------
  ! subroutine calc_rs, to calculate the values of the uptake resistances 
  ! required to calculate the trace gas dry deposition velocity. this routine 
  ! is based on an approach by wesely, 1989, in which the uptake resistances of
  ! trace gases, for which the dry deposition velocities have not been observed,
  ! are estimated based on the henry coefficient and a reactivity coefficient and
  ! the uptake resistances of so2 and o3, of the "big leaf" dry deposition scheme
  ! by ganzeveld and j. lelieveld j. geophys. res., 100, 20,999-21,012,1995,
  ! ganzeveld et al.,j. geophys. res., 103, 5679-5694, 1998 and ganzeveld et al, 
  ! submitted to j. geophys. res., 2001. for more information of the wesely 
  ! approach see atmospheric environment vol 23, no 6, 1293-1304.
  !
  ! the program needs as input data the molecular mass of the defined trace 
  ! gases, the henry coefficient [m atm-1] and an estimated reactivity 
  ! coefficient which has 3 distinct values: 0 for non-reactive species, 
  ! (e.g, so2, acetaldehyde), 0.1 for moderately reactive species (e.g., pan),
  ! and 1 for reactive species (e.g., o3, hno3). these values are defined in
  ! the module mo_xtsurf_def
  !-----------------------------------------------------------------------------
  !   Author:
  !   -------
  !   Laurens Ganzeveld, MPI Mainz                                   2001
  !
  !   Modifications:
  !   --------------
  !   Laurens Ganzeveld, MPI Mainz and
  !   Philip Stier,      MPI Hamburg (implementation in ECHAM5) 2001-2002

  USE mo_tracer, ONLY: ntrac, trlist, GAS, get_tracer

  IMPLICIT NONE

  ! declarations

  INTEGER :: jt

  INTEGER :: idt_so2, idt_so4, idt_o3, idt_hno3, &
             idt_no,  idt_no2

  REAL(dp)    :: diffrb_so2, rsoil_so2, rwater_so2, rws_so2,  &
             rsnow_so2,  rmes_so2,  rcut_so2,   diff_so2, &
             diffrb_o3,  rsoil_o3,  rwater_o3,  rws_o3,   &
             rsnow_o3,   rmes_o3,   rcut_o3,    diff_o3

  LOGICAL :: lo_derived

  ! mz_lg_20020115 definition of the different resistances for the 6 
  !     species of the original dry deposition scheme:
  !     ------------o3, hno3, no, no2, so2 and so4--------------

  ! mz_lg_20020115 definition of terms which correct for the diffusivity 
  !     for the computation of the boundary layer resistance, (v/dx)**2/3
  !     v = 0.189 cm2 s-1 (heat) and dx has been calculated according to:
  !     dx=0.212*sqrt(mh2o/mx) with 0.212 being the diffusivity of water 
  !     vapour and m is the molar mass of the component. 

  !--- attribute specific parameters in the following order:
  !
  !    - soil resistance
  !    - sea water resistance, which is generally similar to the wet skin
  !    - wet skin reservoir resistance
  !    - snow resistance
  !    - mesophyll resistance
  !    - cuticle resistance
  !    - diffusivity coefficient, to correct stomatal resistance for 
  !      differences in diffusivity between water vapour and the 
  !      specific trace gas (sqrt(molmass trace gas)/sqrt(molmass h2o))

  !--- Values for SO2 and O3 are required in any case to estimate the 
  !    resistances of the other species considered in the deposition scheme.
  !    => Define hard-linked resistances  for SO2 and O3 even if they 
  !       are not present as tracer.


  diffrb_so2=1.6_dp
  rsoil_so2=250._dp
  rwater_so2=1._dp
  rws_so2=100._dp
  rsnow_so2=1._dp
  rmes_so2=1._dp
  rcut_so2=1.e5_dp
  diff_so2=1.9_dp

  IF(ANY(trlist%ti(:)%basename=='SO2')) THEN

     CALL get_tracer('SO2',idx=idt_so2)

     diffrb(idt_so2)=diffrb_so2
     rsoil(idt_so2)=rsoil_so2
     rwater(idt_so2)=rwater_so2
     rws(idt_so2)=rws_so2
     rsnow(idt_so2)=rsnow_so2
     rmes(idt_so2)=rmes_so2
     rcut(idt_so2)=rcut_so2
     diff(idt_so2)=diff_so2

  END IF

  diffrb_o3=1.2_dp
  rsoil_o3=400._dp
  rwater_o3=2000._dp
  rws_o3=2000._dp
  rsnow_o3=2000._dp
  rmes_o3=1._dp
  rcut_o3=1.e5_dp
  diff_o3=1.6_dp

  IF(ANY(trlist%ti(:)%basename=='O3')) THEN

     CALL get_tracer('O3',idx=idt_o3)
     diffrb(idt_o3)=diffrb_o3
     rsoil(idt_o3)=rsoil_o3
     rwater(idt_o3)=rwater_o3
     rws(idt_o3)=rws_o3
     rsnow(idt_o3)=rsnow_o3
     rmes(idt_o3)=rmes_o3
     rcut(idt_o3)=rcut_o3
     diff(idt_o3)=diff_o3

  END IF

  IF(ANY(trlist%ti(:)%basename=='SO4')) THEN
     CALL get_tracer('SO4','gas',idx=idt_so4)
     diffrb(idt_so4)=1.8_dp
     rsoil(idt_so4)=1.e5_dp
     rwater(idt_so4)=1.e5_dp
     rws(idt_so4)=1.e5_dp
     rsnow(idt_so4)=1.e5_dp
     rmes(idt_so4)=1.e5_dp
     rcut(idt_so4)=1.e5_dp
     diff(idt_so4)=2.7_dp
  END IF

  IF(ANY(trlist%ti(:)%basename=='HNO3')) THEN
     CALL get_tracer('HNO3',idx=idt_hno3)
     diffrb(idt_hno3)=1.4_dp
     rsoil(idt_hno3)=1._dp
     rwater(idt_hno3)=1._dp
     rws(idt_hno3)=1._dp
     rsnow(idt_hno3)=1._dp
     rmes(idt_hno3)=1._dp
     rcut(idt_hno3)=1._dp
     diff(idt_hno3)=1.9_dp
  END IF

  IF(ANY(trlist%ti(:)%basename=='NO')) THEN
     CALL get_tracer('NO',idx=idt_no)
     diffrb(idt_no)=1.1_dp
     rsoil(idt_no)=1.e5_dp
     rwater(idt_no)=1.e5_dp
     rws(idt_no)=1.e5_dp
     rsnow(idt_no)=1.e5_dp
     rmes(idt_no)=500._dp
     rcut(idt_no)=1.e5_dp
     diff(idt_no)=1.3_dp
  END IF

  IF(ANY(trlist%ti(:)%basename=='NO2')) THEN
     CALL get_tracer('NO2',idx=idt_no2)
     diffrb(idt_no2)=1.2_dp
     rsoil(idt_no2)=600._dp
     rwater(idt_no2)=1.e5_dp
     rws(idt_no2)=1.e5_dp
     rsnow(idt_no2)=1.e5_dp
     rmes(idt_no2)=1._dp
     rcut(idt_no2)=1.e5_dp
     diff(idt_no2)=1.6_dp
  END IF

  ! mz_lg_20020115 the resistances are only calculated for species that 
  !     have not been included in the original dry deposition scheme. 

  DO jt=1, ntrac

     lo_derived=trlist%ti(jt)%basename/='SO2'  .AND. &
                trlist%ti(jt)%basename/='SO4'  .AND. &
                trlist%ti(jt)%basename/='O3'   .AND. &
                trlist%ti(jt)%basename/='HNO3' .AND. &
                trlist%ti(jt)%basename/='NO'   .AND. &
                trlist%ti(jt)%basename/='NO2'

     IF (lo_derived .AND. (trlist%ti(jt)%ndrydep==2) .AND. (trlist%ti(jt)%nphase==gas)) THEN

        ! calculation of term which is used to correct the stomatal resistance
        ! for differences in the diffusitivy (see also equation 4).

        diff(jt)=SQRT(trlist%ti(jt)%moleweight/18._dp)

        ! calculation of the term to correct for differences in diffusivity 
        ! between water vapor and the trace gas. it is calculated from: 
        ! diff bl=(v/dx)**2/3, with v=0.189 sm-1 and dx= dh2o/sqrt(mh2o/mx), 
        ! with dh2o=0.212

        diffrb(jt)=(0.189_dp/(0.212_dp/diff(jt)))**(2._dp/3._dp)

        ! calculation of rmx, the mesophyll resistance

        rmes(jt)=1._dp/(trlist%ti(jt)%henry/3000._dp+100._dp*trlist%ti(jt)%dryreac)

        ! calculation of rlux, the cuticular resistance, equation 7 of wesely's
        ! paper

        rcut(jt)=1._dp/(1.e-5_dp*trlist%ti(jt)%henry+trlist%ti(jt)%dryreac)*rcut_o3

        ! calculation of rgsx, the soil resistance, equation 9 of wesely's
        ! paper

        rsoil(jt)=1._dp/(trlist%ti(jt)%henry/(1.e5_dp*rsoil_so2)+ &
             trlist%ti(jt)%dryreac/rsoil_o3)

        ! the snow resistance is similar as the soil resistance

        rsnow(jt)=rsoil(jt)

        ! calculation of rlux-wet, the wet skin resistance, equation 14 of 
        ! wesely's paper

        rws(jt)=1._dp/(1._dp/(3._dp*rws_so2)+1.e-7_dp*trlist%ti(jt)%henry+ &
             trlist%ti(jt)%dryreac/rws_o3) 

        ! calculation of sea uptake resistance, using equation 9 of wesely's
        ! paper

        rwater(jt)=1._dp/(trlist%ti(jt)%henry/(1.e5_dp*rwater_so2)+ &
             trlist%ti(jt)%dryreac/rwater_o3)

     END IF

  ENDDO

END SUBROUTINE calc_rs
!===========================================================================
SUBROUTINE xtsurf_calcra (kproma, kbdim,   klev,    krow,                     &
                          pepdu2, pkap,    pum1,    pvm1,     pgeom1,  pri,   &
                          ptvir1, ptvl,    ptsm1m,  loland,                   &
                          pcdnl,  pcfm,    pcfnc,                             &
                                  paz0w,   paz0i,   paz0l,                    &
                                  prahwat, prahice, prahveg,  prahslsn,       &
                          pustar,                   pustveg,  pustslsn        )

!============================================================================
! LG- 19-10-2001, still need to add the calculation of the rah and pustar 
!                 over land, ice and water, see vdiff.f90 for more details!! 
!============================================================================

  ! LG- Subroutine in which the aerodynamic resistance (Ra) is being 
  !     calculated This resistance is used in the calculation of the 
  !     dry deposition velocity.Ra (and also the friction velocity u*
  !     are being calculated for each surface cover fraction to consider
  !     pronounced differences in the surface roughness   
  !   
  !     Laurens Ganzeveld, 1998 (see for references the papers mentioned
  !     in the dry deposition model Vd_bl), modified for implementation
  !     in echam5, October, 2001
  ! --------------------------------------------------------------------

  ! LG- added the vegetation-emission module

  USE mo_xtsurf_init, ONLY: z0m
  USE mo_constants,      ONLY: g

  IMPLICIT NONE

  INTEGER ::kproma, kbdim, klev, krow, jl

  ! LG- external parameters

  LOGICAL :: loland(kbdim)

  REAL(dp) ::                                                       &
       pcdnl(kbdim),pcfm(kbdim,klev),pcfnc(kbdim),              &
       pum1(kbdim,klev),pvm1(kbdim,klev),pgeom1(kbdim,klev),    &
       pri(kbdim),ptvir1(kbdim,klev),ptvl(kbdim),ptsm1m(kbdim), &
       paz0l(kbdim), paz0w(kbdim), paz0i(kbdim)


  ! LG- internal parameters

  REAL(dp) ::                                                              &
       cm(kbdim),cmveg(kbdim),cmslsn(kbdim),pustar(kbdim),             &
       pustveg(kbdim),pustslsn(kbdim),psih(kbdim),                     &
       prahwat(kbdim), prahice(kbdim), prahveg(kbdim),prahslsn(kbdim), &
       rh(kbdim),zcdnveg(kbdim),zcdnslsn(kbdim)

  ! LG- external constants

  REAL(dp) :: pepdu2,pkap

  ! LG- internal constants

  REAL(dp), PARAMETER :: zkmkh=0.74_dp

  ! LG- local variables

  REAL(dp) :: zsurf,zmonin,zoverl,zxzsurf,zxzref,ztsurf,zes,ze


  DO jl=1,kproma

     ! LG- calculation of drag coefficient and u*. A change with
     !     the previous calculations in echam4 is that the exchange 
     !     coefficients, calculated in echam5 over land, ice and water,
     !     is being used to explicitly calculate the aerodynamic 
     !     resistances over these surface
       
     cm(jl)=pcdnl(jl)*pcfm(jl,klev)/pcfnc(jl)
     pustar(jl)=SQRT(cm(jl))*sqrt(MAX(pepdu2,pum1(jl,klev)**2+ &
          pvm1(jl,klev)**2))

     ! LG- Computation of stability correction term, The stability
     !     correction functions are taken from Stull (page 383-385) 
     !     (08-11-98) and are slightly different from those by Williams 
     !     and Hicks et al., which were originaly being used in the dry 
     !     deposition scheme. 

     zsurf=(pgeom1(jl,klev)/g)
     if (pri(jl).GT.0._dp) then

        ! LG- calculating the Monin-Obukhov lenght directly applying the 
        !     formula given by Stull, 9.7.5k, page 386

!        print *,'pri>0',jl,pustar(jl),ptvir1(jl,klev),ptvl(jl),&
!           SQRT(MAX(pepdu2,pum1(jl,klev)**2+pvm1(jl,klev)**2)),pkap,g

        zmonin=(pustar(jl)*((ptvir1(jl,klev)+ptvl(jl))/2._dp)* &
             SQRT(MAX(pepdu2,pum1(jl,klev)**2+pvm1(jl,klev)**2)))/ &
             (pkap*g*(ptvir1(jl,klev)-ptvl(jl)))
        zoverl=zsurf/zmonin
        psih(jl)=-4.7_dp*zoverl
     else
        zmonin=zsurf/pri(jl)
        zoverl=zsurf/zmonin
        zxzsurf=zkmkh*(1._dp-9._dp*(zoverl))**(0.5_dp)
        zxzref=zkmkh
        psih(jl)= &
             (2._dp*LOG((1._dp+zxzsurf)/2._dp)+LOG((1._dp+zxzsurf**2)/2._dp)-  &
             2._dp*TAN(zxzsurf))-  & ! primitive function value for z 
             (2._dp*LOG((1._dp+zxzref)/2._dp)+LOG((1._dp+zxzref**2)/2._dp)- &
             2._dp*TAN(zxzref))      ! primitive function value for zz
     endif

     !--- Water:

     prahwat(jl)=MAX(1._dp,(1._dp/(pustar(jl)*pkap))* &
                 (LOG((pgeom1(jl,klev)/g)/paz0w(jl))-psih(jl)))

     !--- Ice:

     prahice(jl)=MAX(1._dp,(1._dp/(pustar(jl)*pkap))* &
                 (LOG((pgeom1(jl,klev)/g)/paz0i(jl))-psih(jl)))

     !--- Land:

     IF (loland(jl)) THEN
        zcdnveg(jl)=(pkap/LOG(1._dp+pgeom1(jl,klev)/ &
             (g*MAX(0.02_dp,z0m(jl,krow)))))**2
        zcdnslsn(jl)=(pkap/LOG(1._dp+pgeom1(jl,klev)/ &
             (g*0.005_dp)))**2
        cmveg(jl)=zcdnveg(jl)*pcfm(jl,klev)/pcfnc(jl)
        cmslsn(jl)=zcdnslsn(jl)*pcfm(jl,klev)/pcfnc(jl)
        pustveg(jl)=SQRT(cmveg(jl))*sqrt(MAX(pepdu2, &
             pum1(jl,klev)**2+pvm1(jl,klev)**2))
        pustslsn(jl)=SQRT(cmslsn(jl))*sqrt(MAX(pepdu2, &
             pum1(jl,klev)**2+pvm1(jl,klev)**2))
        prahveg(jl)=MAX(1._dp,(1._dp/(pustveg(jl)*pkap))* &
             (LOG((pgeom1(jl,klev)/g)/ &
             MAX(0.02_dp,z0m(jl,krow)))-psih(jl)))
        prahslsn(jl)=MAX(1._dp,(1._dp/(pustslsn(jl)*pkap))* &
             (LOG((pgeom1(jl,klev)/g)/ &
             0.005_dp)-psih(jl)))
     ELSE
        cmveg(jl)=cm(jl)
        cmslsn(jl)=cm(jl)
        pustveg(jl)=pustar(jl)
        pustslsn(jl)=pustar(jl)
        prahveg(jl)=prahwat(jl)
        prahslsn(jl)=prahwat(jl)
     ENDIF

     ! Computation of relative humidity out T and Tdew (2m) (Monteith)

     ! LG- assigning the temperature, originally the 2 m temperature was
     !     used in the dry deposition scheme, however, since the 2 m
     !     temperature is calculated at the end of this routine and these
     !     particular calculations have been moved to this location,
     !     the surface temperature which is available here is used instead
     !     of the parameter PTEMP2M.

!     ztsurf=MAX(1.,ptsm1m(jl))
!     zes=0.611*EXP(MIN(10.,(19.59*(ztsurf-tmelt)/ztsurf)))
!     ze=0.611*EXP(MIN(10.,(19.59*(1.-tmelt/ &
!          MIN(ztsurf,(tmelt-zfrac*zcvm4)/(1.-zfrac))))))
!     rh(jl)=ze/zes

     ! LG- end

!     print *,'calc_ra, end',jl,prah(jl),prahveg(jl)
  ENDDO

END SUBROUTINE xtsurf_calcra
!===========================================================================
SUBROUTINE xtsurf_vdbl(kproma,   kbdim,     klev,    krow,    laland,  &
                       psrfl,    prco_leaf, pfws,    prh,     pum1,    &
                       pvm1,     pfri,      pcvbs,   pfrw,    pcvs,    &
                       pcvw,     pvgrat,    prahwat, prahice, prahveg, &
                       prahslsn, ptsm,      ustar,   ustveg,  ustslsn  )
  !=====================================================================
  !    Program to calculate the dry deposition velocity for
  !    a selection of trace gases and the sulfate aerosol
  !    according to the "big leaf" approach" using the ECHAM
  !    surface cover characterization and surface parameters
  !    See for more information about the code the papers by
  !    Ganzeveld and Lelieveld, JGR 100, 1995,
  !    Ganzeveld et al., JGR 103, 1998
  !    Ganzeveld et al., 2001
  !=====================================================================
  !
  !   Author:
  !   -------
  !   Laurens Ganzeveld, MPI Mainz                                   2001
  !
  !   Modifications:
  !   --------------
  !   Rolf Sander,       MPI Mainz   (converted to f90)           04/2001
  !   Laurens Ganzeveld, MPI Mainz and
  !   Philip Stier,      MPI Hamburg (implementation in ECHAM5) 2001-2002

  USE mo_tracer,        ONLY: trlist, ntrac, GAS, get_tracer
  USE mo_xtsurf_init,ONLY: lai, hcols, soilpH
  USE mo_constants,     ONLY: tmelt


  IMPLICIT NONE

  ! mz_LG_20020115 ECHAM specific parameter names and some values, so which are already
  !     decleared in the subroutine 'vdiff', where this code is implemented

  INTEGER :: kproma, kbdim, klev, krow

  LOGICAL :: lso2, lso4gas

  ! mz_LG_20020115 external ECHAM parameters

  LOGICAL :: laland(kbdim)

  REAL(dp) :: pum1(kbdim,klev),pvm1(kbdim,klev),psrfl(kbdim),ptsm(kbdim),       &
          prco_leaf(kbdim),pfws(kbdim),prh(kbdim),pcvs(kbdim),pcvw(kbdim),   &
          pvgrat(kbdim),pfri(kbdim),pcvbs(kbdim),pfrw(kbdim)

  ! mz_LG_20020115 more declarations

  REAL(dp) :: rb(kbdim,ntrac),rbveg(kbdim,ntrac),rbslsn(kbdim,ntrac),            &
       rahcan(kbdim),rc0x(kbdim,ntrac),rsveg(kbdim,ntrac),rleaf(kbdim,ntrac), &
       rsoilso2(kbdim),rsnowhno3so2(kbdim),vdveg(kbdim,ntrac),               &
       vdsoil(kbdim,ntrac),vdws(kbdim,ntrac),vdsn(kbdim,ntrac),              &
       vdland(kbdim,ntrac),vdwat(kbdim,ntrac),stheta(kbdim),                 &
       vdso4slsn(kbdim),vdso4wat(kbdim)

  REAL(dp) :: prahwat(kbdim),prahice(kbdim),prahveg(kbdim),prahslsn(kbdim),      &
          ustar(kbdim),ustveg(kbdim), ustslsn(kbdim)

  ! mz_LG_20020115 and some general declarations

  INTEGER :: jl,jt

  INTEGER :: idt_no,  idt_no2, idt_hno3, idt_o3, &
             idt_so2, idt_so4

  REAL(dp) :: fsicm1,fsic,vdvfac

  REAL(dp),            POINTER :: vdrydep_p(:,:)

  !--- 0) Initialisations: 

  IF(ANY(trlist%ti(:)%basename=='SO2'))  CALL get_tracer('SO2',idx=idt_so2)
  IF(ANY(trlist%ti(:)%basename=='SO4'))  CALL get_tracer('SO4','gas',idx=idt_so4)
  IF(ANY(trlist%ti(:)%basename=='O3'))   CALL get_tracer('O3',idx=idt_o3)
  IF(ANY(trlist%ti(:)%basename=='HNO3')) CALL get_tracer('HNO3',idx=idt_hno3)
  IF(ANY(trlist%ti(:)%basename=='NO'))   CALL get_tracer('NO',idx=idt_no)
  IF(ANY(trlist%ti(:)%basename=='NO2'))  CALL get_tracer('NO2',idx=idt_no2)

  !--- Check availability of compounds:

  lso2=   ANY(trlist%ti(:)%basename=='SO2')
  lso4gas=ANY(trlist%ti(:)%basename=='SO4'.AND.trlist%ti(:)%nphase==GAS)

  ! start loop longitude

  DO jl=1,kproma

     ! mz_LG_20020115 calculation of standard deviation of wind direction, used
     !     for the calculation of the SO4-- deposition velocity, this should 
     !     be replaced once by using the Richardson number as criteria to 
     !     select between two stability classes relevant to the SO4-- deposition
     !     calculations.   

     IF (psrfl(jl).GT.10) THEN
        stheta(jl)=MIN(0.5_dp,SQRT(MAX(1.e-5_dp,1._dp/(prahveg(jl)/9._dp+ &
             SQRT(MAX(0.1_dp,pum1(jl,klev)**2+pvm1(jl,klev)**2))))))
     ELSE
        stheta(jl)=MIN(0.5_dp,SQRT(MAX(1.e-5_dp,1._dp/(prahveg(jl)/4._dp+ &
             SQRT(MAX(0.1_dp,pum1(jl,klev)**2+pvm1(jl,klev)**2))))))
     ENDIF

     ! mz_LG_20020115 from now on using LAI instead of the parameter AREA (05-2000),
     !     for calculation of within canopy aerodynamic resistance

     rahcan(jl)=MAX(1._dp,14._dp*lai(jl,krow)*hcols(jl,krow)/ &
                ustveg(jl))

     ! -- rural continental Vd SO4, parameterized deposition velocities
     !    as a function of the friction velocity. The parameterization is 
     !    developped using the observed rural continental SO4 mass size 
     !    distribution [Mehlmann, see Ganzeveld et al., 1998 for reference]

     vdso4slsn(jl)=MAX(1.e-5_dp,0.08_dp-0.57_dp*ustslsn(jl)+ &
          1.66_dp*ustslsn(jl)**2-0.36_dp*ustslsn(jl)**3)

     ! -- Marine Vd SO4 for marine sulfate distribution

     vdso4wat(jl)=MAX(1.e-5_dp,0.12_dp-0.24_dp*ustar(jl)+ &
          5.25_dp*ustar(jl)**2-1.84_dp*ustar(jl)**3)

     ! -- Marine Vd SO4 for rural continental SO4 mass size distribution

     !      vdso4wat(jl)=MAX(1e-5,0.07+0.1*ustar(jl)+
     !     *    2.1*ustar(jl)**2-0.20*ustar(jl)**3)

     ! mz_LG_20020115 calculation of the HNO3 and SO2 snow resistance as a function of the
     !     snow temperature

     rsnowhno3so2(jl)=MIN(MAX(10._dp,10._dp**(-0.09_dp*(ptsm(jl)-tmelt) &
          +2.4_dp)),1.e5_dp)

     ! mz_LG_20020115 calculation of soil resistance of SO2 as a function of the RH
     !     and the soil pH. There are five different soil pH classes and 
     !     each class has a assigned surface resistance, which is subsequently
     !     corrected for the RH

     !   the soil pH classes are:
     !   SOILPH(JL,1,JR) - soil pH <5.5
     !   SOILPH(JL,2,JR) - soil 5.5 <pH <7.3
     !   SOILPH(JL,3,JR) - soil 7.3< pH <8.5
     !   SOILPH(JL,4,JR) - soil 8.5 <pH
     !   SOILPH(JL,5,JR) - soil 4 < pH <8.5

     ! mz_LG_20020115 be aware that the array index has been changed from an 
     !     array structure given by the longitude, latitude and soil pH 
     !     classes to lon., soilpH class and lat. This is done for the 
     !     interface (see module mo_xtsurf_input.f90) !

     rsoilso2(jl)=MAX(50._dp, &
                  soilph(jl,1,krow)*115._dp+ &
                  soilph(jl,2,krow)*65._dp+ &
                  soilph(jl,3,krow)*25._dp+ &
                  soilph(jl,4,krow)*25._dp+ &
                  soilph(jl,5,krow)*70._dp+ &
                  MIN(MAX(0._dp,1000._dp*EXP(MIN(10._dp,269._dp-ptsm(jl)))), &
                  1.e5_dp))

     IF (prh(jl).LT.0.6_dp.AND.prh(jl).GT.0.4_dp) &  ! semi-arid regions
          rsoilso2(jl)=MAX(50._dp,(rsoilso2(jl)*3.41_dp-85._dp)+ &
                       MIN(MAX(0._dp,1000._dp*EXP(MIN(10._dp,269._dp-ptsm(jl)))), &
                       1.e5_dp))
     IF (prh(jl).LE.0.4_dp) &   ! arid regions
          rsoilso2(jl)=MAX(50._dp,MIN(1000._dp,(rsoilso2(jl)*3.41_dp-85._dp+ &
                       ((0.4_dp-prh(jl))/0.4_dp)*1.e5_dp)+ &
                       MIN(MAX(0._dp,1000._dp*EXP(MIN(10._dp,269._dp-ptsm(jl)))), &
                       1.e5_dp)))

  ENDDO

  ! mz_LG_20020115 end

  ! Loop for tracers, for calculation of tracer specific resistances

  DO jt=1,ntrac

     ! mz_LG_20020525, checking for dry deposition calculation, if not, 
     !     skip the calculations

     IF (trlist%ti(jt)%ndrydep.EQ.2.AND.trlist%ti(jt)%nphase.EQ.GAS) THEN

       vdrydep_p => vdrydep(jt)%ptr

       DO jl=1,kproma

        ! mz_LG_20020115 calculation of quasi-laminar boundary layer resistances

        rbveg(jl,jt)=(2._dp/(ustveg(jl)*0.40_dp))*diffrb(jt)
        rbslsn(jl,jt)=(1._dp/(ustslsn(jl)*0.40_dp))*diffrb(jt)
        rb(jl,jt)=(1._dp/(ustar(jl)*0.40_dp))*diffrb(jt)

        ! mz_LG_20020115 Stomatal resistance computation for each species, by 
        !     multiplying the leaf stomatal resistance with the 
        !     diffusivity term

        rc0x(jl,jt)=diff(jt)*prco_leaf(jl)
        rc0x(jl,jt)=rc0x(jl,jt)/MAX(1.e-5_dp,pfws(jl)) ! correcting for soil moisture

        ! mz_LG_20020115  update March 2000, RLEAF instead of RVEG, and RSVEG (surface
        !      resistance for vegetated fraction) instead of RSVEG

        rleaf(jl,jt)=(1._dp/((1._dp/rcut(jt))+(1._dp/(rc0x(jl,jt)+rmes(jt)))))
        rsveg(jl,jt)=(1._dp/((1._dp/(rahcan(jl)+ &
             (1._dp/(ustveg(jl)*0.40_dp))*diffrb(jt)+rsoil(jt)))+ &
             (1._dp/(rleaf(jl,jt)/MAX(1.e-5_dp,lai(jl,krow))))))

        ! mz_LG_20020115  end

      ENDDO

      ! mz_LG_20020115 definition of the NO and NO2 mesophyllic resistance as a function
      !     of the ozone mesophyllic resistances (see Ganzeveld and Lelieveld, '95)

      IF (ANY(trlist%ti(:)%basename.EQ.'O3')) THEN
        IF (ANY(trlist%ti(:)%basename.EQ.'NO').AND. &
             ANY(trlist%ti(:)%basename.EQ.'NO2')) THEN
             DO jl=1,kproma
              rmes(idt_no)=5._dp*rc0x(jl,idt_o3)
              rmes(idt_no2)=0.5_dp*rc0x(jl,idt_o3)
             ENDDO
        ENDIF
      ENDIF

      ! mz_LG_20020115 end 

      IF (trlist%ti(jt)%basename.NE.'HNO3' .AND. &
          trlist%ti(jt)%basename.NE.'SO2'  .AND. &
          trlist%ti(jt)%basename.NE.'SO4'        ) THEN
        DO jl=1,kproma
           vdveg(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,jt)+rsveg(jl,jt)))
           vdsoil(jl,jt)=(1._dp/(prahslsn(jl)+rbslsn(jl,jt)+rsoil(jt)))
           vdws(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,jt)+rws(jt)))

           ! mz_LG_20020115 for the snow uptake resistance, a minimum of the 
           !     resistance of HNO3 and SO2 is being applied to avoid
           !     having very large deposition velocities over ice/snow
           !     covered surfaces. Despite the lack of observations
           !     to support this assumption, it is not very likely that
           !     for other species than HNO3 and SO2 the uptake by ice/
           !     snow surfaces will be larger

           vdsn(jl,jt)=(1._dp/(prahslsn(jl)+rbslsn(jl,jt)+ &
                MAX(rsnowhno3so2(jl),rsnow(jt))))
        ENDDO
      ENDIF
 
      ! mz_LG_20020115 HNO3

      IF (trlist%ti(jt)%basename.EQ.'HNO3') THEN
         DO jl=1,kproma
            rsveg(jl,jt)=MAX(10._dp,rsveg(jl,jt))
            vdveg(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,jt)+rsveg(jl,jt)))
            vdsoil(jl,jt)=(1._dp/(prahslsn(jl)+rbslsn(jl,jt)+rsoil(jt)))
            vdws(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,jt)+rws(jt)))
            vdsn(jl,jt)= &
            (1._dp/(prahslsn(jl)+rbslsn(jl,jt)+rsnowhno3so2(jl)))
         ENDDO

         ! mz_LG_20020115 SO2

      ELSE IF (trlist%ti(jt)%basename.EQ.'SO2') THEN
         DO jl=1,kproma
            vdveg(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,jt)+rsveg(jl,jt)))
            vdsoil(jl,jt)=(1._dp/(prahslsn(jl)+rbslsn(jl,jt) &
                          +rsoilso2(jl)))
            vdws(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,jt)+rws(jt)))
            vdsn(jl,jt)= &
               (1._dp/(prahslsn(jl)+rbslsn(jl,jt)+rsnowhno3so2(jl)))
         ENDDO

         ! mz_LG_20020115 SO4--

         !--- Assume for gaseous SO4 the same dry deposition velocity as for SO2:

      ELSE IF ( (trlist%ti(jt)%basename.EQ.'SO4' .AND. trlist%ti(jt)%nphase==GAS) .AND. lso2 ) THEN 

         DO jl=1,kproma
            vdveg(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,idt_so2)+rsveg(jl,idt_so2)))
            vdsoil(jl,jt)=(1._dp/(prahslsn(jl)+rbslsn(jl,idt_so2)+rsoilso2(jl)))
            vdws(jl,jt)=(1._dp/(prahveg(jl)+rbveg(jl,idt_so2)+rws(idt_so2)))
            vdsn(jl,jt)= (1._dp/(prahslsn(jl)+rbslsn(jl,idt_so2)+rsnowhno3so2(jl)))
         ENDDO

         !--- Use dry deposition velocity for total sulfate for the case of
         !    no partitioning between gas and aerosol phase:

      ELSE IF (trlist%ti(jt)%basename=='SO4' .AND..NOT. lso4gas) THEN 
         DO jl=1,kproma
            rsveg(jl,jt)=MAX(10._dp,rsveg(jl,jt))
            vdvfac=0.002_dp
            IF(stheta(jl).GT.0.175_dp.AND.psrfl(jl).GT.250_dp) vdvfac=0.01_dp
            vdveg(jl,jt)=1._dp/(prahveg(jl)+1._dp/(vdvfac*ustveg(jl)))
            vdsoil(jl,jt)=vdso4slsn(jl)/100._dp
            vdws(jl,jt)=vdveg(jl,jt)
            vdsn(jl,jt)=vdso4slsn(jl)/100._dp
         ENDDO
      ENDIF

      ! mz_LG_20020115 calculation of the over-land dry deposition velocity, considering
      !     the surface cover fractions

      DO jl=1,kproma

         vdland(jl,jt)=pcvs(jl)*vdsn(jl,jt)+ &
                       (1._dp-pcvs(jl))*(1._dp-pcvw(jl))*pvgrat(jl)*vdveg(jl,jt)+ &
                       (1._dp-pcvs(jl))*(1._dp-pvgrat(jl))*(1._dp-pcvw(jl))*vdsoil(jl,jt)+ &
                       (1._dp-pcvs(jl))*pcvw(jl)*vdws(jl,jt)
      ENDDO

     ! seaice?, the next statements determine if there is seaice coverage
     ! over the ocean

     ! mz_LG_20020213, There is a modification in echam5 compared to echam4
     !     with pfri > 0 over land for grids with frozen lake surfaces.
     !     The value is not contineous but just 0 or 1 over land (to avoid
     !     the introduction of serious complications in some of the physical
     !     routines, personal communications, Uli Schlese, 13-02-2002). 
     !     This change requires a modification of the use of the parameter
     !     seaice for use in the surface trace gas and aerosol exchange 
     !     processes. see also the subroutine xtsurf_drydep_bl, where this
     !     routine is actually being called.

     DO jl=1,kproma
        fsic=pfri(jl)
        fsicm1=1._dp-fsic

        ! mz_LG_20020115 for the sea ice uptake resistance, a minimum of the 
        !     resistance of HNO3 and SO2 is being applied (see also
        !     snow uptake)

        vdwat(jl,jt)=fsic*(1._dp/(prahslsn(jl)+rbslsn(jl,jt)+ &
             MAX(rsnowhno3so2(jl),rsnow(jt))))+ &
                     fsicm1*(1._dp/(prahwat(jl)+rb(jl,jt)+rwater(jt)))
        IF (trlist%ti(jt)%basename.EQ.'HNO3') THEN
           vdwat(jl,idt_hno3)=fsic*(1._dp/(prahslsn(jl) &   ! different formula for hno3
                +rbslsn(jl,idt_hno3)+rsnowhno3so2(jl)))+ &
                fsicm1*(1._dp/(prahwat(jl)+rb(jl,idt_hno3)+rwater(idt_hno3)))
        ELSE IF (trlist%ti(jt)%basename.EQ.'SO2') THEN
           vdwat(jl,idt_so2)=fsic*(1._dp/(prahslsn(jl) &    ! different formula for so2
                +rbslsn(jl,idt_so2)+rsnowhno3so2(jl)))+ &
                fsicm1*(1._dp/(prahwat(jl)+rb(jl,idt_so2)+rwater(idt_so2)))
        ! Assume deposition velocity of SO2 for pure sulfate gas phase:
        ELSE IF ( (trlist%ti(jt)%basename.EQ.'SO4' .AND. trlist%ti(jt)%nphase==GAS) .AND. lso2) THEN 
          vdwat(jl,idt_so4)=vdwat(jl,idt_so2)
        ! Only total sulfate considered:
       ELSE IF (trlist%ti(jt)%basename.EQ.'SO4' .AND..NOT. lso4gas) THEN 
          vdwat(jl,idt_so4)=fsic*vdso4slsn(jl)/100._dp+ &  ! different formula for so4
               fsicm1*vdso4wat(jl)/100._dp
       ENDIF

       ! mz_LG_20020115 finally, calculation of the grid deposition velocity from the 
       !     over-land and ocean dry deposition velocities, the deposition 
       !     velocities in vdrydep are given in [m s-1]!    

       IF(laland(jl)) THEN
          vdrydep_p(jl,krow)=vdland(jl,jt)
       ELSE
          vdrydep_p(jl,krow)=vdwat(jl,jt)
       ENDIF

        ! Set to minimum value and convert to m/s:

        vdrydep_p(jl,krow)=MAX(1.e-5_dp,vdrydep_p(jl,krow))

     ENDDO !jl=1, kproma

  ENDIF

 ENDDO !jt=1, ntrac

END SUBROUTINE xtsurf_vdbl
!==============================================================================
SUBROUTINE xtsurf_vdaer( kproma,   kbdim,   klev,    krow,     loland,  pvgrat, &
                         pcvs,     pcvw,    pfri,    pcvbs,    pfrw,    pum1,   &  
                         pvm1,     pustar,  pustveg, pustslsn, pu10,    pv10,   &
                         paz0w,    prahwat, prahveg, prahslsn, ptslm1,  pevap,  &
                         prh,      pxtm1                                        )

  !=====================================================================
  !    Program to calculate the dry deposition velocity for
  !    aerosols as a function of the particle radius considering the
  !    impaction and difussion. Sedimentation is also calculated but not
  !    included in the calculated deposition velocity since this process
  !    is considered in a separate routine
  !=====================================================================

  !   Author:
  !   -------
  !   Laurens Ganzeveld, MPI Mainz                           04/2001
  !
  !   Modifications:
  !   --------------
  !   Philip Stier,      MPI Hamburg (adaption to ECHAM/M7,
  !                                   non-integrated scheme) 12/2001
  !
  !   Methodology:
  !
  !     This program calculates the integrated deposition velocity from
  !     the mass size distribution for a log normal aerosol distribution
  !     defined by the radius and the log sigma. The model calculates the
  !     Vd over land and over sea considering the diffusion,impaction and
  !     sedimentation. Over sea the effect op particle growth due to the
  !     large relative humidity is accounted for and the effect of
  !     bubble bursting is also considered. The bubble bursting enhances
  !     the dry deposition since it causes the breakdown of the laminar
  !     boundary layer and the scavenging of the particles by the sea spray.
  !     This model version does not contain yet a parameterization which
  !     specifically considers the deposition to vegetated surfaces. For
  !     these surfaces, a surface resistance as a function of canopy
  !     structure should be incorporated. This might possible be implemented
  !     in the future (Laurens Ganzeveld, 29-01-2002)
  !
  !     The model requires as input parameters:
  !     -------------------------------------------------------------------
  !     - UM, windspeed at reference height
  !     - PAZ0M, surface roughness, over sea this term is calculated from UM
  !     - PTSLM1, Air or surface temperature
  !     - LOLAND, land-sea mask
  !     - N, aerosol number [cm -3]
  !     - R, aerosol radius [m]
  !     - LSIGMA, LOG sigma of the log normal distribution
  !     - RHOA_AEROS, the density of the aerosol
  !                   replaced by: densaer (Philip Stier)
  !========================================================================
  !     LG- the parameter names in M7 for the parameters R, LSIGMA, and RHOA 
  !         are pm6rp, sigma (are similar for all the modes of M7, 
  !         LG 23-01-2002) and densaer 
  ! =======================================================================      

  USE mo_aero,          ONLY: cmode
  USE mo_aero_m7,       ONLY: cmr2mmr, sigmaln, nmod
  USE mo_tracer,        ONLY: trlist, AEROSOLNUMBER, AEROSOLMASS
  USE mo_constants,     ONLY: api

  IMPLICIT NONE

  !--- input parameters from echam, the parameter pevap has still to be
  !    checked more carefully, if this is a used constant or actually the
  !    echam evaporation

  INTEGER :: kproma, kbdim, klev, krow

  LOGICAL :: loland(kbdim)

  REAL(dp)    :: pum1(kbdim,klev), pvm1(kbdim,klev), pustar(kbdim),  pustveg(kbdim), &
             pustslsn(kbdim),  pu10(kbdim),      pv10(kbdim),    paz0w(kbdim),   &
             prahwat(kbdim),   prahveg(kbdim),   pvgrat(kbdim),  pcvs(kbdim),    &
             pcvw(kbdim),      pcvbs(kbdim),     pfrw(kbdim),    pfri(kbdim),    &
             prahslsn(kbdim),  ptslm1(kbdim),    pevap(kbdim),   prh(kbdim)

  REAL(dp), INTENT(in) :: pxtm1(kbdim,klev,ntrac)

  !--- Stream and elements:

  REAL(dp),            POINTER :: vdrydep_p(:,:)
  REAL(dp),            POINTER :: rwet_p(:,:,:)
  REAL(dp),            POINTER :: densaer_p(:,:,:)

  !--- Local Variables:

  INTEGER :: ji, jl, jt, jmod, jtype

  INTEGER, PARAMETER :: NNUMBER=1, MASS=2

  !    Aerosol properties and parameters relevant to the calculations:

  REAL(dp)    :: rx1,rx2,um10,s,sc,cunning,sedspeed,vb_veg,vb_slsn,                   &
             vim_veg,vim_slsn,st_veg,st_slsn,st_wat,re,vb_sea,vim_sea,vkdaccsea,       &
             eff,rdrop,zdrop,qdrop,vkd_veg,vkd_slsn,vkd_wat,                      &
             vkc_veg,vkc_slsn,vkc_wat,vdpart_veg,vdpart_slsn,vdpart_wat,dc,relax, &
             eps,phi,alpha1,alpharat,vk1,vk2,zm,zmvd,zn,zdm,zdn,zdlnr

  ! mz_lg_20031014+ added for some modifications of the vegetation surface
  !     resistance based on the paper by Gallagher et al., JGR, 2002. The
  !     paper shows some more details about the parameterization by 
  !     Slinn, Atmos. Environment, Vol 16, 1785-1794, 1982
  REAL(dp)    :: vin_veg,zrebound
  ! mz_lg_20031014-

  REAL(dp)    :: zrint
  REAL(dp)    :: zvdrydep(kbdim)            ! Aerosol Dry Deposition Velocity defined for each mode
  REAL(dp)    :: zr(kbdim)                  ! Aerosol Radius of the respective tracer at klev
  REAL(dp)    :: znmr(kbdim)                ! Aerosol Number Mixing ratio at t=t+1

  !--- Auxiliary fields:

  REAL(dp)    :: um(kbdim),     alpha(kbdim), beta(kbdim), &
             alphae(kbdim), bubble(kbdim)

  LOGICAL :: ldrydep

  INTEGER, PARAMETER :: nint=199        ! number of integration intervals
  REAL(dp)               :: zintstep(nint)  ! integration intervals

  REAL(dp),    PARAMETER :: crmin=0.01E-6_dp   ! smallest radius for which a dry deposition 
                                        ! velocity is calculated [cm]

  !--- Assign values to used constants:

  REAL(dp), PARAMETER :: fln10=2.302585_dp
  REAL(dp), PARAMETER :: w2pi=2.506638_dp
  REAL(dp), PARAMETER :: g=9.8e2_dp          ! cm s-2 at sea level
  REAL(dp), PARAMETER :: dynvisc=1.789e-4_dp ! g cm-1 s-1
  REAL(dp), PARAMETER :: cl=0.066*1e-4_dp    ! mean free path [cm] (particle size also in cm)
  REAL(dp), PARAMETER :: bc= 1.38e-16_dp     ! boltzman constant [g cm-2 s-1 K-1] (1.38e-23 J deg-1)
  REAL(dp), PARAMETER :: kappa=1._dp         ! shapefactor
  REAL(dp), PARAMETER :: visc=0.15_dp        ! molecular viscocity [cm2 s-1]
  REAL(dp), PARAMETER :: daccm=0._dp         ! factor which corrects for evaporation (see paper slinn)
  REAL(dp), PARAMETER :: vkar=0.40_dp        ! von karman constant

  ! mz_lg_20031014+ added for the Vd aerosol over vegetation
  ! mz_lg_20040602+ modified
  REAL(dp), PARAMETER :: zAS    = 10.E-6_dp*1.E2_dp  ! um -> CM, see paper Gallagher and Slinn, 1982
  ! here the smallest collector size is set at 10 um
  ! Those are the values for Slinn's 82 model (see Table 1)
  REAL(dp), PARAMETER :: zAL    = 1.E-3_dp*1.E2_dp   ! mm -> CM, see paper Gallagher and Slinn, 1982
  ! here the largest collector size is set at 1 mm  
  ! mz_lg_20040602-+ modified
  ! mz_lg_20031014-

  !@@@ Check with Laurens (see comments above)
  pevap=0._dp

  !--- Integration stepsizes [m], 0-10 and 10-100,100-1000 um
  !    the first interval is divided into 1000 steps of 0.1 um
  !    whereas for the interval >10 um 90 steps of 1 um are selected
  !    and for the radius > 100 um 90 steps of 10 um are selected

  DATA zintstep /100*.01E-6_dp,90*1.E-6_dp,9*10.E-6_dp/


  !--- 1) Calculate correction factors:

  DO jl=1,kproma

     ! LG- calculation of some parameters required for the deposition calculations

     um(jl)=MAX(0.001_dp,SQRT(pum1(jl,klev)**2+pvm1(jl,klev)**2))
     um10=MAX(0.001_dp,SQRT(pu10(jl)**2+pv10(jl)**2))

     ! bubble bursting effect,see Hummelshoj, equation 10
     ! relationship by Wu (1988), note that Hummelshoj has not
     ! considered the cunningham factor which yields a different
     ! vb curve, with smaller values for small particles

     IF (lwhitecap) THEN
        alpha(jl)=MAX(1.e-10_dp,1.7e-6_dp*um10**3.75_dp)     ! old 10 m windspeed !!
        eff=0.5_dp
        rdrop=0.005_dp        ! cm
        zdrop=10.0_dp         ! cm
        qdrop=5._dp*(100._dp*alpha(jl))
        bubble(jl)=((100._dp*pustar(jl))**2)/(100._dp*um(jl))+eff* &
                   (2._dp*api*rdrop**2)*(2._dp*zdrop)*(qdrop/alpha(jl))
     ELSE
        alpha(jl)=0._dp
        bubble(jl)=0._dp
     ENDIF

     !--- Correction of particle radius for particle growth close to the
     !    surface according to Fitzgerald, 1975, the relative humidity over
     !    the ocean is restricted to 98% (0.98) due to the salinity

     s=MIN(0.98_dp,prh(jl))
     eps=0.6_dp
     beta(jl)=EXP((0.00077_dp*s)/(1.009_dp-s))
     phi=1.058_dp-((0.0155_dp*(s-0.97_dp))/(1.02_dp-s**1.4_dp))
     alpha1=1.2_dp*EXP((0.066_dp*s)/(phi-s))
     vk1=10.2_dp-23.7_dp*s+14.5_dp*s**2
     vk2=-6.7_dp+15.5_dp*s-9.2_dp*s**2
     alpharat=1._dp-vk1*(1._dp-eps)-vk2*(1._dp-eps**2)
     alphae(jl)=alpharat*alpha1

     !--- Over land no correction for large humidity close to the surface:

     IF (loland(jl).OR..NOT.ldr_rh) THEN
        alphae(jl)=1._dp
        beta(jl)=1._dp
     ENDIF

  END DO! jl=1, kproma

  !--- 2)Calculate dry deposition velocity for each internally mixed mode:

  IF (lvd_integrated) THEN

     DO jl=1, kproma

        DO jtype=1, 2

           DO jmod=1, nmod

              rwet_p    => rwet(jmod)%ptr
              densaer_p => densaer(jmod)%ptr

              ! mz_lg_20030521+, small change in criteria to determine if calculations
              !     should be bypassed, see also the same criteria for lvd_integrated=.false.

              IF ( ANY(trlist%ti(:)%mode==jmod.AND.trlist%ti(:)%ndrydep>0) .AND. &
                   rwet_p(jl,klev,krow) > crmin) THEN

                 !--- Initialisations:

                 DO jt=1, ntrac
                    IF (trlist%ti(jt)%mode==jmod .AND. trlist%ti(jt)%nphase==AEROSOLNUMBER ) THEN
                       znmr(jl)=pxtm1(jl,klev,jt)
                    END IF
                 END DO

                 SELECT CASE(jtype)

                 CASE(NNUMBER)

                    zr(jl)=rwet_p(jl,klev,krow)
                 
                 CASE(MASS)

                    zr(jl)=rwet_p(jl,klev,krow)*cmr2mmr(jmod)

                 END SELECT

                 rx1  = 0.0_dp
                 zmvd = 0.0_dp
                 zm   = 0.0_dp

                 DO ji=1,nint-1

                    ! Integration size interval is rx2-rx1:

                    rx1=rx1+zintstep(ji)
                    rx2=rx1+zintstep(ji+1)

                    !--- Calculate the number of particles in the intervall r+dr
                    !    for a log-normal distribution with the moments:
                    !    N=znmr, sigma, CountMeanRadius=pm6rp
                    !
                    !    dn=n(ln(r))dln(r) [m-3]
                    !
                    !    where:
                    !
                    !    r = (rx1+rx2)/2
                    !
                    !    and
                    !
                    !    n(ln(r)) = N/(sqrt(2 PI)*ln(sigma)) * exp(-(ln(r)-ln(CMR))/(2ln(sigma)**2)

                    zrint     = (rx1+rx2)/2._dp

                    !--- Particle Numbers:

                    zdlnr    = LOG(rx2)-LOG(rx1)

                    zn       = znmr(jl)/(2._dp*api*sigmaln(jmod)) * &
                               EXP(-(LOG(zrint)-LOG(zr(jl)))/(2._dp*sigmaln(jmod)**2))
                    zdn      = zn * zdlnr

                    !--- Average particle mass:
                    !    in intervall (zrint-dr,zrint+dr) [kg(tracer)/kg(air)]

                    zdm  = 4._dp/3._dp*api*zrint**3 * densaer_p(jl,klev,krow) * zdn

                    !--- Cunningham factor:
                    !    (Philip Stier, 11-2001, Separation into dry and wet not needed
                    !     as the ambient radius pm6rp incorporates the particle growth)
                    ! =================================================================
                    ! LG- It still must be checked if it is indeed allright to
                    !     calculate only the parameters for the aerosol modal
                    !     parameters given by M7 to include the effect of particle
                    !     growth. The parameterization of the effect of particle growth
                    !     that was included in the original scheme of the aerosol
                    !     dry deposition module corrects for the growth close to the
                    !     surface for a large relative humidity in quasi-laminar layer,
                    !     which is actual a sub-grid scale effect occuring in the
                    !     surface layer of echam. We have included again the calculation
                    !     of the parameters that are used to introduce this effect,
                    !     ALPHAE and BETA and use of this terms can easily be considered
                    !     or ignored, for the latter option setting the calculated values
                    !     to 1 (this is default done for over-land grids)
                    !
                    !     Laurens Ganzeveld, 30-01-2002
                    ! =================================================================

                    !--- Convert radii to cm to get the results in cm s-1:

                    cunning=1._dp+(cl/(alphae(jl)*zrint*1.E2_dp)**beta(jl))*(2.514_dp+0.800_dp*EXP(-0.55_dp*(zrint*1.E2_dp)/cl))

                    !--- Diffusivity:

                    dc=(bc*ptslm1(jl)*cunning)/(3._dp*api*dynvisc*(alphae(jl)*zrint*1.E2_dp)**beta(jl))

                    ! Relaxation:

                    ! mz_lg_20021311, modified based on discussions with
                    ! Philip Stier [kg m-3 => g cm-3]

                    relax=(densaer_p(jl,klev,krow)*1.E-3_dp*(((alphae(jl)*zrint)**beta(jl))**2)* &
                          cunning)/(18._dp*dynvisc*kappa)

                    !@@@ Sedimentation is calculated operator split in the subroutine xt_sediment:
                    !
                    !sedspeed=(((((alphae(jl)*zrint)**beta(jl))**2.)* &
                    !         densaer_p(jl,klev,krow)*1.E-3*g*cunning)/(18.*dynvisc))
                    !         [     kg m-3 => g cm-3          ]

                    ! Calculation of schmidt and stokes number

                    sc      =visc/dc
                    st_veg  =(relax*(100._dp*pustveg(jl))**2)/visc

                    ! mz_lg_20031014+, modified calculation of the Stokes number
                    !     over vegetated surfaces, see paper by Gallagher et al., 
                    !     JGR 2002. zAL is a characteristic radius for the
                    !     largest collectors comprising the surface
                    st_veg  =(relax*(100._dp*pustveg(jl))**2)/(g*zAL)
                    ! mz_lg_20031014-

                    st_slsn =(relax*(100._dp*pustslsn(jl))**2)/visc
                    st_wat  =(relax*(100._dp*pustar(jl))**2)/visc

                    !--- Calculation of the dry deposition velocity
                    !    See paper slinn and slinn, 1980, vd is related to d**2/3
                    !    over land, whereas over sea there is accounted for slip
                    !    vb represents the contribution in vd of the brownian diffusion
                    !    and vi represents the impaction.
                    !
                    !--- Over land, the vegetation and snow and bare soil fractions
                    !    are considered:

                    vb_veg   =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)*(sc**(-2._dp/3._dp))
                    vb_slsn  =(1._dp/vkar)*((pustslsn(jl)/um(jl))**2)*100._dp*um(jl)*(sc**(-2._dp/3._dp))

                    ! mz_lg_20031014+, modified calculation of the impaction over
                    !     vegetated surfaces, see paper by Gallagher et al., JGR 2002. 
                    !     We have applied here the parameterization by Slinn [1982] 
                    !     over vegetated surfaces
                    !     (Philip Stier 30/10/2003, corrected um to um(jl))
                    !vim_veg   =(1./vkar)*((pustveg(jl)/um(jl))**2)*100.*um(jl)*(10.**(-3./st_veg))

                    vim_veg   =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)*(st_veg**2/(1._dp+st_veg**2))
                    ! mz_lg_20031014-

                    vim_slsn  =(1._dp/vkar)*((pustslsn(jl)/um(jl))**2)*100._dp*um(jl)*(10._dp**(-3._dp/st_slsn))

                    ! LG- the term evap has not been defined yet (30-01-2002) and the
                    !     term daccm is set to zero anyhow. It still must be checked if
                    !     this term should be included and how it relates to the factors
                    !     that correct for particle growth close to the surface, according
                    !     to Fitzjarald, 1975 (see ALPHAE and BETA)


                    ! mz_lg_20031014+, modified calculation of the surface resistance over 
                    !     vegetated surfaces, see paper by Gallagher et al., JGR 2002. 
                    !     The calculation includes the interception collection efficiency 
                    !     vim and a rebound correction factor R
                    !vkd_veg  =pevap(jl)*daccm+(vb_veg+vim_veg)

                    vin_veg  =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)*      &
                         (1._dp/2._dp)*(zrint/zAS)**2           ! equation 18
                    zrebound =EXP(-st_veg**0.5_dp)            ! equation 19
                    vkd_veg  =pevap(jl)*daccm+zrebound*(vb_veg+vim_veg+vin_veg)
                    ! mz_lg_20031014-

                    vkd_slsn =pevap(jl)*daccm+(vb_slsn+vim_slsn)

                    !--- Over sea:
                    !    Brownian diffusion for rough elements, see Hummelshoj
                    !    re is the reynolds stress:

                    re        =(100._dp*pustar(jl)*100._dp*paz0w(jl))/visc
                    vb_sea     =(1._dp/3._dp)*100._dp*pustar(jl)*((sc**(-0.5_dp))*re**(-0.5_dp))
                    vim_sea     =100._dp*pustar(jl)*10._dp**(-3._dp/st_wat)
                    vkdaccsea =vb_sea+vim_sea
                    vkd_wat   =(1._dp-alpha(jl))*vkdaccsea+alpha(jl)*(bubble(jl))

                    ! Slinn and Slinn parameterization, calculation of vd:
                    ! ====================================================
                    ! LG- without considering the role of sedimentation !
                    !     This process is being calculated in a separate
                    !     subroutine in echam5!
                    ! ====================================================

                    vkc_veg     =(100._dp/prahveg(jl))
                    vkc_slsn    =(100._dp/prahslsn(jl))
                    vkc_wat     =(100._dp/prahwat(jl))
                    vdpart_veg  =1._dp/((1._dp/vkc_veg)+(1._dp/vkd_veg))
                    vdpart_slsn =1._dp/((1._dp/vkc_slsn)+(1._dp/vkd_slsn))
                    vdpart_wat  =1._dp/((1._dp/vkc_wat)+(1._dp/vkd_wat))

                    !--- Calculate sum(dm*vd(dm)):

                    zmvd = zmvd + zdm*( &
                           pcvs(jl)*vdpart_slsn+                              & ! snow fraction
                           pcvbs(jl)*vdpart_slsn+                             & ! bare soil fraction
                           (1._dp-pcvs(jl))*(1._dp-pcvw(jl))*pvgrat(jl)*vdpart_veg+ & ! vegetation fraction
                           (1._dp-pcvs(jl))*pcvw(jl)*vdpart_veg+                 & ! wet skin fraction
                           pfri(jl)*vdpart_slsn+                              & ! sea ice fraction
                           pfrw(jl)*vdpart_wat                                 )! water fraction

                    !--- Calculate m=sum(dm) [kg(tracer)/kg(air)]:

                    zm   = zm   + zdm

                 END DO !ji=1,nint-1

                 !--- Calculate integrated dry deposition velocity [cm s-1]:
                 !    vd = sum(dm*vd) / m

                 zvdrydep(jl)= zmvd/zm

              ELSE

                 zvdrydep(jl)=0._dp

              END IF

           END DO ! jl=1, kproma

           !--- Assign the calculated deposition velocity to all tracers
           !    in the respecive mode and re-convert it to [m s-1]:

           SELECT CASE(jtype)

           CASE(NNUMBER)

              DO jt=1,ntrac
                 IF (trlist%ti(jt)%ndrydep>0  .AND.     &
                     trlist%ti(jt)%mode==jmod .AND.     &
                     trlist%ti(jt)%nphase==AEROSOLNUMBER)THEN

                    vdrydep_p => vdrydep(jt)%ptr

                    vdrydep_p(1:kproma,krow)=zvdrydep(1:kproma)*1.E-2_dp

                 END IF
              END DO

           CASE(MASS)

              DO jt=1,ntrac
                 IF (trlist%ti(jt)%ndrydep>0  .AND.   &
                     trlist%ti(jt)%mode==jmod .AND.   &
                     trlist%ti(jt)%nphase==AEROSOLMASS)THEN

                    vdrydep_p => vdrydep(jt)%ptr

                    vdrydep_p(1:kproma,krow)=zvdrydep(1:kproma)*1.E-2_dp

                 END IF
              END DO

           END SELECT

        END DO ! jmod=1, nmod

     END DO ! jtype=1, 2


  !--- 3) Calculation of the dry deposition velocity utilizing the log-normal properties:

  ELSE IF( .NOT. lvd_integrated) THEN

     DO jmod=1, nmod

        rwet_p    => rwet(jmod)%ptr
        densaer_p => densaer(jmod)%ptr

        DO jtype=1, 2

           !--- Calculations are done in [cm] to get results in [cm s-1]:

           SELECT CASE(jtype)

           CASE(NNUMBER)

              ldrydep=ANY(trlist%ti(:)%ndrydep==2.AND.trlist%ti(:)%mode==jmod.AND.trlist%ti(:)%nphase==AEROSOLNUMBER)
              zr(1:kproma)=rwet_p(1:kproma,klev,krow)*1.E2_dp

           CASE(MASS)

              ldrydep=ANY(trlist%ti(:)%ndrydep==2.AND.trlist%ti(:)%mode==jmod.AND.trlist%ti(:)%nphase==AEROSOLMASS)
              zr(1:kproma)=rwet_p(1:kproma,klev,krow)*1.E2_dp*cmr2mmr(jmod)

           END SELECT

           IF(ldrydep) THEN

              DO jl=1, kproma

                 !--- Do calculations only for particles larger than 0.0001 um :

                 IF (zr(jl) > crmin) THEN

                    !--- Cunningham factor:

                    cunning=1._dp+(cl/(alphae(jl)*zr(jl))**beta(jl))*(2.514_dp+0.800_dp*EXP(-0.55_dp*(zr(jl))/cl))

                    !--- Diffusivity:

                    dc=(bc*ptslm1(jl)*cunning)/(3._dp*api*dynvisc*(alphae(jl)*zr(jl))**beta(jl))

                    ! Relaxation:

                    ! mz_lg_20021311, modified based on discussions with
                    ! Philip Stier [kg m-3 => g cm-3]

                    relax=(densaer_p(jl,klev,krow)*1.E-3_dp*(((alphae(jl)*zr(jl))**beta(jl))**2)* &
                           cunning)/(18._dp*dynvisc*kappa)

                    ! Sedimentation is calculated operator split in the subroutine xt_sediment:
                    !
                    ! sedspeed=(((((alphae(jl)*zr(jl))**beta(jl))**2.)* &
                    !          densaer_p(jl,klev,krow)*1.E-3*g*cunning)/(18.*dynvisc))

                    ! Calculation of schmidt and stokes number

                    sc      =visc/dc
                    st_veg  =(relax*(100._dp*pustveg(jl))**2)/visc

                    ! mz_lg_20031014+, modified calculation of the Stokes number
                    !     over vegetated surfaces, see paper by Gallagher et al., 
                    !     JGR 2002. zAL is a characteristic radius for the
                    !     largest collectors comprising the surface
                    ! ham_ps_20031201+ included minumum value for Stokes numbers:
                    st_veg  =MAX((relax*(100._dp*pustveg(jl))**2)/(g*zAL),1.E-1_dp)
                    ! mz_lg_20031014-
                    st_slsn =MAX((relax*(100._dp*pustslsn(jl))**2)/visc,1.E-1_dp)
                    st_wat  =MAX((relax*(100._dp*pustar(jl))**2)/visc,1.E-1_dp)
                    ! ham_ps_20031201-

                    !--- Calculation of the dry deposition velocity
                    !    See paper slinn and slinn, 1980, vd is related to d**2/3
                    !    over land, whereas over sea there is accounted for slip
                    !    vb represents the contribution in vd of the brownian diffusion
                    !    and vi represents the impaction.
                    !
                    !--- Over land, the vegetation and snow and bare soil fractions
                    !    are considered:

                    vb_veg   =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)*(sc**(-2._dp/3._dp))
                    vb_slsn  =(1._dp/vkar)*((pustslsn(jl)/um(jl))**2)*100._dp*um(jl)*(sc**(-2._dp/3._dp))
                    vim_veg  =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)*(10._dp**(-3._dp/st_veg))

                    ! mz_lg_20031014+, modified calculation of the impaction over
                    !     vegetated surfaces, see paper by Gallagher et al., JGR 2002. 
                    !     We have applied here the parameterization by Slinn [1982] 
                    !     over vegetated surfaces
                    vim_veg   =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)* &
                               (st_veg**2/(1._dp+st_veg**2))
                    ! mz_lg_20031014-

                    vim_slsn  =(1._dp/vkar)*((pustslsn(jl)/um(jl))**2)*100._dp*um(jl)*(10._dp**(-3._dp/st_slsn))

                    ! LG- the term evap has not been defined yet (30-01-2002) and the
                    !     term daccm is set to zero anyhow. It still must be checked if
                    !     this term should be included and how it relates to the factors
                    !     that correct for particle growth close to the surface, according
                    !     to Fitzjarald, 1975 (see ALPHAE and BETA)

                    ! mz_lg_20031014+, modified calculation of the surface resistance over 
                    !     vegetated surfaces, see paper by Gallagher et al., JGR 2002. 
                    !     The calculation includes the interception collection efficiency 
                    !     vim and a rebound correction factor R
                    !vkd_veg  =pevap(jl)*daccm+(vb_veg+vim_veg)
                    vin_veg  =(1._dp/vkar)*((pustveg(jl)/um(jl))**2)*100._dp*um(jl)*      &
                              (1._dp/2._dp)*(zr(jl)/zAS)**2          ! equation 18
                    zrebound =EXP(-st_veg**0.5_dp)                ! equation 19
                    vkd_veg  =pevap(jl)*daccm+zrebound*(vb_veg+vim_veg+vin_veg)
                    ! mz_lg_20031014-

                    vkd_slsn =pevap(jl)*daccm+(vb_slsn+vim_slsn)

                    !--- Over sea:
                    !    Brownian diffusion for rough elements, see Hummelshoj
                    !    re is the reynolds stress:

                    re        =(100._dp*pustar(jl)*100._dp*paz0w(jl))/visc
                    vb_sea    =(1._dp/3._dp)*100._dp*pustar(jl)*((sc**(-0.5_dp))*re**(-0.5_dp))
                    vim_sea   =100._dp*pustar(jl)*10._dp**(-3._dp/st_wat)
                    vkdaccsea =vb_sea+vim_sea
                    vkd_wat   =(1._dp-alpha(jl))*vkdaccsea+alpha(jl)*(bubble(jl))

                    ! Slinn and Slinn parameterization, calculation of vd:
                    ! ====================================================
                    ! LG- without considering the role of sedimentation !
                    !     This process is being calculated in a separate
                    !     subroutine in echam5!
                    ! ====================================================

                    vkc_veg     =(100._dp/prahveg(jl))
                    vkc_slsn    =(100._dp/prahslsn(jl))
                    vkc_wat     =(100._dp/prahwat(jl))

                    !@@@ Included security checks for small values of vxx_yyy:

                    IF(vkc_veg > 1.E-5_dp .AND. vkd_veg > 1.E-5_dp) THEN
                       vdpart_veg  =1._dp/((1._dp/vkc_veg)+(1._dp/vkd_veg))
                    ELSE
                       vdpart_veg  = 0._dp
                    END IF
                    IF(vkc_slsn > 1.E-5_dp .AND. vkd_slsn > 1.E-5_dp) THEN 
                       vdpart_slsn = 1._dp/((1._dp/vkc_slsn)+(1._dp/vkd_slsn))
                    ELSE
                       vdpart_slsn = 0._dp
                    END IF
                    IF(vkc_wat > 1.E-5_dp .AND. vkd_wat > 1.E-5_dp) THEN
                       vdpart_wat  =1._dp/((1._dp/vkc_wat)+(1._dp/vkd_wat))
                    ELSE
                       vdpart_wat = 0._dp
                    END IF

                    !--- Calculate the dry deposition velocity weighted according to the surface types:

                    zvdrydep(jl) = pcvs(jl)                              *vdpart_slsn + & ! snow fraction
                                   pcvbs(jl)                             *vdpart_slsn + & ! bare soil fraction
                                   (1._dp-pcvs(jl))*(1._dp-pcvw(jl))*pvgrat(jl)*vdpart_veg  + & ! vegetation fraction
                                   (1._dp-pcvs(jl))*pcvw(jl)                *vdpart_veg  + & ! wet skin fraction
                                   pfri(jl)                              *vdpart_slsn + & ! sea ice fraction
                                   pfrw(jl)                              *vdpart_wat      ! water fraction

                 ELSE

                    zvdrydep(jl)=0._dp

                 END IF !zr(jl) > crmin

              END DO ! jl=1, kproma

              SELECT CASE(jtype)
                     
              CASE(NNUMBER)

                 DO jt=1,ntrac
                    IF (trlist%ti(jt)%ndrydep>0  .AND.     &
                        trlist%ti(jt)%mode==jmod .AND.     &
                        trlist%ti(jt)%nphase==AEROSOLNUMBER)THEN

                       vdrydep_p => vdrydep(jt)%ptr

                       vdrydep_p(1:kproma,krow)=zvdrydep(1:kproma)*1.E-2_dp

                    END IF
                 END DO

              CASE(MASS)

                 DO jt=1,ntrac
                    IF (trlist%ti(jt)%ndrydep>0  .AND.   &
                        trlist%ti(jt)%mode==jmod .AND.   &
                        trlist%ti(jt)%nphase==AEROSOLMASS)THEN

                       vdrydep_p => vdrydep(jt)%ptr

                       vdrydep_p(1:kproma,krow)=zvdrydep(1:kproma)*1.E-2_dp

                    END IF
                 END DO

              END SELECT
      
           END IF ! ldrydep

        END DO ! jtype=1, 2

     END DO ! jmod=1, nmod

  END IF !(lvd_integrated)
           
END SUBROUTINE xtsurf_vdaer
!------------------------------------------------------------------------------
END MODULE mo_xtsurf_drydep
