MODULE mo_xtsurf_init
  !----------------------------------------------------------------------
  ! This module handles all input terms required for the calculations
  ! of the surface trace gas and aerosol exchange processes. Most of 
  ! parameters are vegetation and soil data derived from satellite data
  ! a high-resolution geograhical databases. For more details see the
  ! routine where the actual reading occurs. The data are monthly mean
  ! values, whenever there is an annual cycle in the data. In this 
  ! module, there is also the initialisation of the surface resistances
  ! used for the dry deposition calculations.
  ! 
  ! Original by: Martin Schultz and Hans-Stefan Bauer, MPIfM, July 2000
  ! Modified by Laurens Ganzeveld and Andreas Rhodin, October, 2001
  !---------------------------------------------------------------------
  USE mo_kind,          ONLY: dp
  USE mo_decomposition, ONLY: dc => local_decomposition, &
                              gc => global_decomposition
  USE mo_transpose,     ONLY: scatter_gp
  USE mo_mpi,           ONLY: p_pe, p_io
  USE mo_filename,      ONLY: find_next_free_unit
  USE mo_doctor,        ONLY: nout
  USE mo_memory_base,   ONLY: t_stream, new_stream, add_stream_element, &
                              default_stream_setting, AUTO
  USE mo_time_event,    ONLY: io_time_event
  USE mo_tracer,        ONLY: trlist

  IMPLICIT NONE

  !----------------
  ! Public entities
  !----------------
  PRIVATE
  PUBLIC :: xtsurf_init      ! read xtsurf inputdata
  PUBLIC :: alloc_xtsurf_mem ! allocate memory for the surface module

  type (t_stream) ,pointer :: xtsurf

  !-----------------
  ! Private entities
  !-----------------

  ! mz_LG_20020115 declaration of the the maximum number of vegetation layers
  !     this should resemble the number of layers being distinguished
  !     in the input vegetation files (default 2 layers, must be extended
  !     to 4 layers for hight resolution calculations of radiation profiles.
  !     This is required due to the sensitivity of the isoprene emission
  !     calculations for the vertical discretazation, October 2001). 
  !     Whenever there is conflicting number of layers defined, an error
  !     message is given in the subroutine vegetation.f90, where the input
  !     file is being read.

  INTEGER, PUBLIC, PARAMETER :: nveglay_hr=2 
  LOGICAL, PUBLIC, PARAMETER :: lveg_mlay=.FALSE. ! switch for calling
                                                  ! multi-layer vegetation model 
  INTEGER, PUBLIC, PARAMETER :: nspec_vocemis=3   ! number of emitted VOC species
  INTEGER, PUBLIC, PARAMETER :: ncl_soilph=7      ! number of soil pH classes

  ! mz_LG_20020115 surface cover, lad profiles and emisfact are still 3 dim.
  !     parameters (lat, lon and canopy levels). These parameters don't
  !     have an annual cycle (01-02) 

  REAL(dp) ,PUBLIC, POINTER :: dm (:,:)          ! dry matter
  REAL(dp) ,PUBLIC, POINTER :: lai (:,:)         ! LAI
  REAL(dp) ,PUBLIC, POINTER :: lad (:,:,:)       ! LAD profile
  REAL(dp) ,PUBLIC, POINTER :: hcols (:,:)       ! canopy height
  REAL(dp) ,PUBLIC, POINTER :: z0m (:,:)         ! surface roughness
  REAL(dp) ,PUBLIC, POINTER :: disp (:,:)        ! displacement height
  REAL(dp) ,PUBLIC, POINTER :: forestols (:,:)   ! forest fraction
  REAL(dp) ,PUBLIC, POINTER :: emisfact (:,:,:)  ! emission factors VOC's

  ! mz_LG_20020115 soil pH

  REAL(dp) ,PUBLIC, POINTER :: soilph (:,:,:)    ! soil pH classes

  ! mz_LG_20020115 some additional parameters required for the calculation of 
  !     the soil-biogenic NO emission fluxes

  LOGICAL                  :: alloc = .FALSE.

CONTAINS
!==============================================================================
! Public Routines
!==============================================================================
SUBROUTINE alloc_xtsurf_mem

    !-----------------------------------------------------------------
    ! construct the xtsurf tables
    ! all information specific to this tables is set in this subroutine
    !-----------------------------------------------------------------

    USE mo_control,       ONLY: nlev
    USE mo_time_control,  ONLY: ndaylen, delta_time
    USE mo_memory_base,   ONLY: set_stream, NETCDF, GRIDPOINT

    INTEGER :: nstepday

    ! mz_LG_20020115 geographical information required for the xtsurf calculations 

    CALL new_stream (xtsurf,'xtsurf',                              &
                     NETCDF,                                       &
                     interval=io_time_event(1,'months','first',0), &
                     lpost =.TRUE.,                               &
                     lrerun=.TRUE.                                 )

    CALL default_stream_setting (xtsurf ,units ='', &
                                      laccu =.FALSE., &
                                      lrerun=.TRUE., &
                                      lpost =.TRUE.)

    !--------------- 
    ! xtsurf parameters
    !---------------

    ! mz_LG_20020115 surface cover parameters

    CALL add_stream_element (xtsurf,'DM', dm ,units ='g m-2', &
                             gdims=(/dc% nlon, dc% nlat/), &
                             ldims=(/dc% nglon, dc% nglat/), &
                             lpost=.TRUE.,longname='Dry matter',code=101)
    CALL add_stream_element (xtsurf,'LAI', lai ,units ='m2 m-2', &
                             gdims=(/dc% nlon, dc% nlat/), &
                             ldims=(/dc% nglon, dc% nglat/), &
                             lpost=.TRUE.,longname='Leaf Area Index',code=102)

    ! mz_LG_20020115 explicit definition of the three dimensional array. Default the 
    !     second array element is the number of levels and therefore the 
    !     array sequence is changed in the code from lon, lat, levels 
    !     (nveglay_hr for LAD) to lon, levels, lat !!!!

    CALL add_stream_element (xtsurf,'LAD', lad ,units='m2 m-3', & 
                             gdims=(/dc% nlon, nveglay_hr, dc% nlat/), &
                             ldims=(/dc% nglon,nveglay_hr, dc% nglat/), &
                             lpost=.TRUE.,repr=GRIDPOINT, &
                             longname='Leaf Area Density',code=103)
    CALL add_stream_element (xtsurf,'HC', hcols ,units ='m', &
                             gdims=(/dc% nlon, dc% nlat/), &
                             ldims=(/dc% nglon, dc% nglat/), &
                             lpost=.TRUE.,longname='canopy height',code=104)
    CALL add_stream_element (xtsurf,'z0m', z0m ,units ='m', &
                             gdims=(/dc% nlon, dc% nlat/), &
                             ldims=(/dc% nglon, dc% nglat/), &
                             lpost=.TRUE.,longname='surface roughness',code=105)
    CALL add_stream_element (xtsurf,'DISP', disp ,units ='m', &
                             gdims=(/dc% nlon, dc% nlat/), &
                             ldims=(/dc% nglon, dc% nglat/), &
                             longname='displacement height')
    CALL add_stream_element (xtsurf,'forest', forestols ,units ='[0-1]', &
                             gdims=(/dc% nlon, dc% nlat/), &
                             ldims=(/dc% nglon, dc% nglat/), &
                             lpost=.TRUE.,longname='forest fraction',code=106)
    CALL add_stream_element (xtsurf,'emisfact', emisfact ,units='ug C g-1 hr-1', & 
                             gdims=(/dc% nlon, MAX(nlev,nspec_vocemis), dc% nlat/), &
                             ldims=(/dc% nglon, MAX(nlev,nspec_vocemis), dc% nglat/), &
                             lpost=.TRUE.,repr=GRIDPOINT, &
                             longname='emisfactor VOC emissions',code=107)
    CALL add_stream_element (xtsurf,'soilpH', soilpH ,units='-', & 
                             gdims=(/dc% nlon, ncl_soilph, dc% nlat/), &
                             ldims=(/dc% nglon, ncl_soilph, dc% nglat/), &
                             lpost=.TRUE.,repr=GRIDPOINT, &
                             longname='soil pH classes (5)',code=108)

END SUBROUTINE alloc_xtsurf_mem
!==============================================================================
SUBROUTINE xtsurf_init(ithismonth,ilastmonth,iyear)
    !-------------------------------------------------------------
    ! This subroutine reads the input files containing data 
    ! required for the calculations of the surface trace gas and 
    ! aerosol exchange processes  
    !-------------------------------------------------------------
    USE mo_time_control,      ONLY: lstart

    USE mo_control,       ONLY: nn
    USE mo_util_string,   ONLY: char2

    !----------    
    ! Arguments
    !----------
    INTEGER, INTENT(in) :: ithismonth, ilastmonth, iyear
    !----------------
    ! local variables
    !----------------

    CHARACTER(len=80) :: fname 

    ! mz_LG_20020115 calling of routines to read files containing  
    !     data for a whole year (or annual mean or 12 monthly mean
    !     fields).This is only done at the start of the simulations

    IF (lstart .OR. ithismonth/=ilastmonth) THEN

      !---------------------------------------------------------
      ! Read netcdf file with vegetation data for each month
      ! The input file and its structure is defined in the 
      ! namelist xtsurf.nml in the jobscript. 
      !---------------------------------------------------------

      CALL read_xtsurf_input(ithismonth,iyear)

      !---------------------------------------
      ! Read input files with soil data (pH)
      !---------------------------------------

      CALL read_xtsoil_input

    ENDIF

END SUBROUTINE xtsurf_init
!=============================================================================
SUBROUTINE read_xtsurf_input(imonth,iyear)
! ------------------------------------------------------------------
!     this routine reads/assigns the vegetation database, 
!     Leaf Area Index (LAI), Leaf Area Density (LAD) profiles,
!     and other vegetation parameters. This fields have been
!     produced in a preprocessing program using a high-resolution 
!     ecosystem database (Olson, '92) and AVHRR data. This routine 
!     reads the netcdf files xtsurf_yyyy.nc containing all the 
!     relevant surface cover parameters for the whole year (montly 
!     mean values are used for the annual cycle)
!
!     Laurens Ganzeveld, 1998, modified 2002 for implemtation
!     in ECHAM4/5 f90 routines
!
!     December 2001/April 2002, further modified to read in the 
!     high-resolution input netcdf files, with the read data being 
!     regridded to the proper resolution using the programmes developed 
!     by Joeckel and Sander. Moreover, the fields for VOC and NO 
!     emissions are being read in separate routines  
! ------------------------------------------------------------------

  ! mz_LG_20020115 modules required for regridding

  USE mo_exception,          ONLY: finish
  USE mo_read_netcdf77,         ONLY: read_var_hs_nf77_2d

  IMPLICIT NONE
  INTEGER :: i,j,jk,jjk,jl,jr,k,nparam,nlayers
  INTEGER :: iunit,imonth,iyear
  INTEGER :: nproma

  INTEGER, PARAMETER                :: nfield=7
  INTEGER                           :: ierr
  CHARACTER(LEN=256)                :: varname(nfield)
 !
  REAL(dp) ,POINTER     :: field(:,:,:)   
  REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)

  ! mz_LG_20020115 reading file with vegetation data, e.g. LAI, LAD profile,
  !     canopy height. The fields are preprocessed for the proper horizontal 
  !     resolution

  IF (p_pe == p_io) THEN
    
    ALLOCATE (field (dc% nlon,nfield,dc% nlat))

    ! mz_LG_20020115 reading the netcdf with surface cover data and performing 
    !     the regridding in the subroutine regrid_control. The assignment of 
    !     theinputfile names and dimensions of the file is being done in the 
    !     jobscript

    varname(1) = 'lai'
    varname(2) = 'hc'
    varname(3) = 'z0m'
    varname(4) = 'disp'
    varname(5) = 'forest'
    varname(6) = 'lad_under'
    varname(7) = 'lad_upper'
    DO i = 1, nfield
       CALL read_var_hs_nf77_2d ('xtsurf.nc', 'lon', 'lat', 'time', imonth, &
            TRIM(varname(i)), field(:,i,:), ierr)
    END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of surf ---------------------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read_xtsurf_input: '
    WRITE (nout,*)
    WRITE (nout,*) '      reading xtsurf from file xtsurf.nc '
    WRITE (nout,*) '------------------------------------------------------------------------------'

  ENDIF

  ! mz_LG_20020115 end reading

  ALLOCATE (lfield(dc%nproma, nfield, dc% ngpblks))
  CALL scatter_gp (field, lfield, gc)

  ! mz_LG_20020115 assign the fields

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jl=1, nproma

      ! mz_LG_20020115 be aware that the array sequence of the input parameters has been 
      !     changed to be able to use the standard tracer interface routines.
      !     Consequently the use of these parameters in the subroutines should
      !     be changed consistently !

      lai(jl,jr)=lfield(jl,1,jr)                   !leaf area index
      hcols(jl,jr)=MAX(1.e-10_dp,lfield(jl,2,jr))  !canopy height
      z0m(jl,jr)=lfield(jl,3,jr)                   !surface roughness
      disp(jl,jr)=lfield(jl,4,jr)                  !displacement height
      forestols(jl,jr)=lfield(jl,5,jr)             !forest fraction
      ! mz_LG_20020115  the loop of the vertical lad profile is such that the lowest
      !      canopy layer has number 1 and the top canopy layer has number
      !      nlevv (see also voc_geia_avhrr.f and the output file 
      !      lad_olson.out). however, in echam the top vegetation layer has
      !      number 1 and therefore to call the proper lad value consistent
      !      with the number of the layer the layer array is reversed.

      DO jk=1,nveglay_hr
         jjk=nveglay_hr+1-jk ! lowest canopy layer=1 in file 
         lad(jl,jk,jr)=lfield(jl,jjk+5,jr)         ! lad profiles
      ENDDO

    ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_pe == p_io) DEALLOCATE (field)
  RETURN

END SUBROUTINE read_xtsurf_input
!=============================================================================
SUBROUTINE read_xtsoil_input
! ----------------------------------------------------------------
!     this routine reads the soil pH input files for use in the
!     dry deposition scheme. The program reads the high-resolution
!     input netcdf file containing 7 fields with each field 
!     describing for each high-resolution grid square if the 
!     specific soil pH class is present (1) or not (0)
!     These 7 fields are then used in the regridding routine to
!     determine the grid fractions of each soil pH class. 
!     
!     Laurens Ganzeveld, 1998, modified 2001 for implemtation
!     in ECHAM4/5 f90 routines
! ----------------------------------------------------------------

  USE mo_read_netcdf77,         ONLY: read_var_hs_nf77_2d

  IMPLICIT NONE

  INTEGER :: jl,jr,jt,i,j,k
  INTEGER :: nproma

  !  pointers:
  !
  REAL(dp), DIMENSION(:,:,:,:), POINTER :: edat    ! data on echam grid (x,z,s,y)
  INTEGER, PARAMETER                :: nfield=7
  INTEGER                           :: ierr
  CHARACTER(LEN=256)                :: varname(nfield)


  REAL(dp) ,POINTER     :: field(:,:,:)   
  REAL(dp) ,ALLOCATABLE :: lfield(:,:,:)

  ! mz_LG_20020128 reading file with the global soil pH field. The field is 
  !     preprocessed for the proper horizontal resolution

  NULLIFY (FIELD)

  IF (p_pe == p_io) THEN
    
    ALLOCATE (field (dc% nlon,ncl_soilph,dc% nlat))

    varname(1)='F1'
    varname(2)='F2'
    varname(3)='F3'
    varname(4)='F4'
    varname(5)='F5'
    varname(6)='F6'
    varname(7)='F7'
    DO i = 1, nfield
       CALL read_var_hs_nf77_2d('xtsoil.nc', 'lon', 'lat', 'time', 1, &
            TRIM(varname(i)), field(:,i,:), ierr)
    END DO
    WRITE (nout,*) '------------------------------------------------------------------------------'
    WRITE (nout,*) '--- Initialization of soil ---------------------------------------------------'
    WRITE (nout,*)
    WRITE (nout,*) '   read_xtsoil_input: '
    WRITE (nout,*)
    WRITE (nout,*) '      reading xtsoil from file xtsoil.nc '
    WRITE (nout,*) '------------------------------------------------------------------------------'

  ENDIF

11 FORMAT (32f5.3) 

  ALLOCATE (lfield(dc%nproma, ncl_soilph, dc%ngpblks))
  CALL scatter_gp (field, lfield, gc)

  ! mz_LG_20020128, assigning the soil pH classes

  DO jr=1,dc%ngpblks

     IF ( jr == dc% ngpblks ) THEN
        nproma = dc% npromz
     ELSE
        nproma = dc% nproma
     END IF

     DO jt=1,ncl_soilph

        DO jl=1,nproma
           soilpH(jl,jt,jr)=lfield(jl,jt,jr)        !soil pH classes
        ENDDO

     ENDDO
  ENDDO

  DEALLOCATE (lfield)
  IF (p_pe == p_io) DEALLOCATE (field)

  RETURN

END SUBROUTINE read_xtsoil_input
!------------------------------------------------------------------------------
END MODULE mo_xtsurf_init
