!
! This file holds the calls to the entries of the submodels 
! attached to ECHAM.
!
! Change this file to attach submodels to ECHAM.
!
!==============================================================================
  SUBROUTINE call_init_submodels_1
  !
  ! This routine is called from 'initialize' before
  ! the initialization of the tracer interface
  !
  ! New submodels must be introduced from here
  !
    USE mo_control,   ONLY: ldebugs
    USE mo_debugs,    ONLY: init_debugs
    USE mo_tracer,    ONLY: new_submodel
    USE mo_aero,      ONLY: lham, id_ham, setaero

    IMPLICIT NONE

    IF (ldebugs) THEN
       CALL init_debugs
    END IF

    !--- 1) Read namelists and control variables for the HAM aerosol model:

    CALL setaero

    !--- 2) Request the HAM submodel ID:

    IF (lham) CALL new_submodel("HAM",id_ham)

  END SUBROUTINE call_init_submodels_1
!==============================================================================
  SUBROUTINE call_init_submodels_2
  !
  ! This routine is called from 'initialize' after
  ! the initialization of the tracer interface
  !
  ! Submodel initializations requiring information 
  ! from the tracer interface should be introduced 
  ! here
  !
    USE mo_aero,      ONLY: aero_initialize, lham, laero_rad
    USE mo_aero_m7,   ONLY: setaeroM7, m7_initialize
    USE mo_aero_rad,  ONLY: aero_rad_initialize

    IMPLICIT NONE

    !--- 1) Initialise HAM aerosol model:

    IF (lham) THEN

       !--- 2) Initialize the m7 submodel:

       CALL setaeroM7

       CALL m7_initialize

       !--- 3) Initialize the HAM aerosol model:
       !       To be called after m7_initialize!

       CALL aero_initialize

       !--- 4) Initialize the HAM radiation module:

       IF (laero_rad) CALL aero_rad_initialize

    END IF

  END SUBROUTINE call_init_submodels_2 
!==============================================================================
  SUBROUTINE call_request_tracer
  !
  ! This routine is called from 'initrac', module 'mo_tracer'
  ! 
  ! tracer request routines are called from here
  !
  ! Template for a tracer request routine:
  !
  !    SUBROUTINE request_tracer
  !      USE mo_tracer, ONLY: new_tracer
  !      IMPLICIT NONE
  !      CALL new_tracer('tracername1' ,'modulename', ...)
  !      CALL new_tracer('tracername2' ,'modulename', ...)
  !    END SUBROUTINE request_tracer
  !
    USE mo_sub_nml,   ONLY: request_tracer_nml, set_tracer_nml
    USE mo_aero,      ONLY: lham
    USE mo_aero_trac, ONLY: ham_request_tracer

 ! uh +
    USE mo_be10,      ONLY: request_tracer_be10
    USE mo_control,   ONLY: lcn
  ! uh -


    IMPLICIT NONE
    !
    ! routine to request tracers via namelist group /NEW_TRACER/
    !
    CALL request_tracer_nml
    CALL set_tracer_nml

    !--- Request tracers for the aerosol model:

    IF (lham) CALL HAM_request_tracer

  !uh+
    IF (lcn) CALL request_tracer_be10
  !uh-


  END SUBROUTINE call_request_tracer
!==============================================================================
  SUBROUTINE call_init_submodel_memory
  !
  ! This routine is called from subroutine 'init_memory', module
  ! 'mo_memory_streams'. Routines are called from here to allocate memory
  ! and to define output streams.
  !

    USE mo_xtsurf_init,    ONLY: alloc_xtsurf_mem
    USE mo_aero,           ONLY: lham, laero_rad, lodiag
    USE mo_aero_mem,       ONLY: construct_stream_aero, &
                                 construct_stream_diag, &
                                 construct_stream_input
    USE mo_aero_rad_mem,   ONLY: construct_stream_aero_rad
    USE mo_aero_diag,      ONLY: aero_diag_mem
    USE mo_mvstream,       ONLY: init_mvstream

  ! +uh
    USE mo_be10,           ONLY: ini_be10_memory
    USE mo_control,        ONLY: lcn
  ! -uh


    IMPLICIT NONE

    !--- Allocate memory for HAM sub-modules:

    IF (lham) THEN

       CALL alloc_xtsurf_mem

       CALL construct_stream_aero

       CALL construct_stream_diag

       CALL construct_stream_input

       IF (laero_rad) CALL construct_stream_aero_rad

       IF (lodiag) CALL aero_diag_mem

    END IF

    !--- Initialize mean-value calculation:

    CALL init_mvstream

! uh+
    IF (lcn) CALL ini_be10_memory
! uh-



  END SUBROUTINE call_init_submodel_memory
!------------------------------------------------------------------------------
  SUBROUTINE call_free_submodel_memory
    !
    ! Routines are called from here to deallocate memory
    !
    USE mo_aero,          ONLY: lham, ndust, laero_rad
    USE mo_aero_rad,      ONLY: aero_rad_mem_cleanup
    USE mo_bgc_dust_emis, ONLY: bgc_dust_cleanup

    IMPLICIT NONE

    !--- De-allocate memory of HAM sub-modules:

    IF (lham) THEN

       IF (laero_rad) CALL aero_rad_mem_cleanup

       IF (ndust == 2) CALL bgc_dust_cleanup

    END IF

  END SUBROUTINE call_free_submodel_memory
!==============================================================================
  SUBROUTINE call_chem1 (kproma, kbdim, klev, klevp1, ktrac, krow, paphp1, papp1, &
                         ptte, ptm1, pxtm1, pxtte)

    !
    ! This subroutine calls the different parts of the chemistry
    ! module.
    !
    ! It is called by xtdriver (module mo_tracer) which is called by 'physc'
    !
    USE mo_kind,      ONLY : dp
    USE mo_sub_echam, ONLY : radionucl_sink
    IMPLICIT NONE

    ! Scalar arguments
    INTEGER :: kproma, kbdim, klev, klevp1, ktrac, krow

    ! Array arguments
    REAL(dp):: paphp1(kbdim,klevp1), papp1(kbdim,klev), ptte(kbdim,klev),  &
               ptm1(kbdim,klev)
    REAL(dp):: pxtm1(kbdim,klev,ktrac), pxtte(kbdim,klev,ktrac)

    !
    ! commonly used processes calculated by ECHAM:
    !
    CALL radionucl_sink (kproma, kbdim, klev, krow, pxtm1, pxtte)

  END SUBROUTINE call_chem1
!------------------------------------------------------------------------------
  SUBROUTINE call_chem2 (kproma, kbdim, klev,  klevp1, ktrac, krow, &
                         paphp1, papp1, ptte,  ptm1,                &
                         pxtm1,  pxtte,                             &
                         pxlm1,  pxlte, pxim1, pxite                )
    !
    ! similar to call_chem1, but called from other place within physc
    USE mo_kind,      ONLY: dp
    USE mo_aero,      ONLY: lham, lodiag
    USE mo_aero_diag, ONLY: aero_diag

    IMPLICIT NONE

    ! Scalar arguments
    INTEGER :: kproma, kbdim, klev, klevp1, ktrac, krow

    ! Array arguments
    REAL(dp):: paphp1(kbdim,klevp1), papp1(kbdim,klev), ptte(kbdim,klev),  &
               ptm1(kbdim,klev)
    REAL(dp):: pxlm1(kbdim,klev),       pxlte(kbdim,klev), &
               pxim1(kbdim,klev),       pxite(kbdim,klev)
    REAL(dp):: pxtm1(kbdim,klev,ktrac), pxtte(kbdim,klev,ktrac)

    !--- Extended aerosol diagnostics:

    IF (lham .AND. lodiag) CALL aero_diag(kproma, kbdim, & 
                                          klev,   krow,  &
                                          pxtm1,  pxtte, &
                                          pxlm1,  pxlte, &
                                          pxim1,  pxite, &
                                          papp1,  paphp1 )

  END SUBROUTINE call_chem2
!==============================================================================
  SUBROUTINE call_diagn(krow)
  !
  ! This routine is called by xtdiagn (mo_tracer) called at the end of physc
  !  physc. Diagnostics at the end of a time step may be called from here
  !


    USE mo_mvstream, ONLY: acc_mvstream

    IMPLICIT NONE

    INTEGER :: krow

    CALL acc_mvstream (krow, ltrac=.TRUE.)
    CALL acc_mvstream (krow, ltrac=.FALSE.)

 END SUBROUTINE call_diagn
!==============================================================================
  SUBROUTINE call_init_tracers
  !  
  ! This routine is called from xtini to allow initialization of tracer
  ! variables besides initialization with constant values or from the
  ! rerun file.
  !
  ! Base the decision whether to initialize on the following conditions:
  !
  ! trlist% ti(jt)% init   == 0       (not initialized so far)
  ! trlist% ti(jt)% ninit: >= INITIAL (initialisation requested)
  !
  ! Set trlist%ti(jt) to a value =/0 afterwards.
  !
  END SUBROUTINE call_init_tracers
!==============================================================================
  SUBROUTINE call_read_bcond
  !
  ! This routine is called from 'stepon'. Routines which read forcing
  ! data may be called from here.
  !
  ! Modifications:
  ! --------------
  ! Philip, Stier, MPI-MET, Hamburg                2001-2003
  ! (Added call to emission routines for the HAM aerosol model)
  ! 
  ! Martin Werner, MPI-BGC, Jena                   2003
  ! (Added call to emission routines for the MPI-BGC dust model)
  !
  ! Silvia Kloster, MPI-MET, Hamburg               2004
  ! (Added AEROCOM/NIES emission routines)
  ! 

    USE mo_tracer,           ONLY: trlist
    USE mo_time_control,     ONLY: lstart,       lresume,       &
                                   lstop,        l_putrerun,    &
                                   current_date, previous_date, &
                                   get_date_components
    USE mo_chem_bcond,       ONLY: xt_read_chem_fields,         &
                                   xt_read_dust_emiss,          &
                                   xt_volc_emiss_ini
    USE mo_chem_bcond_aerocom, ONLY:  xt_expl_volc_emiss_ini_aerocom ,&
                                      xt_cont_volc_emiss_ini_aerocom
    USE mo_chem_bcond_nies,    ONLY:  convert_nies_emissions_annual, &
                                      convert_nies_emissions_monthly

    USE mo_xtsurf_init,      ONLY: xtsurf_init
    USE mo_xtsurf_drydep,    ONLY: xtsurf_init_drydep
    USE mo_aero,             ONLY: lham, ndust, nemiss
    USE mo_aero_mem,         ONLY: input
    USE mo_chem_bcond_io,    ONLY: xt_read_emiss_fields_so2,     &
                                   xt_read_emiss_fields_fossil,  &
                                   xt_read_emiss_fields_bio,     &
                                   xt_read_emiss_fields_soa,     &
                                   xt_read_emiss_fields_terr,    &
                                   xt_read_emiss_fields_wildfire,&
                                   xt_read_emiss_fields_sea,     &
                                   xt_read_emiss_fields_biofuel, &
                                   xt_read_emiss_fields_ffires,  &
                                   xt_read_emiss_fields_so2ann,  &
                                   xt_read_emiss_fields_bcfoss,  &
                                   xt_read_emiss_fields_annual,  &
                                   xt_read_emiss_fields_m42,     &
                                   xt_read_emiss_fields_m63,     &
                                   xt_read_emiss_fields_m1x1
    USE mo_aero_tools,       ONLY: calc_daylength
    USE mo_bgc_dust_emis,    ONLY: bgc_dust_initialize,         &
                                   bgc_read_annual_fields,      &
                                   bgc_read_fpar_field,         &
                                   bgc_prestat,                 &
                                   bgc_trastat
    USE mo_mpi,              ONLY: p_parallel_io
    USE mo_exception,        ONLY: finish
    USE mo_doctor,           ONLY: nout

! uh+:
    USE mo_control,   ONLY: lcn, lcl36, lgeom
    USE mo_be10,      ONLY: read_bcond_be10, read_bcond_be10_3d, &
                            read_cl36_bomb, read_cl36_bomb2
! uh-



    IMPLICIT NONE

    INTEGER :: ithisyear, ilastyear, ithismonth,  ilastmonth, ithisday, ilastday, &
               ithishour, ilasthour, ithisminute, ithissecond

    LOGICAL :: lnewmonth, lnewday, lex

    CHARACTER(len=1024) :: cyear, cname

    !--- 1) Get time information and check for new month:

    CALL get_date_components (previous_date, month=ilastmonth, year=ilastyear,     &
                              day=ilastday,  hour=ilasthour                        )
    CALL get_date_components (current_date,  month=ithismonth, year=ithisyear,     &
                              day=ithisday,  hour=ithishour,   minute=ithisminute, &
                              second=ithissecond                                   )

    lnewmonth=ilastmonth/=ithismonth

    !--- Read HAM emissions:

    IF (lham) THEN

       !--- 2) Call initialization of the dry deposition routines:

       CALL xtsurf_init (ithismonth,ilastmonth,ithisyear)

       IF (ANY(trlist%ti(:)%ndrydep==2)) CALL xtsurf_init_drydep


       !--- 3) Call initialization of pararmeters for the chemistry:

       CALL calc_daylength


       !--- 4) Read emission fields and offline tracers for the Hamburg Aerosol Model

       !--- 4.1) Files to be read at the start of a run:

       IF (lstart) THEN 

          !--- Read and initialize volcanic emissions:

          IF (nemiss==1)CALL xt_volc_emiss_ini

          IF (nemiss==2 .OR. nemiss==3) THEN 
             CALL xt_expl_volc_emiss_ini_aerocom
             CALL xt_cont_volc_emiss_ini_aerocom
          END IF

          !--- Read offline emission fields and store in stream:
!          IF (nemiss==1)  CALL xt_read_emiss_fields('emiss_fields_annual_1x1.nml',input,0,2)
! nemiss==1 not yet tested!!!
          IF (nemiss==1)  CALL xt_read_emiss_fields_annual
          IF (nemiss==2 ) THEN ! AEROCOM Emissions fields
             CALL xt_read_emiss_fields_so2
             CALL xt_read_emiss_fields_fossil
             CALL xt_read_emiss_fields_bio
          END IF
          IF (nemiss==3) THEN ! NIES Emission fields
             IF (p_parallel_io) THEN

                WRITE(cyear,fmt='(I4)') ithisyear
                cname='emiss_nies_annual_SO2_fossil_fuel_'//TRIM(cyear)//'.nc'
                INQUIRE(file=TRIM(cname), exist=lex)
                IF (.NOT. lex) CALL finish('call_submodels','namelist '//TRIM(cname)//' does not exist')
             ENDIF

             CALL xt_read_emiss_fields_so2ann(cname)
             IF (p_parallel_io) THEN  

                WRITE(cyear,fmt='(I4)') ithisyear
                cname='emiss_nies_annual_BC_fossil_fuel_'//TRIM(cyear)//'.nc'
                INQUIRE(file=TRIM(cname), exist=lex)
                IF (.NOT. lex) CALL finish('call_submodels','namelist '//TRIM(cname)//' does not exist')
             ENDIF

             CALL xt_read_emiss_fields_bcfoss(cname)


             CALL convert_nies_emissions_annual

          END IF
       END IF

       !--- Input and initialization for dust emissions:

       SELECT CASE (ndust)

       CASE (1) 

          !--- Dust emissions LSCE:

          IF(lstart) THEN
             CALL xt_read_dust_emiss
          END IF

       CASE (2)

          !--- Dust emissions MPI BGC:

          IF(lstart .OR. lresume) THEN
             CALL bgc_prestat                  ! read in dust emission statistics
             CALL bgc_read_annual_fields       ! read in annual fields
             CALL bgc_dust_initialize          ! initialize dust emission scheme
          END IF

       END SELECT

       !--- 4.2) Files to be read monthly:

       IF (lstart .OR. lnewmonth) THEN

          !--- Read offline emission fields and store in stream:

          IF (nemiss==1) THEN 
              CALL xt_read_emiss_fields_m1x1(ithismonth)
              CALL xt_read_emiss_fields_m42(ithismonth)
              CALL xt_read_emiss_fields_m63(ithismonth)

!             CALL xt_read_emiss_fields('emiss_fields_monthly_1x1.nml',input,ithismonth,2)
!
!             CALL xt_read_emiss_fields('emiss_fields_monthly_T42.nml',input,ithismonth,2)
!
!             CALL xt_read_emiss_fields('emiss_fields_monthly_T63.nml',input,ithismonth,2)

          END IF

          IF (nemiss==2) THEN  ! AEROCOM emissions 2000

              CALL xt_read_emiss_fields_soa(ithismonth)
              CALL xt_read_emiss_fields_terr(ithismonth)
              CALL xt_read_emiss_fields_sea(ithismonth)

             !--- Create wildfire filename:

             IF (p_parallel_io) THEN

                WRITE(cyear,fmt='(I4)') ithisyear

                cname='emiss_aerocom_monthly_wildfire_'//TRIM(cyear)//'.nc'

                INQUIRE(file=TRIM(cname), exist=lex)

                !--- If year is not present check for climatological file:

                IF (.NOT. lex) THEN

                   WRITE(nout,*) '----------------------------------------------------------'
                   WRITE(nout,*) '--- Warning: Missing HAM emission file:                ---'
                   WRITE(nout,*) '---'
                   WRITE(nout,*) '---          ',TRIM(cname)

                   cname='emiss_aerocom_monthly_wildfire_CLIM.nc'

                   INQUIRE(file=TRIM(cname), exist=lex)

                   IF (lex) THEN

                      WRITE(nout,*) '---'
                      WRITE(nout,*) '---          using'
                      WRITE(nout,*) '---'
                      WRITE(nout,*) '---          ',TRIM(cname)
                      WRITE(nout,*) '---'
                      WRITE(nout,*) '---          instead!                                  ---'
                      WRITE(nout,*) '----------------------------------------------------------'

                   ELSE

                      CALL finish('call_submodels','namelist '//TRIM(cname)//' does not exist')

                   END IF

                END IF

             END IF

             !--- Read wildfire emissions:

             CALL xt_read_emiss_fields_wildfire(cname,ithismonth)

          END IF

          IF (nemiss==3) THEN ! NIES Emission fields

!             CALL xt_read_emiss_fields('emiss_aerocom_monthly_SOA.nml',input,ithismonth,2)
!
!             CALL xt_read_emiss_fields('emiss_fields_monthly_dms_terr.nml',input,ithismonth,2)
!
!             CALL xt_read_emiss_fields('emiss_fields_monthly_dms_sea.nml',input,ithismonth,2)
              CALL xt_read_emiss_fields_soa(ithismonth)
              CALL xt_read_emiss_fields_terr(ithismonth)
              CALL xt_read_emiss_fields_sea(ithismonth)


             IF (p_parallel_io) THEN

                WRITE(cyear,fmt='(I4)') ithisyear

                cname='emiss_nies_monthly_BC_biofuel_'//TRIM(cyear)//'.nc'
                INQUIRE(file=TRIM(cname), exist=lex)
                IF (.NOT. lex) CALL finish('call_submodels','namelist '//TRIM(cname)//' does not exist')

             ENDIF

!             CALL xt_read_emiss_fields(cname,input,ithismonth,2)
             CALL xt_read_emiss_fields_biofuel(cname,ithismonth)

             IF (p_parallel_io) THEN

                WRITE(cyear,fmt='(I4)') ithisyear

                cname='emiss_nies_monthly_BC_ffires_'//TRIM(cyear)//'.nc'
                INQUIRE(file=TRIM(cname), exist=lex)
                IF (.NOT. lex) CALL finish('call_submodels','namelist '//TRIM(cname)//' does not exist')

             ENDIF

!             CALL xt_read_emiss_fields(cname,input,ithismonth,2)
             CALL xt_read_emiss_fields_ffires(cname,ithismonth)

             CALL convert_nies_emissions_monthly

          ENDIF

          !--- Read offline chemical species and store in psuedo tracer:

          CALL xt_read_chem_fields(ithismonth)

       END IF

       !--- Read monthly FPAR fields for MPI BGC dust emission scheme:

       IF (lstart .OR. lresume .OR. lnewmonth ) THEN

          IF (ndust==2) CALL bgc_read_fpar_field(ithismonth)

       END IF

       !--- 4.3) Routines called at the end of all calculations:

       IF (l_putrerun .OR. lstop) THEN

          !--- BGC dust emission statistics:

          IF (ndust==2) CALL bgc_trastat(ithisyear, ithismonth,  ithisday,   &
               ithishour, ithisminute, ithissecond )

       END IF

    END IF


!uh+

  IF(lstart.OR.lnewmonth) THEN
!     IF(lcn) CALL read_bcond_be10(ithisyear, ithismonth)
     IF(lcn) THEN
        IF (lgeom) THEN
           CALL read_bcond_be10_3d(ithisyear, ithismonth)
        ELSE
           CALL read_bcond_be10(ithisyear, ithismonth)
        END IF
     END IF
  END IF

! for the cl36 bomb run

!  IF (lcn.AND.lcl36) THEN
!     IF (lstart.OR.lresume) CALL read_cl36_bomb
!  END IF
!  lnewday=ilastday/=ithisday

!  IF (lcn.AND.lcl36) THEN
!     IF (ithisyear==1952.or.ithisyear==1954.or.ithisyear==1956     &
!          .or.ithisyear==1958.or.ithisyear==1962.or.ithisyear==1968    &
!          .or.ithisyear==1970.or.ithisyear==1971.or.ithisyear==1974) THEN

!        IF (lnewday) THEN
!           CALL read_cl36_bomb2(ithisday,ithismonth,ithisyear)
!        END IF

!     END IF
!  END IF

!uh-



  END SUBROUTINE call_read_bcond
!==============================================================================
  SUBROUTINE call_chem_bcond (kproma, kbdim,  klev,  klevp1, krow,   ptsw,   &
                              pgeom1, pxtems, paphp1, pxtte, loland, pseaice, &
                              lpglac, ptslm1)

    !
    ! This is the interface routine between ECHAM's standard tracer treatment
    ! and the chemistry routines.
    !
    ! This routine is called from vdiff
    !
    ! M. Schultz and Hans-Stefan Bauer, MPI Hamburg, November 2000
    ! Philip Stier (adapted to HAM),    MPI Hamburg,          2001-2003
    ! Silvia Kloster, MPI-MET, Hamburg               2004
    ! (Added AEROCOM/NIES emission routines)
    !

    USE mo_kind,          ONLY: dp
    USE mo_aero,          ONLY: lham, ndust, nseasalt, nemiss
    USE mo_tracer,        ONLY: ntrac
    USE mo_exception,     ONLY: finish
    USE mo_bgc_dust_emis, ONLY: dust_emissions_bgc
    USE mo_chem_bcond,    ONLY: dms_emissions,             &
                                piston_velocity,           &
                                so2_emissions,             &
                                volcanic_so2_emissions,    &
                                carbon_emissions,          &
                                seasalt_emissions_lsce,    &
                                seasalt_emissions_monahan, &
                                dust_emissions_lsce
    USE mo_chem_bcond_aerocom, ONLY: so2_emissions_aerocom, &
                                     carbon_emissions_aerocom, &
                                     cont_volc_so2_emissions_aerocom, &
                                     expl_volc_so2_emissions_aerocom
    USE mo_chem_bcond_nies,    ONLY: carbon_emissions_nies, so2_emissions_nies


! uh+
    USE mo_be10,           ONLY: chem_bcond_be10, chem_bcond_pb210, chem_cl36_bomb
    USE mo_control,        ONLY: lcn, lpb210, lcl36, lbomb
    USE mo_mpi,            ONLY: p_pe
    USE mo_time_control,   ONLY: get_date_components, current_date
! uh-



    !--- Arguments:

    INTEGER, INTENT(in)    :: kproma, kbdim, klev, klevp1, krow
    REAL(dp),INTENT(in)    :: ptsw(kbdim)
    LOGICAL, INTENT(in)    :: loland(kbdim)

 !uh+
    LOGICAL, INTENT(in)    :: lpglac(kbdim)
  !uh-


    REAL(dp),INTENT(in)    :: pseaice(kbdim) ,         &
                              paphp1(kbdim,klevp1),    &
                              pgeom1(kbdim,klev)
  ! uh+
    REAL(dp), INTENT(in)       :: ptslm1(kbdim)

  ! uh-

    REAL(dp),INTENT(inout) :: pxtte(kbdim,klev,ntrac), &
                              pxtems(kbdim,ntrac)

    !--- Call HAM emissions:

    IF (lham) THEN

       CALL piston_velocity(kproma, kbdim, krow, loland , ptsw)

       CALL dms_emissions(kproma, kbdim, krow, loland, pseaice, pxtems)
    
       SELECT CASE (nemiss)
       
       CASE (1)

          CALL so2_emissions(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)
       
          CALL carbon_emissions(kproma, kbdim, krow, pxtems)

          CALL volcanic_so2_emissions(kproma, kbdim, klev, klevp1, krow, paphp1, pxtte, pgeom1)

       CASE (2)

          CALL so2_emissions_aerocom(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)

          CALL carbon_emissions_aerocom(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)
       
          CALL cont_volc_so2_emissions_aerocom(&
               kproma, kbdim, klev, klevp1, krow, paphp1, pxtte, pgeom1)       
       
          CALL expl_volc_so2_emissions_aerocom(&
               kproma, kbdim, klev, klevp1, krow, paphp1, pxtte, pgeom1)

       CASE (3)

          CALL so2_emissions_nies(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)
       
          CALL carbon_emissions_nies(kproma, kbdim, klev, klevp1, krow, paphp1, pxtems, pxtte)
       
          CALL cont_volc_so2_emissions_aerocom(&
               kproma, kbdim, klev, klevp1, krow, paphp1, pxtte, pgeom1)       
       
          CALL expl_volc_so2_emissions_aerocom(&
               kproma, kbdim, klev, klevp1, krow, paphp1, pxtte, pgeom1)

       CASE DEFAULT
       
          CALL finish('call_chem_bcond','selected emission inventory not implemented')
       
       END SELECT


       !--- Sea Salt emisssions:
    
       SELECT CASE (nseasalt)

       CASE (1)

          CALL seasalt_emissions_monahan(kproma, kbdim, krow, loland, pseaice, pxtems)

       CASE(2)

          CALL seasalt_emissions_lsce(kproma, kbdim, krow, pseaice, pxtems)

       CASE (0)

          ! No emissions

       CASE DEFAULT

          CALL finish('call_chem_bcond','selected sea salt emission scheme not implemented')

       END SELECT

       
       !--- Dust emisssions:

       SELECT CASE (ndust)

       CASE (1)

          CALL dust_emissions_lsce(pxtems, kproma, kbdim, krow)

       CASE (2) 

          CALL dust_emissions_bgc(pxtems, kproma, kbdim, krow)

       CASE (0) 

          ! No emissions

       CASE DEFAULT

          CALL finish('call_chem_bcond','selected dust emission scheme not implemented')

       END SELECT

    END IF

!uh+
    IF (lcn) THEN
       CALL chem_bcond_be10(kbdim,klev,kproma,krow,pxtte)
       IF(lpb210) THEN
          CALL chem_bcond_pb210(kbdim,klev,klevp1,kproma,krow, pxtte,paphp1,loland,lpglac,ptslm1)
       END IF

! for the cl36 bomb run
       IF (lbomb.AND.lcl36) THEN
          
          CALL chem_cl36_bomb(pxtte,paphp1, kbdim, kproma, klev, klevp1, krow)
          
       END IF
    END IF
!uh-

  END SUBROUTINE call_chem_bcond
!==============================================================================
