SUBROUTINE m7(kproma,kbdim,   klev,   krow,   &  ! ECHAM indices
              papp1, prelhum, ptp1,   pdz,    &  !   "   thermodynamics
              pso4g, pso2g,   paerml, paernl, &  !  M7   tracers
              pm6rp, pm6dry,  prhop,  pww      ) !   "   aerosol properties
  !
  !   ****m7* Aerosol model for the system so4,bc,oc,ss,dust in 7 modes.
  !
  !   Authors:
  !   ---------
  !   E. Vignati, JRC/EI     (original source)                   2000
  !   P. Stier, MPI          (f90-version, changes, comments)    2001 
  !
  !   Purpose
  !   ---------
  !   Aerosol model for the system so4,bc,oc,ss,dust in 7 modes.
  !
  !   Externals
  !   ---------
  !
  !   *m7_averageproperties* 
  !       calculates the average mass for all modes and the particle 
  !       dry radius and density for the insoluble modes.
  !    
  !   *m7_equiz*   
  !       calculates the ambient radius of sulphate particles
  !
  !   *m7_equimix* 
  !       calculates the ambient radius of so4,bc,oc (dust) particles
  !
  !   *m7_equil* 
  !       calculates the ambient radius of so4,ss particles 
  !
  !   *m7_dgas*    
  !       calculates the sulfate condensation on existing particles
  !
  !   *m7_dnum*    
  !       calculates new gas phase sulfate and aerosol numbers and masses
  !       after condensation, nucleation and coagulation over one timestep
  !
  !   *m7_dconc*   
  !       repartitions aerosol number and mass between the
  !       the modes to account for condensational growth and the formation
  !       of an accumulation mode from the upper tail of the aitken mode and 
  !       of a coarse mode from the upper tail of the accumulation mode
  !

  USE mo_kind,    ONLY: dp
  USE mo_aero_m7, ONLY: lsnucl, lscoag, lscond,         &
                        nmod,   nss,    nsol,   naermod

  IMPLICIT NONE 

  !--- Parameter list:
  !
  !  papp1      = atmospheric pressure at time t+1 [Pa]
  !  prelhum    = atmospheric relative humidity [% (0-1)]
  !  ptp1       = atmospheric temperature at time t+1 [K]
  !  pso4g      = mass of gas phase sulfate [molec. cm-3]
  !  pso2g      = mass of gas phase SO2 [molec. cm-3]
  !  paerml     = total aerosol mass for each compound 
  !               [molec. cm-3 for sulphate and ug m-3 for bc, oc, ss, and dust]
  !  paernl     = aerosol number for each mode [cm-3]
  !  prhop      = mean mode particle density [g cm-3]
  !  pdz        = layer thickness dz for diagnostics [m]
  !  pm6rp      = mean mode actual radius (wet radius for soluble modes 
  !               and dry radius for insoluble modes) [cm]
  !  pm6dry     = dry radius for soluble modes [cm]
  !  pww        = aerosol water content for each mode [kg(water) m-3(air)]
  !
  !--- Local variables:
  !
  !  zttn       = average mass for single compound in each mode 
  !               [in molec. for sulphate and in ug for bc, oc, ss, and dust]
  !  zhplus     = number of h+ in mole [???] (kg water)-1
  !  zso4_x     = mass of sulphate condensed on insoluble mode x [molec. cm-3]
  !               (calculated in dgas used in concoag)


  ! Parameters:

  INTEGER :: kproma, kbdim, klev, krow

  REAL(dp):: prelhum(kbdim,klev),        papp1(kbdim,klev),           &
             ptp1(kbdim,klev),           pdz(kbdim,klev),             &
             pso4g(kbdim,klev),          pso2g(kbdim,klev)
 
  REAL(dp):: paerml(kbdim,klev,naermod), paernl(kbdim,klev,nmod),     &
             pm6rp(kbdim,klev,nmod),     pm6dry(kbdim,klev,nsol),     &
             prhop(kbdim,klev,nmod),     pww(kbdim,klev,nmod)

  ! Local variables:

  REAL(dp):: zso4_5(kbdim,klev),         zso4_6(kbdim,klev),          &
             zso4_7(kbdim,klev)

  REAL(dp):: zhplus(kbdim,klev,nss)

  REAL(dp):: zttn(kbdim,klev,naermod)

  !
  !--- 0) Initialisations: -------------------------------------------------
  !
  zhplus(:,:,:) = 0._dp
  pm6dry(:,:,:) = 0._dp 
  pm6rp(:,:,:)  = 0._dp
  zttn(:,:,:)   = 0._dp
  prhop(:,:,:)  = 0._dp
  pww(:,:,:)    = 0._dp 
  zso4_5(:,:)   = 0._dp
  zso4_6(:,:)   = 0._dp
  zso4_7(:,:)   = 0._dp
  !
  !
  !--- 1) Calculation of particle properties under ambient conditions: -----
  !
  !--- 1.1) Calculate mean particle mass for all modes 
  !         and dry radius and density for the insoluble modes.
  !
  CALL m7_averageproperties(kproma, kbdim, klev, paernl, paerml, zttn, pm6rp, prhop)
  !
  !--- 1.2) Calculate ambient count median radii and density 
  !         for lognormal distribution of particles.
  !
  !         Sulfate particles:
  !
  CALL m7_equiz(kproma,  kbdim, klev,   &
                papp1,   zttn,  ptp1,   &
                prelhum, pm6rp, pm6dry, &
                prhop,   pww,   paernl  )
  !         
  !         Mixed particles with sulfate, b/o carbon and dust: 
  !
  CALL m7_equimix(kproma,  kbdim, klev,   &
                  papp1,   zttn,  ptp1,   &
                  prelhum, pm6rp, pm6dry, &
                  prhop,   pww,   paernl  )
  !
  !         Accumulation and coarse mode particles in presence of
  !         sea salt particles:
  !
  CALL m7_equil(kproma, kbdim,  klev,   prelhum, paerml, paernl, &
                pm6rp,  pm6dry, zhplus, pww,     prhop,  ptp1    )
  !
  !--- 2) Calculate changes in aerosol mass and gas phase sulfate ----------
  !       due to sulfate condensation:
  !       No change in particle mass/number relationships.
  !
  IF (lscond) CALL m7_dgas(kproma, kbdim,  klev,  krow, &
                           pso4g,  paerml, paernl,      & 
                           ptp1,   papp1,  pm6rp,       &
                           zso4_5, zso4_6, zso4_7,      &
                           pdz                          )
  !
  !
  !--- 3) Calculate change in particle number concentrations ---------------
  !       due to nucleation and coagulation:
  !       Change particle mass/number relationships.
  !
  IF (lsnucl.OR.lscoag) CALL m7_dnum(kproma, kbdim,   klev,   krow,  &
                                     pso4g,  paerml,  paernl, ptp1,  &
                                     papp1,  prelhum, pm6rp,  prhop, &
                                     zso4_5, zso4_6,  zso4_7, pdz    )
  !
  !
  !--- 4) Recalculation of particle properties under ambient conditions: ---
  !
  !--- 4.1) Recalculate mean masses for all modes 
  !         and dry radius and density for the insoluble modes.
  !
  CALL m7_averageproperties(kproma, kbdim, klev, paernl, paerml, zttn, pm6rp, prhop)
  !
  !--- 4.2) Calculate ambient count median radii and density 
  !         for lognormal distribution of particles.
  !
  !         Sulfate particles:
  !
  CALL m7_equiz(kproma,  kbdim, klev,   &
                papp1,   zttn,  ptp1,   &
                prelhum, pm6rp, pm6dry, &
                prhop,   pww,   paernl  )
  !
  !         Mixed particles with sulfate, b/o carbon and dust:
  !
  CALL m7_equimix(kproma,  kbdim, klev,   &
                  papp1,   zttn,  ptp1,   &
                  prelhum, pm6rp, pm6dry, &
                  prhop,   pww,   paernl  )
  !
  !         Accumulation and coarse mode particles in presence of
  !         sea salt particles:  
  ! 
  CALL m7_equil(kproma, kbdim,  klev,   prelhum, paerml, paernl,  &
                pm6rp,  pm6dry, zhplus, pww,     prhop,  ptp1     )
  !
  !--- 5) Repartitition particles among the modes: -------------------------
  !
  IF (lscond.OR.lscoag) THEN
     
     CALL m7_dconc(kproma, kbdim, klev, paerml, paernl, pm6dry)
     
  END IF
  !
  !--- 6) Recalculation of particle properties under ambient conditions: ---
  !
  !--- 6.1) Calculate mean particle mass for all modes 
  !         and dry radius and density for the insoluble modes:
  !
  CALL m7_averageproperties(kproma, kbdim, klev, paernl, paerml, zttn, pm6rp, prhop)
  !
  !--- 6.2) Calculate ambient count median radii and density 
  !         for lognormal distribution of particles.
  !
  !         Sulfate particles:
  !
  CALL m7_equiz(kproma,  kbdim, klev,   &
                papp1,   zttn,  ptp1,   &
                prelhum, pm6rp, pm6dry, &
                prhop,   pww,   paernl  )
  !
  !         Mixed particles with sulfate, b/o carbon and dust: 
  !
  CALL m7_equimix(kproma,  kbdim, klev,   &
                  papp1,   zttn,  ptp1,   &
                  prelhum, pm6rp, pm6dry, &
                  prhop,   pww,   paernl  )
  !
  !         Accumulation and coarse mode particles in presence of
  !         sea salt particles:
  !
  CALL m7_equil(kproma, kbdim,  klev, prelhum, paerml, paernl, &
                pm6rp,  pm6dry, zhplus, pww,   prhop,  ptp1    )
  !
END SUBROUTINE m7
