SUBROUTINE m7_interface(kproma, kbdim, klev, krow, ktrac,  &  ! ECHAM indices 
                        papp1,    paphp1,                  &  !   "   thermodynamics
                        ptm1,     ptte,                    &  !   "     "
                        pqm1,     pqte,                    &  !   "     " 
                        paclc,                             &  !   "   cloud
                        pxtm1,    pxtte                     ) !   "   tracer
  !
  !**** *m7_interface*  serves as interface between m7 and echam5
  !
  !      Authors:
  !      --------
  !      J. Feichter and J. Wilson                         may  1997
  !      Adopted to m7 by J. Feichter and E. Vignati       oct. 2000
  !      P. Stier                                               2001
  !
  !      Purpose:
  !      --------
  !      This routine serves as interface between ECHAM and m7. 
  !
  !**    Interface:
  !      ----------
  !      *m7_interface*   is called from   *physc*
  !
  !      Method:
  !      -------
  !      The interface takes the mixing ratios of the compounds (xtm1) at
  !      the time t-dt, adds the tendencies (xte) of the preceeding proceses,
  !      converts the units and hands them over to m7. 
  !      After the call of m7 the returning concentrations are reconverted
  !      to mixing ratios, the tendencies of the mixing ratios are calculated
  !      and handed over to ECHAM.
  !

  USE mo_kind,           ONLY: dp
  USE mo_geoloc,         ONLY: gboxarea_2d
  USE mo_constants,      ONLY: rd, vtmpc1, g
  USE mo_time_control,   ONLY: time_step_len
  USE mo_convect_tables, ONLY: tlucuaw, jptlucu1, jptlucu2
  USE mo_exception,      ONLY: finish
  USE mo_doctor,         ONLY: NERR
  USE mo_aero_m7
  USE mo_aero_trac
  USE mo_aero_mem,       ONLY: rdry, rwet, densaer, relhum
  USE mo_aero_massfix,   ONLY: xt_borrow

  IMPLICIT NONE 

  !--- Parameter list:
  !
  !  zww = aerosol water content for each mode [kg(water) m-3(air)]
  !  zdz = layer thickness for conversions for diagnostics [m]
  !
  
  ! Parameters:

  INTEGER :: kproma, kbdim, klev, ktrac, krow

  REAL(dp):: papp1(kbdim,klev),         paclc(kbdim,klev),&
             ptm1(kbdim,klev),          ptte(kbdim,klev), &
             pqm1(kbdim,klev),          pqte(kbdim,klev)

  REAL(dp):: paphp1(kbdim,klev+1)

  REAL(dp):: pxtm1(kbdim,klev,ktrac),   pxtte(kbdim,klev,ktrac)

  ! Streams and stream elements:

  REAL(dp),        POINTER :: rdry_p(:,:,:)
  REAL(dp),        POINTER :: rwet_p(:,:,:)
  REAL(dp),        POINTER :: densaer_p(:,:,:)

  ! Local variables:

  INTEGER :: it,jl,jk,jc,jmod,ilevp1

  REAL(dp):: ztmst, zqtmst, zqp1, zqsp1, zqp1_amb

  REAL(dp):: zfac,                       zqfac,                   &
            zfacm,                      zqfacm,                  &
            zfacn,                      zqfacn,                  &
            zrhofacm,                   zqrhofacm,               &
            zrhofacn,                   zqrhofacn,               & 
            zrhofac,                    zqrhofac,                &
            zeps,                       zaclc

  REAL(dp):: zmass_pre(5),               zmass_post(5)

  REAL(dp):: zgboxarea(kbdim)

  REAL(dp):: zgso4(kbdim,klev),          zgso2(kbdim,klev),       &
            ztp1(kbdim,klev),           zrhoa(kbdim,klev),       &
            zrelhum(kbdim,klev),        zdpg(kbdim,klev),        &
            zdz(kbdim,klev)

  REAL(dp):: zaerml(kbdim,klev,naermod), zaernl(kbdim,klev,nmod), &
            zm6rp(kbdim,klev,nmod),     zm6dry(kbdim,klev,nsol), &
            zrhop(kbdim,klev,nmod),     zww(kbdim,klev,nmod)

  LOGICAL :: loabort(5)=.FALSE.

  !--- 0. Initialisations: -----------------------------------------------

  zmass_pre  = 0._dp
  zmass_post = 0._dp

  ztmst  = time_step_len
  zqtmst = 1._dp/time_step_len

  zeps=1.E-10_dp

  ilevp1=klev+1

  !--- Mass conserving correction of negative tracer values:

  CALL xt_borrow(kproma, kbdim,  klev,  ilevp1, ktrac, krow, &
                 papp1,  paphp1,                             &
                 pxtm1,  pxtte                               )

  !--- 1. Calculate necessary parameters:---------------------------------

  !--- Factor to transform mass sulfur in kg into molecules per kg:

  zfacm  = 6.022e+20_dp/32._dp

  !--- Factor to transform kg into micro gram:

  zfac   = 1.e09_dp

  !--- Factor to transform N/m**3 into N/cm**3:

  zfacn  = 1.0e-06_dp

  zqfac  = 1.0_dp/zfac
  zqfacm = 1.0_dp/zfacm
  zqfacn = 1.0_dp/zfacn

  !--- 2) Calculate ambient properties: ----------------------------------

  DO jk=1,klev
     DO  jl=1,kproma

        zqp1        =pqm1(jl,jk) +pqte(jl,jk) *ztmst
        ztp1(jl,jk) =ptm1(jl,jk) +ptte(jl,jk) *ztmst

        !--- 2.1) Calculate air density:
        !         (currently neglects volume occupied by liquid and ice water => physc)

        zrhoa(jl,jk)=papp1(jl,jk)/(ztp1(jl,jk)*rd*(1._dp+vtmpc1*zqp1))

        !--- 2.2) New calculation of the relative humidity:
        !         Only with respect to water.

        it          = NINT(ztp1(jl,jk)*1000._dp)
        it = MAX(MIN(it,jptlucu2),jptlucu1)

        zqsp1       = tlucuaw(it)/papp1(jl,jk)
        zqsp1       = MIN(zqsp1,0.5_dp)
        zqsp1       = zqsp1/(1._dp-vtmpc1*zqsp1)

        zaclc=MIN(paclc(jl,jk),1.0_dp-zeps)

        zqp1_amb=MAX( 0.0_dp , (zqp1-zqsp1*zaclc)/(1._dp-zaclc) )

        zrelhum(jl,jk)=zqp1_amb/zqsp1

        zrelhum(jl,jk)=MAX(0.0_dp,MIN(zrelhum(jl,jk),1.0_dp))

        !--- Air mass auxiliary variable:

        zdpg(jl,jk)=(paphp1(jl,jk+1)-paphp1(jl,jk))/g

        !--- Layer thickness zdz=dp/(rho*g) [m]:

        zdz(jl,jk)=zdpg(jl,jk)/zrhoa(jl,jk)

     ENDDO
!ham_ps:changed position to suppress loop-fusion on NEC-SX6
     relhum(1:kproma,jk,krow)=zrelhum(1:kproma,jk)

  ENDDO

  !--- 3) Convert units and add the tendencies of preceeding proceses: ----
  !       Transform:
  !         - mass of sulfur species from [mass(S)/mass(air)]
  !           into [molecules/cm+3]
  !         - mass of all other species into micro-gram/cubic-meter
  !         - particle numbers are transformed from [N/kg(air)]
  !           into [N/cm+3]
  !
  !--- 3.1 Gases:

  DO jk=1,klev
     DO jl=1,kproma

        !--- Calculate auxiliary factors:

        zrhofac =zrhoa(jl,jk)*zfac
        zrhofacm=zrhoa(jl,jk)*zfacm
        zrhofacn=zrhoa(jl,jk)*zfacn

        !--- 3.1 Gases:

        zgso2(jl,jk)=zrhofacm*(pxtm1(jl,jk,idt_so2)+pxtte(jl,jk,idt_so2)*ztmst)
        zgso4(jl,jk)=zrhofacm*(pxtm1(jl,jk,idt_so4)+pxtte(jl,jk,idt_so4)*ztmst)

        !--- 3.2 Particle mass:

        !--- 3.2.1) Sulfate mass:

        zaerml(jl,jk,iso4ns)=zrhofacm*(pxtm1(jl,jk,idt_ms4ns)+pxtte(jl,jk,idt_ms4ns)*ztmst)
        zaerml(jl,jk,iso4ks)=zrhofacm*(pxtm1(jl,jk,idt_ms4ks)+pxtte(jl,jk,idt_ms4ks)*ztmst)
        zaerml(jl,jk,iso4as)=zrhofacm*(pxtm1(jl,jk,idt_ms4as)+pxtte(jl,jk,idt_ms4as)*ztmst)
        zaerml(jl,jk,iso4cs)=zrhofacm*(pxtm1(jl,jk,idt_ms4cs)+pxtte(jl,jk,idt_ms4cs)*ztmst)
        
        !--- 3.2.2) Black Carbon:

        zaerml(jl,jk,ibcks)=zrhofac*(pxtm1(jl,jk,idt_mbcks)+pxtte(jl,jk,idt_mbcks)*ztmst)
        zaerml(jl,jk,ibcas)=zrhofac*(pxtm1(jl,jk,idt_mbcas)+pxtte(jl,jk,idt_mbcas)*ztmst)
        zaerml(jl,jk,ibccs)=zrhofac*(pxtm1(jl,jk,idt_mbccs)+pxtte(jl,jk,idt_mbccs)*ztmst)
        zaerml(jl,jk,ibcki)=zrhofac*(pxtm1(jl,jk,idt_mbcki)+pxtte(jl,jk,idt_mbcki)*ztmst)

        !--- 3.2.3) Organic Carbon:

        zaerml(jl,jk,iocks)=zrhofac*(pxtm1(jl,jk,idt_mocks)+pxtte(jl,jk,idt_mocks)*ztmst)
        zaerml(jl,jk,iocas)=zrhofac*(pxtm1(jl,jk,idt_mocas)+pxtte(jl,jk,idt_mocas)*ztmst)
        zaerml(jl,jk,ioccs)=zrhofac*(pxtm1(jl,jk,idt_moccs)+pxtte(jl,jk,idt_moccs)*ztmst)
        zaerml(jl,jk,iocki)=zrhofac*(pxtm1(jl,jk,idt_mocki)+pxtte(jl,jk,idt_mocki)*ztmst)

        !--- 3.2.4) Sea Salt:

        zaerml(jl,jk,issas)=zrhofac*(pxtm1(jl,jk,idt_mssas)+pxtte(jl,jk,idt_mssas)*ztmst)
        zaerml(jl,jk,isscs)=zrhofac*(pxtm1(jl,jk,idt_msscs)+pxtte(jl,jk,idt_msscs)*ztmst)

        !--- 3.2.5) Dust:

        zaerml(jl,jk,iduas)=zrhofac*(pxtm1(jl,jk,idt_mduas)+pxtte(jl,jk,idt_mduas)*ztmst)
        zaerml(jl,jk,iducs)=zrhofac*(pxtm1(jl,jk,idt_mducs)+pxtte(jl,jk,idt_mducs)*ztmst)
        zaerml(jl,jk,iduai)=zrhofac*(pxtm1(jl,jk,idt_mduai)+pxtte(jl,jk,idt_mduai)*ztmst)
        zaerml(jl,jk,iduci)=zrhofac*(pxtm1(jl,jk,idt_mduci)+pxtte(jl,jk,idt_mduci)*ztmst)

        !--- 3.3) Particle numbers:

        zaernl(jl,jk,inucs)=zrhofacn*(pxtm1(jl,jk,idt_nns)+pxtte(jl,jk,idt_nns)*ztmst)
        zaernl(jl,jk,iaits)=zrhofacn*(pxtm1(jl,jk,idt_nks)+pxtte(jl,jk,idt_nks)*ztmst)
        zaernl(jl,jk,iaccs)=zrhofacn*(pxtm1(jl,jk,idt_nas)+pxtte(jl,jk,idt_nas)*ztmst)
        zaernl(jl,jk,icoas)=zrhofacn*(pxtm1(jl,jk,idt_ncs)+pxtte(jl,jk,idt_ncs)*ztmst)
        zaernl(jl,jk,iaiti)=zrhofacn*(pxtm1(jl,jk,idt_nki)+pxtte(jl,jk,idt_nki)*ztmst)
        zaernl(jl,jk,iacci)=zrhofacn*(pxtm1(jl,jk,idt_nai)+pxtte(jl,jk,idt_nai)*ztmst)
        zaernl(jl,jk,icoai)=zrhofacn*(pxtm1(jl,jk,idt_nci)+pxtte(jl,jk,idt_nci)*ztmst)

     ENDDO
  ENDDO

  !--- Discard potential remaining negative values:

  zgso2(1:kproma,:)=MAX(zgso2(1:kproma,:),0._dp)
  zgso4(1:kproma,:)=MAX(zgso4(1:kproma,:),0._dp)

  zaerml(1:kproma,:,:)=MAX(zaerml(1:kproma,:,:),0._dp)
  zaernl(1:kproma,:,:)=MAX(zaernl(1:kproma,:,:),0._dp)

  !--- Sum total mass of all compounds for mass diagnostics:
        
  IF(lmass_diag) CALL sum_mass(zmass_pre)


  !--- 4) Call of m7: -----------------------------------------------------

  CALL m7(kproma,kbdim,   klev,   krow,   &  ! ECHAM indices
          papp1, zrelhum, ztp1,   zdz,    &  !   "   thermodynamics
          zgso4, zgso2,   zaerml, zaernl, &  !  M7   tracers
          zm6rp, zm6dry,  zrhop,  zww      ) !   "   aerosol properties 

  !--- 5) Reconvert masses and numbers into mixing ratios, other ----------
  !       quantities to SI units and calculate the tendencies (xtte):
  !

  DO jk=1,klev
     DO jl=1,kproma

        zqrhofac=zqfac/zrhoa(jl,jk)
        zqrhofacm=zqfacm/zrhoa(jl,jk)
        zqrhofacn=zqfacn/zrhoa(jl,jk)

        !--- 5.1 Gases:

        pxtte(jl,jk,idt_so2)=(zgso2(jl,jk)*zqrhofacm-pxtm1(jl,jk,idt_so2))*zqtmst
        pxtte(jl,jk,idt_so4)=(zgso4(jl,jk)*zqrhofacm-pxtm1(jl,jk,idt_so4))*zqtmst

        !--- 5.2) Particle mass:
        
        !--- 5.2.1) Sulfate mass:
        
        pxtte(jl,jk,idt_ms4ns)=(zaerml(jl,jk,iso4ns)*zqrhofacm-pxtm1(jl,jk,idt_ms4ns))*zqtmst
        pxtte(jl,jk,idt_ms4ks)=(zaerml(jl,jk,iso4ks)*zqrhofacm-pxtm1(jl,jk,idt_ms4ks))*zqtmst
        pxtte(jl,jk,idt_ms4as)=(zaerml(jl,jk,iso4as)*zqrhofacm-pxtm1(jl,jk,idt_ms4as))*zqtmst
        pxtte(jl,jk,idt_ms4cs)=(zaerml(jl,jk,iso4cs)*zqrhofacm-pxtm1(jl,jk,idt_ms4cs))*zqtmst

        !--- 5.2.2) Black Carbon:

        pxtte(jl,jk,idt_mbcks)=(zaerml(jl,jk,ibcks)*zqrhofac-pxtm1(jl,jk,idt_mbcks))*zqtmst
        pxtte(jl,jk,idt_mbcas)=(zaerml(jl,jk,ibcas)*zqrhofac-pxtm1(jl,jk,idt_mbcas))*zqtmst
        pxtte(jl,jk,idt_mbccs)=(zaerml(jl,jk,ibccs)*zqrhofac-pxtm1(jl,jk,idt_mbccs))*zqtmst
        pxtte(jl,jk,idt_mbcki)=(zaerml(jl,jk,ibcki)*zqrhofac-pxtm1(jl,jk,idt_mbcki))*zqtmst

        !--- 5.2.3) Organic Carbon:

        pxtte(jl,jk,idt_mocks)=(zaerml(jl,jk,iocks)*zqrhofac-pxtm1(jl,jk,idt_mocks))*zqtmst
        pxtte(jl,jk,idt_mocas)=(zaerml(jl,jk,iocas)*zqrhofac-pxtm1(jl,jk,idt_mocas))*zqtmst
        pxtte(jl,jk,idt_moccs)=(zaerml(jl,jk,ioccs)*zqrhofac-pxtm1(jl,jk,idt_moccs))*zqtmst
        pxtte(jl,jk,idt_mocki)=(zaerml(jl,jk,iocki)*zqrhofac-pxtm1(jl,jk,idt_mocki))*zqtmst

        !--- 5.2.4) Sea Salt:

        pxtte(jl,jk,idt_mssas)=(zaerml(jl,jk,issas)*zqrhofac-pxtm1(jl,jk,idt_mssas))*zqtmst
        pxtte(jl,jk,idt_msscs)=(zaerml(jl,jk,isscs)*zqrhofac-pxtm1(jl,jk,idt_msscs))*zqtmst

        !--- 5.2.5) Dust:

        pxtte(jl,jk,idt_mduas)=(zaerml(jl,jk,iduas)*zqrhofac-pxtm1(jl,jk,idt_mduas))*zqtmst
        pxtte(jl,jk,idt_mducs)=(zaerml(jl,jk,iducs)*zqrhofac-pxtm1(jl,jk,idt_mducs))*zqtmst
        pxtte(jl,jk,idt_mduai)=(zaerml(jl,jk,iduai)*zqrhofac-pxtm1(jl,jk,idt_mduai))*zqtmst
        pxtte(jl,jk,idt_mduci)=(zaerml(jl,jk,iduci)*zqrhofac-pxtm1(jl,jk,idt_mduci))*zqtmst

        !--- 5.3 Particle numbers:

        pxtte(jl,jk,idt_nns)=(zaernl(jl,jk,inucs)*zqrhofacn-pxtm1(jl,jk,idt_nns))*zqtmst
        pxtte(jl,jk,idt_nks)=(zaernl(jl,jk,iaits)*zqrhofacn-pxtm1(jl,jk,idt_nks))*zqtmst
        pxtte(jl,jk,idt_nas)=(zaernl(jl,jk,iaccs)*zqrhofacn-pxtm1(jl,jk,idt_nas))*zqtmst
        pxtte(jl,jk,idt_ncs)=(zaernl(jl,jk,icoas)*zqrhofacn-pxtm1(jl,jk,idt_ncs))*zqtmst
        pxtte(jl,jk,idt_nki)=(zaernl(jl,jk,iaiti)*zqrhofacn-pxtm1(jl,jk,idt_nki))*zqtmst
        pxtte(jl,jk,idt_nai)=(zaernl(jl,jk,iacci)*zqrhofacn-pxtm1(jl,jk,idt_nai))*zqtmst
        pxtte(jl,jk,idt_nci)=(zaernl(jl,jk,icoai)*zqrhofacn-pxtm1(jl,jk,idt_nci))*zqtmst

     ENDDO
  ENDDO

  !--- 6) Convert M7 quantities to SI units and store in streams:

  DO jmod=1, nmod

     rdry_p    => rdry(jmod)%ptr
     rwet_p    => rwet(jmod)%ptr
     densaer_p => densaer(jmod)%ptr

     !--- Mean mode density from [g/cm3] to [kg/m3]:
     
     densaer_p(1:kproma,:,krow)=zrhop(1:kproma,:,jmod)*1.E3_dp

     !--- Ambient Count Median Radius from [cm] to [m]:

     rwet_p(1:kproma,:,krow)=zm6rp(1:kproma,:,jmod)/100._dp

     !--- Dry Count Median Radius from [cm] to [m]:

     IF (nmod <=nsol) THEN 
        rdry_p(1:kproma,:,krow)=zm6dry(1:kproma,:,jmod)/100._dp
     ELSE
        rdry_p(1:kproma,:,krow)=zm6rp(1:kproma,:,jmod)/100._dp
     END IF

  END DO

  !--- Store diagnostic aerosol properties in pseudo-tracers:

  pxtm1(1:kproma,:,idt_mwans)=zww(1:kproma,:,1)/zrhoa(1:kproma,:)
  pxtm1(1:kproma,:,idt_mwaks)=zww(1:kproma,:,2)/zrhoa(1:kproma,:)
  pxtm1(1:kproma,:,idt_mwaas)=zww(1:kproma,:,3)/zrhoa(1:kproma,:)
  pxtm1(1:kproma,:,idt_mwacs)=zww(1:kproma,:,4)/zrhoa(1:kproma,:)

  !--- 7) Perform mass conservation check: 

  IF(lmass_diag) THEN

     !--- Sum total mass of all compounds for mass diagnostics:

     CALL sum_mass(zmass_post)

     !--- Perform mass conservation check:

     DO jc=1, 5

        IF( ABS(zmass_pre(jc)-zmass_post(jc)) > 0.100_dp*ABS(MAX(zmass_pre(jc),zmass_post(jc)))) THEN

           WRITE(NERR,*) 'm7_interface: m7 violates the mass conservation for compound number = ', jc, ">10%"

           IF (MAX(zmass_pre(jc),zmass_post(jc)) >0._dp) THEN
              WRITE(NERR,*) '              change in mass : ', &
                            (zmass_post(jc)-zmass_pre(jc))/MAX(zmass_pre(jc),zmass_post(jc))*100._dp,"%"
           ELSE
              WRITE(NERR,*) '              change in mass : ', &
                            zmass_post(jc)-zmass_pre(jc)
           END IF

           loabort(jc)=.TRUE.        
        
        ELSE IF( ABS(zmass_pre(jc)-zmass_post(jc)) > 0.010_dp*ABS(MAX(zmass_pre(jc),zmass_post(jc)))) THEN

           WRITE(NERR,*) 'm7_interface: m7 violates the mass conservation for compound number = ', jc, ">1%"

           IF (MAX(zmass_pre(jc),zmass_post(jc)) >0._dp) THEN
              WRITE(NERR,*) '              change in mass : ', &
                            (zmass_post(jc)-zmass_pre(jc))/MAX(zmass_pre(jc),zmass_post(jc))*100._dp,"%"
           ELSE
              WRITE(NERR,*) '              change in mass : ', &
                            zmass_post(jc)-zmass_pre(jc)
           END IF

        ELSE IF( ABS(zmass_pre(jc)-zmass_post(jc)) > 0.001_dp*ABS(MAX(zmass_pre(jc),zmass_post(jc)))) THEN

           WRITE(NERR,*) 'm7_interface: m7 violates the mass conservation for compound number = ', jc, ">0.1%"

           IF (MAX(zmass_pre(jc),zmass_post(jc)) >0._dp) THEN
              WRITE(NERR,*) '              change in mass : ', &
                            (zmass_post(jc)-zmass_pre(jc))/MAX(zmass_pre(jc),zmass_post(jc))*100._dp,"%"
           ELSE
              WRITE(NERR,*) '              change in mass : ', &
                            zmass_post(jc)-zmass_pre(jc)
           END IF

        END IF

     END DO

     IF (ANY(loabort))THEN
        WRITE(NERR,*) 'Mass conservation failure for compounds:', loabort
        CALL finish('m7_interface','m7 violates the mass conservation')
     END IF

  END IF

  !--- Auxiliary routines:

  CONTAINS

    SUBROUTINE sum_mass(pmasssum)

      IMPLICIT NONE

      REAL(dp) :: pmasssum(5)

      pmasssum=0._dp

      !--- Store stream in local array for optimization:

      zgboxarea(1:kproma)=gboxarea_2d(1:kproma,krow)

      DO jk=1,klev
         DO jl=1,kproma
            pmasssum(1)=pmasssum(1) + (zgso2(jl,jk)         + &
                                       zgso4(jl,jk)         + &
                                       zaerml(jl,jk,iso4ns) + &
                                       zaerml(jl,jk,iso4ks) + &
                                       zaerml(jl,jk,iso4as) + &
                                       zaerml(jl,jk,iso4cs))*zdpg(jl,jk)*zgboxarea(jl)

            pmasssum(2)=pmasssum(2) + (zaerml(jl,jk,ibcks)  + &
                                       zaerml(jl,jk,ibcas)  + &
                                       zaerml(jl,jk,ibccs)  + &
                                       zaerml(jl,jk,ibcki))*zdpg(jl,jk)*zgboxarea(jl)

            pmasssum(3)=pmasssum(3) + (zaerml(jl,jk,iocks)  + &
                                       zaerml(jl,jk,iocas)  + &
                                       zaerml(jl,jk,ioccs)  + &
                                       zaerml(jl,jk,iocki))*zdpg(jl,jk)*zgboxarea(jl)

            pmasssum(4)=pmasssum(4) + (zaerml(jl,jk,issas)  + &
                                       zaerml(jl,jk,isscs))*zdpg(jl,jk)*zgboxarea(jl)

            pmasssum(5)=pmasssum(5) + (zaerml(jl,jk,iduas)  + &
                                       zaerml(jl,jk,iducs)  + &
                                       zaerml(jl,jk,iduai)  + &
                                       zaerml(jl,jk,iduci))*zdpg(jl,jk)*zgboxarea(jl)
         ENDDO
      ENDDO

    END SUBROUTINE sum_mass

END SUBROUTINE m7_interface
