SUBROUTINE xt_chemistry(                                 &
           kproma,     kbdim,      ktdia,      klev,     &
           krow,                                         &
           pmlwc,      pmiwc,      pmratepr,   pmrateps, &
           pfrain,     pfsnow,     pfevapr,    pfsubls,  &
           pmsnowacl,  paclc,      ptm1,       ptte,     &
           pxtm1,      pxtte,      paphp1,     papp1,    &
           prhop1                                        )
  !
  !**** *xt_chemistry*  calculates dry and wet chemistry
  !
  !      J. Feichter     University of Hamburg           30/06/1992
  !      H.-S. Bauer     MPI (conversion to f90)         15/06/2000
  !      P. Stier        MPI-MET (changes, updates, 
  !                               modularisation   )      2001-2003
  !
  !      Purpose
  !      ---------
  !      Computes the homogeneous and heterogeneous oxidation 
  !      of the species for the sulfur cycle chemistry as described
  !      in Feichter et al. (1996). 
  !
  !**    Interface
  !      -----------
  !      *xtchemie*   is called from   *cloud*
  !
  !      Externals
  !      ------------
  !      *xt_wetdep*  calculates the wet deposition
  !
  USE mo_kind,          ONLY: dp
  USE mo_time_control,  ONLY: time_step_len, delta_time
  USE mo_constants,     ONLY: tmelt, g, avo, argas
  USE mo_geoloc,        ONLY: rdayl_x
  USE mo_tracer,        ONLY: ntrac, get_tracer, trlist
  USE mo_aero_massfix,  ONLY: xt_borrow
  USE mo_aero_trac,     ONLY:                                &
             idt_dms,    idt_so2,     idt_so4,               &
             idt_ms4as,  idt_ms4cs,                          &
             idt_nas,    idt_ncs
  USE mo_aero_mem,      ONLY :                               &
             d_prod_so2_dms_oh,                              &
             d_prod_so4_dms_oh,                              &
             d_prod_so2_dms_no3,                             &
             d_prod_so4_so2_oh,                              &
             d_prod_so4_gas, d_prod_so4_liq,                 &
             d_prod_ms4as, d_prod_ms4cs, daylength


  IMPLICIT NONE

  !----------------------------------------------------
  !
  INTEGER :: krow, kproma, kbdim, klev, niter, jk, jl,          &
             jt, ktdia, jn, ilevp1

  INTEGER :: idt_oh,    idt_h2o2,     idt_o3,    idt_no2
  
  REAL(dp):: pmlwc(kbdim,klev),        pmiwc(kbdim,klev),       &
             pmratepr(kbdim,klev),     pmrateps(kbdim,klev),    &
             pfrain(kbdim,klev),       pfsnow(kbdim,klev),      &
             pfevapr(kbdim,klev),      pfsubls(kbdim,klev),     &
             pmsnowacl(kbdim,klev),    paclc(kbdim,klev),       &
             ptm1(kbdim,klev),         ptte(kbdim,klev),        &
             prhop1(kbdim,klev),                                &
             papp1(kbdim,klev),        paphp1(kbdim,klev+1)

  REAL(dp):: pxtm1(kbdim,klev,ntrac),  pxtte(kbdim,klev,ntrac) 

  ! Local arrays

  REAL(dp):: zrdayl(kbdim), &
             zdayfac(kbdim)            ! Weighting factor to account for 
                                       ! the usage of monthly mean OH fields
                                       ! while OH is only present during daytime.
                                       ! (zdayfac = 1 / (relative daylength) )

  REAL(dp):: zlfrac(kbdim,klev),       zrkh2o2(kbdim,klev),     &
             zso4(kbdim,klev),         zdso4(kbdim,klev),       &
             zdso2(kbdim,klev),        zdms2so4(kbdim,klev),    &
             zdms2so2oh(kbdim,klev),   zdms2so2no3(kbdim,klev), &
             zdpg(kbdim,klev),         ztp1(kbdim,klev),        &
             zdum2d(kbdim,klev)

  REAL(dp):: zzoh(kbdim,klev),         zzh2o2(kbdim,klev),      &
   &         zzo3(kbdim,klev),         zzno2(kbdim,klev)               

  REAL(dp):: zxtp1(kbdim,klev,ntrac),  zxtte(kbdim,klev,ntrac),  &
             zxtp1c(kbdim,klev,ntrac), zxtp10(kbdim,klev,ntrac), &
             zdum3d(kbdim,klev,ntrac)

  ! Local scalars

  LOGICAL, PARAMETER :: lstrat=.TRUE.  ! Flag indicating call of
                                       ! xt_wetdep from the 
                                       ! stratiform cloud scheme.


  REAL(dp):: zt, ztk1, ztk2,zmin, zk2i,                             &
          zk2, zk2f, zk3, zmolgs, zmolgh2o2, zmolgair, zmolgw,   &
          ze1k, ze1h, ze2k, ze2h, ze3k, ze3h, zq298, zrgas,      &
          zhpbase, znamair, zlwcmin, zlwcl, zlwcv,               &
          zhp, zqtp1, zrk, zrke, zh_so2, zpfac, zp_so2, zf_so2,  & 
          zh_h2o2, zp_h2o2, zf_h2o2, x, zc, ztc, ze1, ze2,       &
          ze3, zfac1, zrkfac, zza, za21, za22, zph_o3, zf_o3,    &
          zdt, zh2o2m, zso2m, zso4m, zsumo3, zq, zso2mh, zdso2h, & 
          zso2l, zso4l, zzb, zzp, zzq, zzp2, zqhp, zsumh2o2,     &
          za2, zheneff, zrko3, zso2mo, zdso2o, zdso2tot,         &
          ztmst, zqtmst,                                         &
          zm, zhil, zexp, ztk23b, zdms,                          &
          zo2, ztk12, ztoso2, ztk3,                              &
          zqt, zqt3, zrhoair, zkno2o3, zkn2o5aq, zrx1, zrx2,     &
          zkno2no3, zeqn2o5, zkn2o5, zno3, zfraction, zeps

  !--- Constants:

  REAL(dp), PARAMETER :: zso4massc=3.25E-15_dp ! Mass of coarse mode so4 particle
                                        ! as assumed for the in-cloud 
                                        ! formation of sulfate aerosol in the 
                                        ! absence of other particles. [kg]
                                        ! 3.25E-15 kg is equivalent to a 
                                        ! mono-disperse radius of 0.75 um. 

  !--- 0) Initialisations: -----------------------------------------

  zeps=EPSILON(1._dp)

  ilevp1=klev+1

  !--- OH weighting factor:

  zrdayl(1:kproma) = rdayl_x(1:kproma,krow)

  WHERE(daylength(1:kproma,krow) > zeps)
     zdayfac(1:kproma)= 1._dp/daylength(1:kproma,krow)
  ELSEWHERE
     zdayfac(1:kproma)= 0._dp
  END WHERE

  !--- Mass conserving correction of negative tracer values:

  CALL xt_borrow(kproma, kbdim,  klev,  ilevp1, ntrac, krow, &
                 papp1,  paphp1,                             &
                 pxtm1,  pxtte                               )

  !--- Get tracers:

  CALL get_tracer('OH',idx=idt_oh)
  CALL get_tracer('H2O2',idx=idt_h2o2)  
  CALL get_tracer('O3',idx=idt_o3)
  CALL get_tracer('NO2',idx=idt_no2)

  !--- Constants:

  ztmst=time_step_len
  zqtmst=1._dp/ztmst
  niter=5

  zxtte(:,:,:)    = 0._dp             ! Preset initial tracer tendency 
  zlfrac(:,:)     = 0._dp
  zmin            = 1.E-20_dp         ! Liquid water content threshold
  zdso4(:,:)      = 0._dp             ! Sulfate produced by heterogeneous
                                   ! oxidation of SO2
  zdso2(:,:)      = 0._dp             ! SO2 homogeneously oxidised to SO4
  zdms2so4(:,:)   = 0._dp             ! DMS (assumed) oxidised to SO4
  zdms2so2oh(:,:)   = 0._dp           ! DMS (assumed) oxidised to SO2 via OH
  zdms2so2no3(:,:)   = 0._dp          ! DMS (assumed) oxidised to SO2 via No3

  !--- Reaction rate SO2-OH:

  zk2i=2.0e-12_dp
  zk2=4.0e-31_dp
  zk2f=0.45_dp

  !--- Reaction rate DMS-NO3:

  zk3=1.9e-13_dp

  !--- Molecular weights in g:

  zmolgs=32.064_dp
  zmolgh2o2=34.01474_dp
  zmolgair=28.84_dp
  zmolgw=18.015_dp
  zhpbase=2.5e-06_dp
  ze1k=1.1e-02_dp
  ze1h=2300._dp
  ze2k=1.23_dp
  ze2h=3020._dp
  ze3k=1.2e-02_dp
  ze3h=2010._dp
  zq298=1._dp/298._dp
  zrgas=8.2e-02_dp
  znamair=1.e-03_dp*avo/zmolgair
  zlwcmin=1.e-07_dp
  !
  !--- Calculate temperature at t=t+1:

  ztp1(1:kproma,:)=ptm1(1:kproma,:)+ptte(1:kproma,:)

  !--- Calculate tracer concentrations at t=t+1:

  zxtp1(1:kproma,:,:)=pxtm1(1:kproma,:,:) + pxtte(1:kproma,:,:)*ztmst
  
  !--- Distribute equally for cloudy and cloud free part:

  zxtp1c(1:kproma,:,:)=zxtp1(1:kproma,:,:)
  zxtp10(1:kproma,:,:)=zxtp1(1:kproma,:,:)

  !--- Convert offline oxidant concentrations from
  !    volume mixing ratio into [molecule cm-3]:

  DO 112 jk=1,klev
     DO 111 jl=1,kproma

        zc=(papp1(jl,jk)/(argas*ptm1(jl,jk)))*1.E-6_dp*avo
        !  (          mol m-3              )
        !  (          mol cm-3                   )
        !  (          molecules cm-3                 )

        zzoh(jl,jk)   = zxtp1(jl,jk,idt_oh)   * zc
        zzh2o2(jl,jk) = zxtp1(jl,jk,idt_h2o2) * zc
        zzo3(jl,jk)   = zxtp1(jl,jk,idt_o3)   * zc
        zzno2(jl,jk)  = zxtp1(jl,jk,idt_no2)  * zc

111  END DO
112 END DO
  !
  !
  DO 212 jk=ktdia,klev
     DO 210 jl=1,kproma

        zdpg(jl,jk)=(paphp1(jl,jk+1)-paphp1(jl,jk))/g

        IF(paclc(jl,jk).LT.1.e-04_dp.OR.pmlwc(jl,jk).LT.zmin) THEN
           pmratepr(jl,jk)=0._dp
           pmlwc(jl,jk)=0._dp
        END IF
        IF(paclc(jl,jk).LT.1.e-04_dp.OR.pmiwc(jl,jk).LT.zmin) THEN
           pmrateps(jl,jk)=0._dp
           pmiwc(jl,jk)=0._dp
        END IF
        IF (paclc(jl,jk).LT.1.e-04_dp) THEN
           paclc(jl,jk)=0._dp
        END IF
210  END DO
212 END DO

  !--- Calculate total sulfate in the accumulation and coarse mode:
  
  zso4(1:kproma,:)=zxtp1(1:kproma,:,idt_ms4as)+ zxtp1(1:kproma,:,idt_ms4cs)
  zso4(1:kproma,:)=MAX(zso4(1:kproma,:),0._dp)

  !
  !--- 1) Calculate the reaction-rates for SO2-H2O2:
  !
  DO 308 jk=ktdia,klev
     DO 306 jl=1,kproma
        IF(pmlwc(jl,jk).GT.zmin) THEN
           zlwcl=pmlwc(jl,jk)*prhop1(jl,jk)*1.e-06_dp
           zlwcv=pmlwc(jl,jk)*prhop1(jl,jk)*1.e-03_dp
           zhp=zhpbase+zso4(jl,jk)*1000._dp/(pmlwc(jl,jk)*zmolgs)
           zqtp1=1._dp/ptm1(jl,jk)-zq298
           zrk=zfarr(8.e+04_dp,-3650._dp,zqtp1)/(0.1_dp+zhp)
           zrke=zrk/(zlwcl*avo)
           !
           zh_so2=zfarr(ze2k,ze2h,zqtp1)
           zpfac=zrgas*zlwcv*ptm1(jl,jk)
           zp_so2=zh_so2*zpfac
           zf_so2=zp_so2/(1._dp+zp_so2)
           !
           zh_h2o2=zfarr(9.7e+04_dp,6600._dp,zqtp1)
           zp_h2o2=zh_h2o2*zpfac
           zf_h2o2=zp_h2o2/(1._dp+zp_h2o2)
           !
           zrkh2o2(jl,jk)=zrke*zf_so2*zf_h2o2
        ELSE
           zrkh2o2(jl,jk)=0._dp
        END IF
306  END DO
308 END DO
  !
  !--- 2) Heterogenous chemistry:
  !

  !OCL NOVREC,NOALIAS

  DO 322 jk=ktdia,klev
     DO 320 jl=1,kproma

        IF(zxtp1(jl,jk,idt_so2).GT.zmin.AND.pmlwc(jl,jk).GT.zmin) THEN
           x=prhop1(jl,jk)
           ztc=ptm1(jl,jk)-tmelt
           !
           zqtp1=1._dp/ptm1(jl,jk)-zq298
           ze1=zfarr(ze1k,ze1h,zqtp1)
           ze2=zfarr(ze2k,ze2h,zqtp1)
           ze3=zfarr(ze3k,ze3h,zqtp1)

           !--- Calculate the liquid water content in [l/cm**3]:

           zlwcl=pmlwc(jl,jk)*prhop1(jl,jk)*1.e-06_dp

           !--- Calculate conversion factor for [molec/cm**3(air)] to [mole/l(aq)]:

           zfac1=1._dp/(zlwcl*avo)
           
           !--- Calculate the liquid water volume fraction [m**3/m**3]:

           zlwcv=pmlwc(jl,jk)*prhop1(jl,jk)*1.e-03_dp

           !--- Calculate conversion factor R_gas * T * V(aq)/V(air):

           zrkfac=zrgas*ptm1(jl,jk)*zlwcv

           !--- Multiplication with the Henry coefficient for SO2:

           zza=ze2*zrkfac

           za21=4.39e+11_dp*EXP(-4131._dp/ptm1(jl,jk))
           za22=2.56e+03_dp*EXP(-926._dp/ptm1(jl,jk))
           zph_o3=ze1*zrkfac
           zf_o3=zph_o3/(1._dp+zph_o3)
           zdt=ztmst/REAL(niter,dp)
           !
           zh2o2m=zzh2o2(jl,jk)
           zso2m=zxtp1(jl,jk,idt_so2)*xtoc(x,zmolgs)
           zso4m=zso4(jl,jk)*xtoc(x,zmolgs)
           !
           zsumh2o2=0._dp
           zsumo3=0._dp
           !
!CDIR UNROLL=5
           DO 309 jn=1,niter
              zq=zrkh2o2(jl,jk)*zh2o2m
              zso2mh=zso2m*EXP(-zq*zdt)
              !
              zdso2h=zso2m-zso2mh
              zh2o2m=zh2o2m-zdso2h
              zh2o2m=MAX(0._dp,zh2o2m)
              zsumh2o2=zsumh2o2+zdso2h
              !
              zso4m=zso4m+zdso2h
              !   calculate the ph value
              zso2l=zso2mh*zfac1
              zso4l=zso4m*zfac1
              zzb=zhpbase+zso4l
              zzp=(zza*ze3-zzb-zza*zzb)/(1._dp+zza)
              zzq=-zza*ze3*(zzb+zso2l)/(1._dp+zza)
              zzp=0.5_dp*zzp
              zzp2=zzp*zzp
              zhp=-zzp+SQRT(zzp2-zzq)
              zqhp=1._dp/zhp
              !
              !--- Calculate the reaction rate for SO2-O3:
              !
              za2=(za21+za22*zqhp)*zfac1

              !--- Calculate effective Henry coefficient neglecting SO3-- :

              zheneff=1._dp+ze3*zqhp
              zp_so2=zza*zheneff
              
              !--- Calculate the liquid fraction of total SO2, i.e. SO2(aq)/SO2(tot):

              zf_so2=zp_so2/(1._dp+zp_so2)

              zrko3=za2*zf_o3*zf_so2
              !
              zq=zzo3(jl,jk)*zrko3
              zso2mo=zso2mh*EXP(-zq*zdt)
              zdso2o=zso2mh-zso2mo
              zso4m=zso4m+zdso2o
              zso2m=zso2mo
              zsumo3=zsumo3+zdso2o
309        END DO
           !
           zdso2tot=zxtp1(jl,jk,idt_so2)-zso2m*ctox(x,zmolgs)
           zdso2tot=MIN(zdso2tot,zxtp1(jl,jk,idt_so2))

           zxtp1c(jl,jk,idt_so2)=zxtp1(jl,jk,idt_so2)-zdso2tot
           
           !--- zdso2tot is the total amount of SO2 oxidised by 
           !    heterogeneous chemistry to sulfate, i.e. d(SO4):

           zdso4(jl,jk)=zdso2tot

           !--- Liquid fraction of the total SO2:

           zlfrac(jl,jk)=zf_so2

        END IF
320  END DO
322 END DO


  !--- ) Sulfate produced within cloud droplets is distributed 
  !      to accumulation and coarse mode sulfuric acid mass:

  DO 332 jk=ktdia,klev
     DO 330 jl=1,kproma

        !--- Accumulation and Coarse Mode particles present:

        IF ((zxtp1(jl,jk,idt_nas) >= 1.E-3_dp) .AND. (zxtp1(jl,jk,idt_ncs) >= 1.E-3_dp)) THEN

           zfraction=zxtp1(jl,jk,idt_nas)/(zxtp1(jl,jk,idt_nas)+zxtp1(jl,jk,idt_ncs))

           zxtp1c(jl,jk,idt_ms4as)=zxtp1(jl,jk,idt_ms4as) + zdso4(jl,jk)*zfraction

           zxtp1c(jl,jk,idt_ms4cs)=zxtp1(jl,jk,idt_ms4cs) + zdso4(jl,jk)*(1-zfraction)

        !--- Accumulation but no Coarse Mode particles present:

        ELSE IF ((zxtp1(jl,jk,idt_nas) >= 1.E-3_dp) .AND. (zxtp1(jl,jk,idt_ncs) < 1.E-3_dp)) THEN

           zxtp1c(jl,jk,idt_ms4as)=zxtp1(jl,jk,idt_ms4as) + zdso4(jl,jk)

        !--- No Accumulation but Coarse Mode particles present:

        ELSE IF ((zxtp1(jl,jk,idt_nas) < 1.E-3_dp) .AND. (zxtp1(jl,jk,idt_ncs) >= 1.E-3_dp)) THEN

           zxtp1c(jl,jk,idt_ms4cs)=zxtp1(jl,jk,idt_ms4cs) + zdso4(jl,jk)

           !--- Neither Accumulation Mode nor Coarse Mode particles present.
           !
           !    Assumption: Cloud water sulfate is assumed to exits as 
           !                coarse mode particles as this resembles most
           !                the observed "droplet mode" 
           !                (See Seinfeld & Pandis, p. 821, 1998)

        ELSE IF ((zxtp1(jl,jk,idt_nas) < 1.E-3_dp) .AND. (zxtp1(jl,jk,idt_ncs) < 1.E-3_dp)) THEN

           zxtp1c(jl,jk,idt_ms4cs)=zxtp1(jl,jk,idt_ms4cs) + zdso4(jl,jk)

           !--- Increase the coarse mode soluble particle number tendency:

           zxtp1c(jl,jk,idt_ncs)=zxtp1c(jl,jk,idt_ncs)+(zdso4(jl,jk)/zso4massc)

        END IF

330  END DO
332 END DO


  !--- Calculate tracer tendencies due to heterogeneous chemistry:

  tracer : DO jt=1, ntrac
     DO jk=ktdia, klev
        DO jl=1, kproma
           zxtte(jl,jk,jt)=((zxtp10(jl,jk,jt)*(1._dp-paclc(jl,jk))+ &
                             zxtp1c(jl,jk,jt)*paclc(jl,jk))-pxtm1(jl,jk,jt))*zqtmst - &
                           pxtte(jl,jk,jt)
           !
           !--- Security check:
           !
           !    Total tendency is not allowed to exeed 
           !    threshold that produces negative values.
           !
           !    pxtp1 = pxtm1 + (pxtte+zxtte)*dt
           !
           !<=> 0     < pxtm1 + (pxtte+zxtte)*dt
           !
           !               pxtm1  
           ! => zxtte > - ------- - pxtte
           !                dt

           zxtte(jl,jk,jt)=MAX(-pxtm1(jl,jk,jt)*zqtmst-pxtte(jl,jk,jt) , zxtte(jl,jk,jt)) 

           !--- Change of total tendencies:

           pxtte(jl,jk,jt)=pxtte(jl,jk,jt)+zxtte(jl,jk,jt)  

        END DO
     END DO
  END DO tracer

  !--- Store sulfate production for diagnostics:
  
  DO jk=1, klev
     
     d_prod_ms4as(1:kproma,krow)=d_prod_ms4as(1:kproma,krow) + &
                                 zxtte(1:kproma,jk,idt_ms4as)*zdpg(1:kproma,jk)*delta_time

     d_prod_ms4cs(1:kproma,krow)=d_prod_ms4cs(1:kproma,krow) + &
                                 zxtte(1:kproma,jk,idt_ms4cs)*zdpg(1:kproma,jk)*delta_time     

     d_prod_so4_liq(1:kproma,jk,krow)=d_prod_so4_liq(1:kproma,jk,krow) + &
                                      (zxtte(1:kproma,jk,idt_ms4as)+zxtte(1:kproma,jk,idt_ms4cs)) * &
                                      zdpg(1:kproma,jk)*delta_time

  END DO


  !--- Preliminary call to wet deposition (should be moved out of chemistry):

  IF (trlist%anywetdep/=0) THEN

    CALL xt_wetdep(kbdim,  kproma,    klev,     ktdia,    & ! dimensions
                   krow,                                  & ! longitude (kproma-block) index
                   zxtp10, zxtp1c,                        & ! tracers
                   zdum2d, zdum3d,    zdum2d,             & ! convective fluxes and corresp. mmr
                   pxtm1,                                 & ! stratiform properties (not needed for convective)
                   pmlwc,  pmiwc,     pmratepr, pmrateps, & ! cloud properties
                   pfrain, pfsnow,    pfevapr,  pfsubls,  & !   "       "
                   paclc,  pmsnowacl, lstrat,             & !   "       "
                   ptm1,   zdpg,      zlfrac,             & ! thermodynamic quantities
                   pxtte                                  ) ! fraction of SO2 in the liquid phase

  END IF


  !--- 4) Gas phase chemistry:
  !       Oxidation with OH
  !
  !--- 4.1) Day-time chemistry:
  !
  !OCL NOVREC,NOALIAS
     
  DO 412 jk=1,klev
     DO 410 jl=1,kproma
        IF(NINT(zrdayl(jl)).EQ.1) THEN

           x=prhop1(jl,jk)
           ztk2=zk2*(ptm1(jl,jk)/300._dp)**(-3.3_dp)
           zm=x*znamair
           zhil=ztk2*zm/zk2i
           zexp=LOG10(zhil)
           zexp=1._dp/(1._dp+zexp*zexp)
           ztk23b=ztk2*zm/(1._dp+zhil)*zk2f**zexp
           zdso2(jl,jk)=zxtp1(jl,jk,idt_so2)*zzoh(jl,jk)*ztk23b*zdayfac(jl)
           zdso2(jl,jk)=MIN(zdso2(jl,jk),zxtp1(jl,jk,idt_so2)*zqtmst)
           zdso2(jl,jk)=MAX(zdso2(jl,jk),0._dp)
           pxtte(jl,jk,idt_so2)=pxtte(jl,jk,idt_so2)-zdso2(jl,jk)
           pxtte(jl,jk,idt_so4)=pxtte(jl,jk,idt_so4)+zdso2(jl,jk)

           IF(zxtp1(jl,jk,idt_dms).LE.zmin) THEN
              zdms=0._dp
           ELSE
              zt=ptm1(jl,jk)
              !   zo2 = 21% of density-air * avogadro / molg-air
              zo2=xtoc(x,zmolgair)*0.21_dp
              !   H abstraction
              ztk1=9.6e-12_dp*EXP(-234._dp/zt)
              !   OH addition
              ztk2=1.7e-42_dp*EXP(7810._dp/zt)*zo2/(1._dp+5.5e-31_dp*EXP(7460._dp/zt)*zo2)
              ztk12=ztk1+ztk2
              zdms=zxtp1(jl,jk,idt_dms)*xtoc(x,zmolgs)*zzoh(jl,jk)*zdayfac(jl)*ztk12
              zdms=zdms*ctox(x,zmolgs)
              zdms=MIN(zdms,zxtp1(jl,jk,idt_dms)*zqtmst)
              pxtte(jl,jk,idt_dms)=pxtte(jl,jk,idt_dms)-zdms
              !
              !--- ztoso2 is the fraction of DMS oxidized to SO2:
              !
              ztoso2=(ztk1+0.75_dp*ztk2)/(ztk1+ztk2)

              zdms2so2oh(jl,jk)=zdms*ztoso2
              
              pxtte(jl,jk,idt_so2)=pxtte(jl,jk,idt_so2)+zdms2so2oh(jl,jk)
             

              !--- (1-ztoso2) is converted to MSA and assumed to occur as sulfate:

              zdms2so4(jl,jk)=zdms*(1._dp-ztoso2)

              pxtte(jl,jk,idt_so4)=pxtte(jl,jk,idt_so4)+zdms2so4(jl,jk)

           END IF

        END IF
410  END DO
412 END DO

  
  !--- 4.2) Night-time chemistry:
  !
  DO 416 jk=1,klev
     DO 414 jl=1,kproma
        IF(NINT(zrdayl(jl)).NE.1) THEN
           x=prhop1(jl,jk)
           IF(zxtp1(jl,jk,idt_dms).LE.zmin) THEN
              zdms=0._dp
           ELSE
              ztk3=zk3*EXP(520._dp/ptm1(jl,jk))
              !    calculate the steady state concentration of NO3
              zqt=1._dp/ptm1(jl,jk)
              zqt3=300._dp*zqt
              zrhoair=prhop1(jl,jk)*znamair
              zkno2o3=1.2e-13_dp*EXP(-2450._dp*zqt)
              zkn2o5aq=0.1e-04_dp
              zrx1=2.2e-30_dp*zqt3**3.9_dp*zrhoair
              zrx2=1.5e-12_dp*zqt3**0.7_dp
              zkno2no3=zrx1/(1._dp+zrx1/zrx2)*0.6_dp**(1._dp/(1._dp+(LOG10(zrx1/zrx2))**2))
              zeqn2o5=4.e-27_dp*EXP(10930._dp*zqt)
              zkn2o5=zkno2no3/zeqn2o5
              !
              zno3=zkno2o3*(zkn2o5+zkn2o5aq)*zzno2(jl,jk)*zzo3(jl,jk)
              zzq=zkno2no3*zkn2o5aq*zzno2(jl,jk)+(zkn2o5+zkn2o5aq)*ztk3 &
                   &   *zxtp1(jl,jk,idt_dms)*xtoc(x,zmolgs)
              IF(zzq.GT.0._dp) THEN
                 zno3=zno3/zzq
              ELSE
                 zno3=0._dp
              END IF
              zdms=zxtp1(jl,jk,idt_dms)*zno3*ztk3
              zdms=MIN(zdms,zxtp1(jl,jk,idt_dms)*zqtmst)
              pxtte(jl,jk,idt_dms)=pxtte(jl,jk,idt_dms)-zdms
              pxtte(jl,jk,idt_so2)=pxtte(jl,jk,idt_so2)+zdms
              zdms2so2no3(jl,jk)=zdms
           ENDIF
        ENDIF
414  END DO
416 END DO

   !--- Store producion of sulfate and SO2 for diagnostics:

  DO jk=1, klev

     d_prod_so4_dms_oh(1:kproma,krow)=d_prod_so4_dms_oh(1:kproma,krow)+&
                                      zdms2so4(1:kproma,jk)*zdpg(1:kproma,jk)*delta_time

     d_prod_so2_dms_oh(1:kproma,krow)=d_prod_so2_dms_oh(1:kproma,krow)+&
                                      zdms2so2oh(1:kproma,jk)*zdpg(1:kproma,jk)*delta_time

     d_prod_so2_dms_no3(1:kproma,krow)=d_prod_so2_dms_no3(1:kproma,krow)+&
                                       zdms2so2no3(1:kproma,jk)*zdpg(1:kproma,jk)*delta_time

     d_prod_so4_so2_oh(1:kproma,krow)=d_prod_so4_so2_oh(1:kproma,krow)+&
                                      zdso2(1:kproma,jk)*zdpg(1:kproma,jk)*delta_time

     d_prod_so4_gas(1:kproma,jk,krow)=d_prod_so4_gas(1:kproma,jk,krow) + &
                                      (zdso2(1:kproma,jk)+zdms2so4(1:kproma,jk))*zdpg(1:kproma,jk)*delta_time

  END DO
  
CONTAINS

  !--- 5) Auxiliary functions:

  FUNCTION xtoc(x,y)

    !
    !     Function for changing the units
    !     from mass-mixing ratio to molecules per cm**3 and vice versa
    !
    !   x = density of air, y = mol weight in gramm

    IMPLICIT NONE

    ! Function return value
    REAL(dp) :: xtoc

    ! Scalar arguments
    REAL(dp) :: x, y

    xtoc=x*6.022e+20_dp/y

  END FUNCTION xtoc


  FUNCTION ctox(x,y)

    !
    !     Function for changing the units
    !     from molecules per cm**3 to mass-mixing ratio.
    !
    !   x = density of air, y = mol weight in gramm

    IMPLICIT NONE

    ! Function return value
    REAL(dp) :: ctox

    ! Scalar arguments
    REAL(dp) :: x, y

    ctox=y/(6.022e+20_dp*x)

  END FUNCTION ctox


  FUNCTION zfarr(zk,zh,ztpq)

    IMPLICIT NONE

    ! Function return value
    REAL(dp) :: zfarr

    ! Scalar arguments
    REAL(dp) :: zk, zh, ztpq

    zfarr=zk*EXP(zh*ztpq) 

  END FUNCTION zfarr
     
END SUBROUTINE xt_chemistry
