SUBROUTINE xt_drydep(kproma, kbdim, klev,     krow,   pcfm,     pcfnc,  &
                     pepdu2, pkap,  pum1,     pvm1,   pgeom1,   pri,    &
                     ptvir1, ptvl,  prh,      paz0,   ptslm1,   loland, &
                     pfrl,   pfrw,  pfri,     pcvs,   pcvw,     pvgrat, &
                     psrfl,  prco_leaf,       pfws,   pu10,     pv10,   &
                     pxtems, pxtm1, pdensair, paphp1, pforest,  ptsi,   &
                     pws,    pwsmx, paz0l,    paz0w,  paz0i,    pcdnl   )
  !
  ! Purpose:
  ! ---------
  ! This subroutine calculates the dry deposition 
  ! of aerosols and gases for calculated or prescribed 
  ! dry deposition velocities. 
  !
  ! Authors:
  ! ----------
  ! Philip Stier,      MPI-MET
  !
  ! Interface:
  ! ----------
  ! *xt_drydep* is called from *vdiff*
  !
  ! Method:
  ! -------
  ! The routine reduces the emission flux of the respective tracers
  ! according to the dry deposition velocities that are used
  ! from the stream vdrydep. It also calles the routines for the actual
  ! calculation of the dry deposition velocities.
  !
  ! Arguments:
  ! ----------
  !
  !  paz0     : roughness length
  !  paz0l    :      "            over land
  !  paz0w    :      "            over water
  !  paz0i    :      "            over ice
  !  ...

  USE mo_kind,             ONLY: dp
  USE mo_exception,        ONLY: finish
  USE mo_time_control,     ONLY: delta_time,    time_step_len
  USE mo_constants,        ONLY: g
  USE mo_tracer,           ONLY: ntrac,         trlist,      GAS 
  USE mo_xtsurf_drydep,    ONLY: xtsurf_calcra, xtsurf_vdbl, xtsurf_vdaer
  USE mo_aero_mem,         ONLY: vdrydep, ddep

!uh+

  USE mo_be10,             ONLY: area_avg, idt_be10, idt_be7, idt_cl36, idt_na22, &
                                 BE_FLAG,   &
                                 be10_ddep_acc, be7_ddep_acc, cl36_ddep_acc, &
                                 na22_ddep_acc, pb210_ddep_acc, idt_pb210
  USE mo_control,          ONLY: lcn, lbe10, lbe7, lcl36, lna22, lpb210
!uh-


  IMPLICIT NONE

  !--- Parameters:
  
  INTEGER :: kproma,           kbdim,            klev,             krow

  REAL(dp):: pepdu2,           pkap

  REAL(dp):: pcdnl(kbdim),     pcfnc(kbdim),     pri(kbdim),       ptvl(kbdim),        &
             prh(kbdim),       paz0(kbdim),      ptslm1(kbdim),    pfrl(kbdim),        &
             pfrw(kbdim),      pfri(kbdim),      pcvs(kbdim),      pcvw(kbdim),        &
             pvgrat(kbdim),    psrfl(kbdim),     prco_leaf(kbdim), pfws(kbdim),        &
             pu10(kbdim),      pv10(kbdim),      pdensair(kbdim),  pforest(kbdim),     &
             ptsi(kbdim),      pws(kbdim),       pwsmx(kbdim),                         &
             paz0l(kbdim),     paz0w(kbdim),     paz0i(kbdim)

  REAL(dp):: pum1(kbdim,klev), pvm1(kbdim,klev), pcfm(kbdim,klev), pgeom1(kbdim,klev), &
             ptvir1(kbdim,klev)

  REAL(dp):: paphp1(kbdim,klev+1) 

  REAL(dp):: pxtems(kbdim,ntrac)

  REAL(dp):: pxtm1(kbdim,klev,ntrac)
            

  LOGICAL :: loland(kbdim)

  !--- Local scalars:

  INTEGER :: jl, jt        

!uh+
  INTEGER :: jk, jj
!uh-


  REAL(dp):: zdz

  !--- Local arrays:

  REAL(dp):: zrahwat(kbdim),   zrahice(kbdim),   zrahveg(kbdim),   zrahslsn(kbdim),   &
             zustar(kbdim),    zustveg(kbdim),   zustslsn(kbdim),                     &
             zevap(kbdim),     zvgrat(kbdim),    zcvbs(kbdim)

  REAL(dp):: zdrydepflux(kbdim,ntrac)


!uh+
  REAL(dp)    :: zdummy(kbdim,klev,ntrac), vt(kbdim)


  !--- Diagnostic stream:

  REAL(dp),        POINTER :: ddep_p(:,:)
  REAL(dp),        POINTER :: vdrydep_p(:,:)

  !--- 0) Initialisations: -------------------------------------------------------------

!uh+
  DO jt=1,ntrac
     DO jk=1,klev
        DO jl=1,kproma
           zdummy(jl,jk,jt)=0.
        END DO
     END DO
  END DO
!uh-


  
  !--- Calculate bare soil fraction:
  !    It is calculated as residual term over land.

  DO jl=1,kproma

     IF (loland(jl)) THEN
        zvgrat(jl)=pvgrat(jl)                                           ! vegetation fraction
        zcvbs(jl)=(1._dp-pcvs(jl))*(1._dp-pvgrat(jl))*(1._dp-pcvw(jl))  ! bare soil fraction
     ELSE
        zvgrat(jl)=0._dp
        zcvbs(jl)=0._dp
     ENDIF

  ENDDO


  !--- 1) Calculate dry deposition velocities ------------------------------------------

! uh+
!  IF ( ANY(trlist%ti(:)%ndrydep>2) ) THEN
  IF ( ANY(trlist%ti(:)%ndrydep>4) ) THEN
!uh-

     CALL finish('xt_drydep', 'Other dry deposition schemes not yet implemented')

  END IF

  !--- 1.1) Scheme for prescibed dry deposition velocities:

  IF( ANY(trlist%ti(:)%ndrydep==1) ) THEN
     
     CALL xt_vdrydep_presc(kproma, kbdim,  klev,    loland, krow,     &
                           paphp1, pcvs,   pforest, pfri,   ptsi,     &
                           pcvw,   ptslm1, pws,     pwsmx,  pdensair  )

  END IF

  !--- 1.2) Scheme for explicitly calculated dry deposition velocities:
  !         (Overwrites the prescribed velocities if both are requested)

!uh:
!  IF ( ANY(trlist%ti(:)%ndrydep==2) ) THEN
  IF ( ANY(trlist%ti(:)%ndrydep>=2) ) THEN


     !--- Calculate the aerodynamic resistance:

     CALL xtsurf_calcra (kproma,   kbdim,    klev,    krow,                     &
                         pepdu2,   pkap,     pum1,    pvm1,     pgeom1,  pri,   &
                         ptvir1,   ptvl,     ptslm1,  loland,                   &
                         pcdnl,    pcfm,     pcfnc,                             &
                                   paz0w,    paz0i,   paz0l,                    &
                                   zrahwat,  zrahice, zrahveg,  zrahslsn,       &
                         zustar,                      zustveg,  zustslsn        )
     
     !--- Calculate the dry deposition velocity for gases:

     CALL xtsurf_vdbl   (kproma,   kbdim,    klev,    krow,     loland,        &
                         psrfl,    prco_leaf,pfws,    prh,      pum1,          &
                         pvm1,     pfri,     zcvbs,   pfrw,     pcvs,          &
                         pcvw,     zvgrat,   zrahwat, zrahice,  zrahveg,       &
                         zrahslsn, ptslm1,   zustar,  zustveg,  zustslsn )

     !--- Calculate the dry deposition velocity for aerosols:

     CALL xtsurf_vdaer  (kproma,   kbdim,    klev,    krow,     loland,  zvgrat, &
                         pcvs,     pcvw,     pfri,    zcvbs,    pfrw,    pum1,   &  
                         pvm1,     zustar,   zustveg, zustslsn, pu10,    pv10,   &
                         paz0w,    zrahwat,  zrahveg, zrahslsn, ptslm1,  zevap,  &
                         prh,      pxtm1                                         )


! uh+
     IF (lcn) THEN
        DO jt=1,ntrac
           IF (trlist%ti(jt)%ndrydep==BE_FLAG) THEN
              CALL area_avg(kproma,klev,kbdim,krow,2,pxtm1,zdummy,jt)
! copy the calculated drydep. velocity for other CN tracers
              DO jj=jt+1,ntrac
                 IF (trlist%ti(jj)%ndrydep==BE_FLAG) THEN
                    vdrydep_p => vdrydep(jt)%ptr
                    vt(1:kproma)=vdrydep_p(1:kproma,krow)
                    vdrydep_p => vdrydep(jj)%ptr
                    vdrydep_p(1:kproma,krow)=vt(1:kproma)
                 END IF
              END DO

              EXIT
           END IF
        END DO

     END IF !lcn
!uh-
     
  END IF



  !--- 2) Change emissions tendencies due to dry deposition: --------------

  DO jt=1, ntrac
     
     IF (trlist%ti(jt)%ndrydep > 0) THEN

        vdrydep_p => vdrydep(jt)%ptr

        DO jl=1, kproma

           ! Security check, limit the deposition velocity to 2x the 
           ! vertical grid velocity of the lowest model layer.

           zdz=(paphp1(jl,klev+1)-paphp1(jl,klev))/(pdensair(jl)*g)
           vdrydep_p(jl,krow)=MIN(vdrydep_p(jl,krow) , 2._dp*(zdz/time_step_len))

           !--- Calculate the tracer flux to the surface that is 
           !    equivalent to the deposition velocity:

           zdrydepflux(jl,jt)=pxtm1(jl,klev,jt)*pdensair(jl)*vdrydep_p(jl,krow)

           !--- Reduce emission flux:

           pxtems(jl,jt)=pxtems(jl,jt)-zdrydepflux(jl,jt)

        END DO

     ELSE ! trlist%ti(jt)%ndrydep <= 0

        zdrydepflux(1:kproma,jt)=0._dp

     END IF
        
  END DO

     
  !--- 4) Accumulate diagnostics:

  DO jt=1, ntrac

     IF (trlist%ti(jt)%ndrydep > 0) THEN

        ddep_p => ddep(jt)%ptr

        ddep_p(1:kproma,krow)=ddep_p(1:kproma,krow)+zdrydepflux(1:kproma,jt)*delta_time

! uh+: for mass balance and fluxes
        IF (lcn) THEN

           IF(trlist%ti(jt)%ndrydep==BE_FLAG) THEN
              DO jl=1,kproma
                 IF(lbe10.AND.jt==idt_be10) THEN
                    be10_ddep_acc(jl,krow)=be10_ddep_acc(jl,krow)+zdrydepflux(jl,jt)*delta_time
                 END IF
                 IF(lbe7.AND.jt==idt_be7) THEN
                    be7_ddep_acc(jl,krow)=be7_ddep_acc(jl,krow)+zdrydepflux(jl,jt)*delta_time
                 END IF
                 IF(lcl36.AND.jt==idt_cl36) THEN
                    cl36_ddep_acc(jl,krow)=cl36_ddep_acc(jl,krow)+zdrydepflux(jl,jt)*delta_time
                 END IF
                 IF(lna22.AND.jt==idt_na22) THEN
                    na22_ddep_acc(jl,krow)=na22_ddep_acc(jl,krow)+zdrydepflux(jl,jt)*delta_time
                 END IF
                 IF(lpb210.AND.jt==idt_pb210) THEN
                    pb210_ddep_acc(jl,krow)=pb210_ddep_acc(jl,krow)+zdrydepflux(jl,jt)*delta_time
                 END IF
              END DO
           END IF
        END IF!lcn
!uh-


     END IF

  END DO !jt=1, ntrac

END SUBROUTINE xt_drydep
