SUBROUTINE xt_vdrydep_presc(kproma, kbdim,  klev,    krow,  loland,   &
                            paphp1, pcvs,   pforest, pfri,  ptsi,     &
                            pcvw,   ptslm1, pws,     pwsmx, pdensair  )

  ! Purpose:
  ! ---------
  ! This routine prescribes dry deposition velocities for 
  ! aerosols and gases in dependency of surface type, etc.
  ! ...
  !
  ! Authors:
  ! ----------
  ! Hans Feichter, MPI-MET
  ! Philip Stier,  MPI-MET
  !
  ! Method:
  ! -------
  ! Currently rudimentary!
  !
  ! Interface:
  ! ----------
  ! *xt_vdrydep_presc* is called from *xt_drydep*

  USE mo_kind,         ONLY: dp
  USE mo_control,      ONLY: nlev
  USE mo_time_control, ONLY: time_step_len
  USE mo_constants,    ONLY: tmelt,       g
  USE mo_tracer,       ONLY: ntrac,       trlist, &
                             AEROSOLMASS, AEROSOLNUMBER, GAS
  USE mo_physc2,       ONLY: ctfreez, & ! sea water freezing temp.
                             csncri     ! m water equivalent critical snow depth
  USE mo_aero_mem,     ONLY: smelt
  USE mo_aero_mem,     ONLY: vdrydep


  IMPLICIT NONE

  !--- Parameters:
  !--- Local variables:
  !
  ! zvdrd   dry deposition velocity in m/s
  ! zvdrd(jl,1)  for so2 gas
  ! zvdrd(jl,2)  for aerosols
  ! zvdcoarse    for dust and seasalt aerosols (coarse mode)
  
  ! Parameters:
  
  INTEGER :: kproma,         kbdim,          klev,          krow

  REAL(dp):: pfri(kbdim),    pcvs(kbdim),    pcvw(kbdim),               &
             pforest(kbdim), ptsi(kbdim),    ptslm1(kbdim), pws(kbdim), &
             pwsmx(kbdim),   pdensair(kbdim)

  REAL(dp):: paphp1(kbdim,klev+1)

  ! Local scalars:

  INTEGER :: jl,             jt  

  REAL(dp):: zvd2ice,        zvd4ice,        zvd2nof,       zvd4nof,    &
             zvwc2,          zvw02,          zvwc4,         zvw04

  ! Local arrays:

  REAL(dp):: zmaxvdry(kbdim)
  REAL(dp):: zvdrd(kbdim,2)  

  LOGICAL :: loland(kbdim)

  ! Constants:

  REAL(dp), PARAMETER :: zvdcoarse = 0.01_dp, & ! V_dep coarse mode [m/s]
                         zvdaitken = 0.01_dp    ! V_dep aitken mode [m/s]

  !--- Stream and elements:

  REAL(dp),            POINTER :: vdrydep_p(:,:)


  !--- 1) Calculation of the dry deposition velocity: -----------------------

  !--- Coefficients for zvdrd = function of soil moisture:
  !
  zvwc2=(0.8e-2_dp - 0.2e-2_dp)/(1._dp - 0.9_dp)
  zvw02=zvwc2-0.8e-2_dp
  zvwc4=(0.2e-2_dp - 0.025e-2_dp)/(1._dp - 0.9_dp)
  zvw04=zvwc4-0.2e-2_dp
  
  !--- Maximal deposition velocity is maximal vertical grid velocity:

  DO 205 jl=1,kproma

     zmaxvdry(jl)=(paphp1(jl,nlev+1)-paphp1(jl,nlev))/    &
                  (g*pdensair(jl)*time_step_len)

205 END DO


  DO 206 jl=1,kproma

     !--- 1.1) Over sea: 

     IF (.NOT.loland(jl)) THEN
        !         - sea ice -
        !           - melting/not melting seaice-
        IF (ptsi(jl).GE.(ctfreez-0.01_dp)) THEN
           zvd2ice=0.8e-2_dp                             ! SO2
           zvd4ice=0.2e-2_dp                             ! others
        ELSE
           zvd2ice=0.1e-2_dp
           zvd4ice=0.025e-2_dp
        END IF
        zvdrd(jl,1)=(1._dp-pfri(jl))*1.0e-2_dp+pfri(jl)*zvd2ice
        zvdrd(jl,2)=(1._dp-pfri(jl))*0.2e-2_dp+pfri(jl)*zvd4ice
     ELSE

        !--- 1.2) Over land:
        !        - non-forest areas -
        !         -  snow/no snow -
        IF (pcvs(jl).GT.csncri) THEN
           !  - melting/not melting snow -
           IF (smelt(jl,krow).GT.0._dp) THEN
              zvd2nof=0.8e-2_dp
              zvd4nof=0.2e-2_dp
           ELSE
              zvd2nof=0.1e-2_dp
              zvd4nof=0.025e-2_dp
           END IF
        ELSE
           !  -  frozen/not frozen soil -
           IF(ptslm1(jl).LE.tmelt) THEN
              zvd2nof=0.2e-2_dp
              zvd4nof=0.025e-2_dp
           ELSE
              !  - wet/dry -
              !  - completely wet -

              IF (pcvw(jl).GE.0.01_dp .OR. pws(jl).EQ.pwsmx(jl)) THEN
                 zvd2nof=0.8e-2_dp
                 zvd4nof=0.2e-2_dp
              ELSE
                 !   - dry -

                 IF ((pws(jl)/pwsmx(jl)).LT.0.9_dp) THEN
                    zvd2nof=0.2e-2_dp
                    zvd4nof=0.025e-2_dp
                 ELSE
                    !  - partly wet -
                    zvd2nof=zvwc2*(pws(jl)/pwsmx(jl))-zvw02
                    zvd4nof=zvwc4*(pws(jl)/pwsmx(jl))-zvw04
                 END IF
              END IF
           END IF
        END IF
        zvdrd(jl,1)=pforest(jl)*0.8e-2_dp+(1._dp-pforest(jl))*zvd2nof
        zvdrd(jl,2)=pforest(jl)*0.2e-2_dp+(1._dp-pforest(jl))*zvd4nof
     END IF
     !
     !---  Ask zvdrd for maximum:
     !
     zvdrd(jl,1)=MIN(zvdrd(jl,1),zmaxvdry(jl))
     zvdrd(jl,2)=MIN(zvdrd(jl,2),zmaxvdry(jl))

206  END DO

  DO jt=1, ntrac
           
     IF (trlist%ti(jt)%ndrydep==1) THEN

        vdrydep_p => vdrydep(jt)%ptr

        IF(trlist%ti(jt)%basename=='SO2' .OR. (trlist%ti(jt)%basename=='SO4'.AND. trlist%ti(jt)%nphase==GAS) ) THEN

           vdrydep_p(1:kproma,krow)=zvdrd(1:kproma,1)

        ELSE IF(trlist%ti(jt)%nphase==AEROSOLMASS .OR. trlist%ti(jt)%nphase==AEROSOLNUMBER) THEN

           SELECT CASE (trlist%ti(jt)%mode)

           CASE(1)
              vdrydep_p(1:kproma,krow)=0._dp
           CASE(2)
              vdrydep_p(1:kproma,krow)=zvdaitken
           CASE(3)
              vdrydep_p(1:kproma,krow)=zvdrd(jl,2)
           CASE(4)
              vdrydep_p(1:kproma,krow)=zvdcoarse
           CASE(5)
              vdrydep_p(1:kproma,krow)=zvdaitken
           CASE(6)
              vdrydep_p(1:kproma,krow)=zvdrd(jl,2)
           CASE(7)
              vdrydep_p(1:kproma,krow)=zvdcoarse
           END SELECT

        END IF

     END IF

  END DO

          
END SUBROUTINE xt_vdrydep_presc
